@@ -1,23 +1,35 @@
-import mock
+from __future__ import absolute_import
+from __future__ import unicode_literals
 
-from testify import setup, TestCase, run, assert_equal
+import mock
+from testify import assert_equal
+from testify import run
+from testify import setup
+from testify import TestCase
 from testify.assertions import assert_in
+
 from tests.testingutils import autospec_method
 from tron import mcp
 from tron.api import controller
-
-from tron.api.controller import JobCollectionController, ConfigController
-from tron.config import ConfigError, manager, config_parse
-from tron.core import job, service, jobrun, actionrun
+from tron.api.controller import ConfigController
+from tron.api.controller import JobCollectionController
+from tron.config import config_parse
+from tron.config import ConfigError
+from tron.config import manager
+from tron.core import actionrun
+from tron.core import job
+from tron.core import jobrun
 
 
 class JobCollectionControllerTestCase(TestCase):
-
     @setup
     def setup_controller(self):
-        self.collection     = mock.create_autospec(job.JobCollection,
-                                enable=mock.Mock(), disable=mock.Mock())
-        self.controller     = JobCollectionController(self.collection)
+        self.collection = mock.create_autospec(
+            job.JobCollection,
+            enable=mock.Mock(),
+            disable=mock.Mock(),
+        )
+        self.controller = JobCollectionController(self.collection)
 
     def test_handle_command_enabled(self):
         self.controller.handle_command('enableall')
@@ -29,15 +41,18 @@ class JobCollectionControllerTestCase(TestCase):
 
 
 class ActionRunControllerTestCase(TestCase):
-
     @setup
     def setup_controller(self):
-        self.action_run = mock.create_autospec(actionrun.ActionRun,
-            cancel=mock.Mock())
+        self.action_run = mock.create_autospec(
+            actionrun.ActionRun,
+            cancel=mock.Mock(),
+        )
         self.job_run = mock.create_autospec(jobrun.JobRun)
         self.job_run.is_scheduled = False
         self.controller = controller.ActionRunController(
-            self.action_run, self.job_run)
+            self.action_run,
+            self.job_run,
+        )
 
     def test_handle_command_start_failed(self):
         self.job_run.is_scheduled = True
@@ -66,20 +81,25 @@ class ActionRunControllerTestCase(TestCase):
         assert_in("Attempting to kill", result)
 
 
-
 class JobRunControllerTestCase(TestCase):
-
     @setup
     def setup_controller(self):
-        self.job_run = mock.create_autospec(jobrun.JobRun,
-            run_time=mock.Mock(), cancel=mock.Mock())
+        self.job_run = mock.create_autospec(
+            jobrun.JobRun,
+            run_time=mock.Mock(),
+            cancel=mock.Mock(),
+        )
         self.job_scheduler = mock.create_autospec(job.JobScheduler)
         self.controller = controller.JobRunController(
-            self.job_run, self.job_scheduler)
+            self.job_run,
+            self.job_scheduler,
+        )
 
     def test_handle_command_restart(self):
         self.controller.handle_command('restart')
-        self.job_scheduler.manual_start.assert_called_with(self.job_run.run_time)
+        self.job_scheduler.manual_start.assert_called_with(
+            self.job_run.run_time,
+        )
 
     def test_handle_mapped_command(self):
         result = self.controller.handle_command('start')
@@ -94,7 +114,6 @@ class JobRunControllerTestCase(TestCase):
 
 
 class JobControllerTestCase(TestCase):
-
     @setup
     def setup_controller(self):
         self.job_scheduler = mock.create_autospec(job.JobScheduler)
@@ -114,54 +133,7 @@ class JobControllerTestCase(TestCase):
         self.job_scheduler.manual_start.assert_called_with(run_time=run_time)
 
 
-class ServiceInstanceControllerTestCase(TestCase):
-
-    @setup
-    def setup_controller(self):
-        self.instance = mock.create_autospec(service.ServiceInstance)
-        self.controller = controller.ServiceInstanceController(self.instance)
-
-    def test_handle_command_stop(self):
-        response = self.controller.handle_command('stop')
-        self.instance.stop.assert_called_with()
-        assert_in("stopping", response)
-
-    def test_handle_command_stop_failure(self):
-        self.instance.stop.return_value = False
-        response = self.controller.handle_command('stop')
-        self.instance.stop.assert_called_with()
-        assert_in("Failed to stop", response)
-
-    def test_handle_command_start(self):
-        response = self.controller.handle_command('start')
-        self.instance.start.assert_called_with()
-        assert_in("starting", response)
-
-    def test_handle_command_start_failure(self):
-        self.instance.start.return_value = False
-        response = self.controller.handle_command('start')
-        self.instance.start.assert_called_with()
-        assert_in("Failed to start", response)
-
-
-class ServiceControllerTestCase(TestCase):
-
-    @setup
-    def setup_controller(self):
-        self.service = mock.create_autospec(service.Service)
-        self.controller = controller.ServiceController(self.service)
-
-    def test_handle_command_stop(self):
-        self.controller.handle_command('stop')
-        self.service.disable.assert_called_with()
-
-    def test_handle_command_start(self):
-        self.controller.handle_command('start')
-        self.service.enable.assert_called_with()
-
-
 class ConfigControllerTestCase(TestCase):
-
     @setup
     def setup_controller(self):
         self.mcp = mock.create_autospec(mcp.MasterControlProgram)
@@ -172,7 +144,8 @@ class ConfigControllerTestCase(TestCase):
     def test_render_template(self):
         config_content = "asdf asdf"
         container = self.manager.load.return_value = mock.create_autospec(
-            config_parse.ConfigContainer)
+            config_parse.ConfigContainer,
+        )
         container.get_node_names.return_value = ['one', 'two', 'three']
         container.get_master.return_value.command_context = {'zing': 'stars'}
         content = self.controller.render_template(config_content)
@@ -186,7 +159,10 @@ class ConfigControllerTestCase(TestCase):
 
     def test_strip_header_named(self):
         expected = "\nthing"
-        name, content = 'something', self.controller.TEMPLATE + expected
+        name, content = 'something', "{}{}".format(
+            self.controller.TEMPLATE,
+            expected,
+        )
         assert_equal(self.controller.strip_header(name, content), expected)
 
     def test_strip_header_named_missing(self):
@@ -222,8 +198,12 @@ class ConfigControllerTestCase(TestCase):
         resp = self.controller.read_config(name)
         self.controller._get_config_content.assert_called_with(name)
         self.controller.render_template.assert_called_with(
-            self.controller._get_config_content.return_value)
-        assert_equal(resp['config'], self.controller.render_template.return_value)
+            self.controller._get_config_content.return_value,
+        )
+        assert_equal(
+            resp['config'],
+            self.controller.render_template.return_value,
+        )
         assert_equal(resp['hash'], self.manager.get_hash.return_value)
 
     def test_read_config_no_header(self):
@@ -232,7 +212,10 @@ class ConfigControllerTestCase(TestCase):
         autospec_method(self.controller.render_template)
         resp = self.controller.read_config(name, add_header=False)
         assert not self.controller.render_template.called
-        assert_equal(resp['config'], self.controller._get_config_content.return_value)
+        assert_equal(
+            resp['config'],
+            self.controller._get_config_content.return_value,
+        )
 
     def test_update_config(self):
         autospec_method(self.controller.strip_header)
@@ -248,10 +231,14 @@ class ConfigControllerTestCase(TestCase):
     def test_update_config_failure(self):
         autospec_method(self.controller.strip_header)
         striped_content = self.controller.strip_header.return_value
-        name, content, config_hash = None, mock.Mock(), mock.Mock()
+        name, config_hash = None, mock.Mock()
         self.manager.get_hash.return_value = config_hash
         self.manager.write_config.side_effect = ConfigError("It broke")
-        error = self.controller.update_config(name, striped_content, config_hash)
+        error = self.controller.update_config(
+            name,
+            striped_content,
+            config_hash,
+        )
         assert_equal(error, "It broke")
         self.manager.write_config.assert_called_with(name, striped_content)
         assert not self.mcp.reconfigure.call_count
@@ -261,10 +248,38 @@ class ConfigControllerTestCase(TestCase):
         error = self.controller.update_config(name, content, config_hash)
         assert_equal(error, "Configuration has changed. Please try again.")
 
+    def test_delete_config(self):
+        name, content, config_hash = None, "", mock.Mock()
+        self.manager.get_hash.return_value = config_hash
+        assert not self.controller.delete_config(name, content, config_hash)
+        self.manager.delete_config.assert_called_with(name)
+        self.mcp.reconfigure.assert_called_with()
+        self.manager.get_hash.assert_called_with(name)
+
+    def test_delete_config_failure(self):
+        name, content, config_hash = None, "", mock.Mock()
+        self.manager.get_hash.return_value = config_hash
+        self.manager.delete_config.side_effect = Exception("some error")
+        error = self.controller.delete_config(name, content, config_hash)
+        assert error
+        self.manager.delete_config.assert_called_with(name)
+        assert not self.mcp.reconfigure.call_count
+
+    def test_delete_config_hash_mismatch(self):
+        name, content, config_hash = None, "", mock.Mock()
+        error = self.controller.delete_config(name, content, config_hash)
+        assert_equal(error, "Configuration has changed. Please try again.")
+
+    def test_delete_config_content_not_empty(self):
+        name, content, config_hash = None, "content", mock.Mock()
+        error = self.controller.delete_config(name, content, config_hash)
+        assert error
+
     def test_get_namespaces(self):
         result = self.controller.get_namespaces()
         self.manager.get_namespaces.assert_called_with()
         assert_equal(result, self.manager.get_namespaces.return_value)
 
+
 if __name__ == "__main__":
     run()
