@@ -1,24 +1,35 @@
-import datetime
-from testify import TestCase, run, assert_equal, setup
-from tests.testingutils import Turtle
+from __future__ import absolute_import
+from __future__ import unicode_literals
 
-from tron.api.requestargs import get_integer, get_string, get_bool, get_datetime
+import datetime
+
+from testify import assert_equal
+from testify import run
+from testify import setup
+from testify import TestCase
+
+from tests.testingutils import Turtle
+from tron.api.requestargs import get_bool
+from tron.api.requestargs import get_datetime
+from tron.api.requestargs import get_integer
+from tron.api.requestargs import get_string
 
 
 class RequestArgsTestCase(TestCase):
-
     @setup
     def setup_args(self):
         self.args = {
-            'number':   ['123'],
-            'string':   ['astring'],
-            'boolean':  ['1'],
-            'datetime': ['2012-03-14 15:09:26']
+            b'number': [b'123'],
+            b'string': [b'astring'],
+            b'boolean': [b'1'],
+            b'datetime': [b'2012-03-14 15:09:26'],
         }
         self.datetime = datetime.datetime(2012, 3, 14, 15, 9, 26)
         self.request = Turtle(args=self.args)
 
     def _add_arg(self, name, value):
+        name = name.encode()
+        value = value.encode()
         if name not in self.args:
             self.args[name] = []
         self.args[name].append(value)
