@@ -1,18 +1,23 @@
 """
  Assertions for testify.
 """
-from testify.assertions import assert_not_reached, assert_in
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
+from testify.assertions import assert_in
+from testify.assertions import assert_not_reached
 
 
 def assert_raises(expected_exception_class, callable_obj, *args, **kwargs):
     """Returns the exception if the callable raises expected_exception_class"""
     try:
         callable_obj(*args, **kwargs)
-    except expected_exception_class, e:
+    except expected_exception_class as e:
         # we got the expected exception
         return e
-    assert_not_reached("No exception was raised (expected %s)" %
-                       expected_exception_class)
+    assert_not_reached(
+        "No exception was raised (expected %s)" % expected_exception_class,
+    )
 
 
 def assert_length(sequence, expected, msg=None):
@@ -32,4 +37,4 @@ def assert_call(turtle, call_idx, *args, **kwargs):
 def assert_mock_calls(expected, mock_calls):
     """Assert that all expected calls are in the list of mock_calls."""
     for expected_call in expected:
-        assert_in(expected_call, mock_calls)
\ No newline at end of file
+        assert_in(expected_call, mock_calls)
