@@ -1,9 +1,21 @@
-import datetime
-import mock
-from testify import TestCase, setup, assert_raises, assert_equal, run
+from __future__ import absolute_import
+from __future__ import unicode_literals
 
-from tron import command_context, scheduler, node
-from tron.core import job, jobrun, actionrun, serviceinstance
+import datetime
+
+import mock
+from testify import assert_equal
+from testify import assert_raises
+from testify import run
+from testify import setup
+from testify import TestCase
+
+from tron import command_context
+from tron import node
+from tron import scheduler
+from tron.core import actionrun
+from tron.core import job
+from tron.core import jobrun
 from tron.core.jobrun import JobRunCollection
 
 
@@ -20,14 +32,15 @@ class EmptyContextTestCase(TestCase):
 
 
 class BuildFilledContextTestCase(TestCase):
-
     def test_build_filled_context_no_objects(self):
         output = command_context.build_filled_context()
         assert not output.base
         assert not output.next
 
     def test_build_filled_context_single(self):
-        output = command_context.build_filled_context(command_context.JobContext)
+        output = command_context.build_filled_context(
+            command_context.JobContext,
+        )
         assert isinstance(output.base, command_context.JobContext)
         assert not output.next
 
@@ -62,21 +75,24 @@ class SimpleDictContextTestCase(SimpleContextTestCaseBase):
 
 
 class SimpleObjectContextTestCase(SimpleContextTestCaseBase):
-
     @setup
     def build_context(self):
         class Obj(object):
             foo = 'bar'
+
         self.context = command_context.CommandContext(Obj)
 
 
 class ChainedDictContextTestCase(SimpleContextTestCaseBase):
-
     @setup
     def build_context(self):
         self.next_context = command_context.CommandContext(
-                dict(foo='bar', next_foo='next_bar'))
-        self.context = command_context.CommandContext(dict(), self.next_context)
+            dict(foo='bar', next_foo='next_bar'),
+        )
+        self.context = command_context.CommandContext(
+            dict(),
+            self.next_context,
+        )
 
     def test_chain_get(self):
         assert_equal(self.context['next_foo'], 'next_bar')
@@ -86,9 +102,12 @@ class ChainedDictOverrideContextTestCase(SimpleContextTestCaseBase):
     @setup
     def build_context(self):
         self.next_context = command_context.CommandContext(
-                dict(foo='your mom', next_foo='next_bar'))
+            dict(foo='your mom', next_foo='next_bar'),
+        )
         self.context = command_context.CommandContext(
-                dict(foo='bar'), self.next_context)
+            dict(foo='bar'),
+            self.next_context,
+        )
 
     def test_chain_get(self):
         assert_equal(self.context['next_foo'], 'next_bar')
@@ -99,11 +118,13 @@ class ChainedObjectOverrideContextTestCase(SimpleContextTestCaseBase):
     def build_context(self):
         class MyObject(object):
             pass
+
         obj = MyObject()
         obj.foo = 'bar'
 
         self.next_context = command_context.CommandContext(
-                dict(foo='your mom', next_foo='next_bar'))
+            dict(foo='your mom', next_foo='next_bar'),
+        )
         self.context = command_context.CommandContext(obj, self.next_context)
 
     def test_chain_get(self):
@@ -111,14 +132,19 @@ class ChainedObjectOverrideContextTestCase(SimpleContextTestCaseBase):
 
 
 class JobContextTestCase(TestCase):
-
     @setup
     def setup_job(self):
         self.last_success = mock.Mock(run_time=datetime.datetime(2012, 3, 14))
         mock_scheduler = mock.create_autospec(scheduler.ConstantScheduler)
-        run_collection = mock.create_autospec(JobRunCollection,
-                        last_success=self.last_success)
-        self.job = job.Job("jobname", mock_scheduler, run_collection=run_collection)
+        run_collection = mock.create_autospec(
+            JobRunCollection,
+            last_success=self.last_success,
+        )
+        self.job = job.Job(
+            "jobname",
+            mock_scheduler,
+            run_collection=run_collection,
+        )
         self.context = command_context.JobContext(self.job)
 
     def test_name(self):
@@ -126,8 +152,8 @@ class JobContextTestCase(TestCase):
 
     def test__getitem__last_success(self):
         item = self.context["last_success:day-1"]
-        expected = (self.last_success.run_time - datetime.timedelta(days=1)).day
-        assert_equal(item, str(expected))
+        expected_date = self.last_success.run_time - datetime.timedelta(days=1)
+        assert_equal(item, str(expected_date.day))
 
         item = self.context["last_success:shortdate"]
         assert_equal(item, "2012-03-14")
@@ -149,7 +175,6 @@ class JobContextTestCase(TestCase):
 
 
 class JobRunContextTestCase(TestCase):
-
     @setup
     def setup_context(self):
         self.jobrun = mock.create_autospec(jobrun.JobRun, run_time='sometime')
@@ -176,12 +201,14 @@ class JobRunContextTestCase(TestCase):
 
 
 class ActionRunContextTestCase(TestCase):
-
     @setup
     def build_context(self):
         mock_node = mock.create_autospec(node.Node, hostname='something')
-        self.action_run = mock.create_autospec(actionrun.ActionRun,
-            action_name='something', node=mock_node)
+        self.action_run = mock.create_autospec(
+            actionrun.ActionRun,
+            action_name='something',
+            node=mock_node,
+        )
         self.context = command_context.ActionRunContext(self.action_run)
 
     def test_actionname(self):
@@ -191,42 +218,11 @@ class ActionRunContextTestCase(TestCase):
         assert_equal(self.context.node, self.action_run.node.hostname)
 
 
-class ServiceInstanceContextTestCase(TestCase):
-
-    @setup
-    def build_context(self):
-        self.service_instance = mock.create_autospec(
-            serviceinstance.ServiceInstance,
-            instance_number=123,
-            node=mock.Mock(hostname='something'),
-            config=mock.Mock(name='name', pid_file=mock.MagicMock()))
-        self.context = command_context.ServiceInstanceContext(self.service_instance)
-
-    def test_instance_number(self):
-        assert_equal(self.context.instance_number, self.service_instance.instance_number)
-
-    def test_node(self):
-        assert_equal(self.context.node, self.service_instance.node.hostname)
-
-    def test_name(self):
-        assert_equal(self.context.name, self.service_instance.config.name)
-
-    def test_pid_file(self):
-        self.service_instance.parent_context = {'one': 'thing'}
-        self.service_instance.config.pid_file = '%(one)s %(instance_number)s'
-        assert_equal(self.context.pid_file, 'thing 123')
-
-
 class FillerTestCase(TestCase):
-
     @setup
     def setup_filler(self):
         self.filler = command_context.Filler()
 
-    def test_filler_with_service_instance_pid_file(self):
-        context = command_context.ServiceInstanceContext(self.filler)
-        assert_equal(context.pid_file, self.filler)
-
     def test_filler_with_job__getitem__(self):
         context = command_context.JobContext(self.filler)
         todays_date = datetime.date.today().strftime("%Y-%m-%d")
@@ -239,4 +235,4 @@ class FillerTestCase(TestCase):
 
 
 if __name__ == '__main__':
-    run()
\ No newline at end of file
+    run()
