@@ -1,15 +1,25 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 import datetime
+
 import mock
-from testify import TestCase, run, assert_equal, setup
+from testify import assert_equal
+from testify import run
+from testify import setup
+from testify import TestCase
 from testify.assertions import assert_in
+
 from tests.assertions import assert_raises
-from tron.config import config_utils, ConfigError, schema
-from tron.config.config_utils import build_list_of_type_validator, ConfigContext
+from tron.config import config_utils
+from tron.config import ConfigError
+from tron.config import schema
+from tron.config.config_utils import build_list_of_type_validator
+from tron.config.config_utils import ConfigContext
 from tron.config.config_utils import valid_identifier
 
 
 class UniqueNameDictTestCase(TestCase):
-
     @setup
     def setup_dict(self):
         self.msg = "The key %s was there."
@@ -25,7 +35,6 @@ class UniqueNameDictTestCase(TestCase):
 
 
 class ValidatorIdentifierTestCase(TestCase):
-
     def test_valid_identifier_too_long(self):
         assert_raises(ConfigError, valid_identifier, 'a' * 256, mock.Mock())
 
@@ -39,7 +48,6 @@ class ValidatorIdentifierTestCase(TestCase):
 
 
 class BuildListOfTypeValidatorTestCase(TestCase):
-
     @setup
     def setup_validator(self):
         self.item_validator = mock.Mock()
@@ -58,7 +66,6 @@ class BuildListOfTypeValidatorTestCase(TestCase):
 
 
 class BuildEnumValidatorTestCase(TestCase):
-
     @setup
     def setup_enum_validator(self):
         self.enum = dict(a=1, b=2)
@@ -70,12 +77,19 @@ class BuildEnumValidatorTestCase(TestCase):
         assert_equal(self.validator('b', self.context), 'b')
 
     def test_invalid(self):
-        exception = assert_raises(ConfigError, self.validator, 'c', self.context)
-        assert_in('Value at  is not in %s: ' % str(set(self.enum)), str(exception))
+        exception = assert_raises(
+            ConfigError,
+            self.validator,
+            'c',
+            self.context,
+        )
+        assert_in(
+            'Value at  is not in %s: ' % str(set(self.enum)),
+            str(exception),
+        )
 
 
 class ValidTimeTestCase(TestCase):
-
     @setup
     def setup_config(self):
         self.context = config_utils.NullConfigContext
@@ -93,49 +107,78 @@ class ValidTimeTestCase(TestCase):
         assert_equal(time_spec.second, 12)
 
     def test_valid_time_invalid(self):
-        assert_raises(ConfigError, config_utils.valid_time,
-            "14:32:12:34", self.context)
+        assert_raises(
+            ConfigError,
+            config_utils.valid_time,
+            "14:32:12:34",
+            self.context,
+        )
         assert_raises(ConfigError, config_utils.valid_time, None, self.context)
 
 
 class ValidTimeDeltaTestCase(TestCase):
-
     @setup
     def setup_config(self):
         self.context = config_utils.NullConfigContext
 
     def test_valid_time_delta_invalid(self):
-        exception = assert_raises(ConfigError,
-            config_utils.valid_time_delta,'no time', self.context)
+        exception = assert_raises(
+            ConfigError,
+            config_utils.valid_time_delta,
+            'no time',
+            self.context,
+        )
         assert_in('not a valid time delta: no time', str(exception))
 
     def test_valid_time_delta_valid_seconds(self):
         for jitter in [' 82s ', '82 s', '82 sec', '82seconds  ']:
             delta = datetime.timedelta(seconds=82)
-            assert_equal(delta, config_utils.valid_time_delta(jitter, self.context))
+            assert_equal(
+                delta,
+                config_utils.valid_time_delta(
+                    jitter,
+                    self.context,
+                ),
+            )
 
     def test_valid_time_delta_valid_minutes(self):
         for jitter in ['10m', '10 m', '10   min', '  10minutes']:
             delta = datetime.timedelta(seconds=600)
-            assert_equal(delta, config_utils.valid_time_delta(jitter, self.context))
+            assert_equal(
+                delta,
+                config_utils.valid_time_delta(
+                    jitter,
+                    self.context,
+                ),
+            )
 
     def test_valid_time_delta_invalid_unit(self):
         for jitter in ['1 year', '3 mo', '3 months']:
-            assert_raises(ConfigError,
-                config_utils.valid_time_delta, jitter, self.context)
+            assert_raises(
+                ConfigError,
+                config_utils.valid_time_delta,
+                jitter,
+                self.context,
+            )
 
 
 class ConfigContextTestCase(TestCase):
-
     def test_build_config_context(self):
-        path, nodes, namespace = 'path', set([1,2,3]), 'namespace'
+        path, nodes, namespace = 'path', {1, 2, 3}, 'namespace'
+        clusters = {'c1', 'c2'}
         command_context = mock.MagicMock()
         parent_context = config_utils.ConfigContext(
-            path, nodes, command_context, namespace)
+            path,
+            nodes,
+            clusters,
+            command_context,
+            namespace,
+        )
 
         child = parent_context.build_child_context('child')
         assert_equal(child.path, '%s.child' % path)
         assert_equal(child.nodes, nodes)
+        assert_equal(child.clusters, clusters)
         assert_equal(child.namespace, namespace)
         assert_equal(child.command_context, command_context)
         assert not child.partial
@@ -144,22 +187,27 @@ class ConfigContextTestCase(TestCase):
 StubConfigObject = schema.config_object_factory(
     'StubConfigObject',
     ['req1', 'req2'],
-    ['opt1', 'opt2']
+    ['opt1', 'opt2'],
 )
 
+
 class StubValidator(config_utils.Validator):
     config_class = StubConfigObject
 
-class ValidatorTestCase(TestCase):
 
+class ValidatorTestCase(TestCase):
     @setup
     def setup_validator(self):
         self.validator = StubValidator()
 
     def test_validate_with_none(self):
         expected_msg = "A StubObject is required"
-        exception = assert_raises(ConfigError,
-            self.validator.validate, None, config_utils.NullConfigContext)
+        exception = assert_raises(
+            ConfigError,
+            self.validator.validate,
+            None,
+            config_utils.NullConfigContext,
+        )
         assert_in(expected_msg, str(exception))
 
     def test_validate_optional_with_none(self):
@@ -167,5 +215,6 @@ class ValidatorTestCase(TestCase):
         config = self.validator.validate(None, config_utils.NullConfigContext)
         assert_equal(config, None)
 
+
 if __name__ == "__main__":
-    run()
\ No newline at end of file
+    run()
