@@ -1,49 +1,80 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 import datetime
 import shutil
 import tempfile
-import mock
 
-from testify import run, setup, TestCase, assert_equal, turtle, teardown
-from testify.assertions import assert_raises, assert_in
+import mock
+import six
+from testify import assert_equal
+from testify import run
+from testify import setup
+from testify import teardown
+from testify import TestCase
+from testify import turtle
+from testify.assertions import assert_in
+from testify.assertions import assert_raises
+
 from tests import testingutils
 from tests.assertions import assert_length
-from tests.testingutils import Turtle, autospec_method
-
-from tron import node, actioncommand
-from tron.core import jobrun, actiongraph
-from tron.core.actionrun import ActionCommand, ActionRun
-from tron.core.actionrun import ActionRunCollection, ActionRunFactory
+from tests.testingutils import autospec_method
+from tests.testingutils import Turtle
+from tron import actioncommand
+from tron import node
+from tron.config.schema import ExecutorTypes
+from tron.core import actiongraph
+from tron.core import jobrun
+from tron.core.actionrun import ActionCommand
+from tron.core.actionrun import ActionRun
+from tron.core.actionrun import ActionRunCollection
+from tron.core.actionrun import ActionRunFactory
+from tron.core.actionrun import PaaSTAActionRun
+from tron.core.actionrun import SSHActionRun
 from tron.serialize import filehandler
 
 
 class ActionRunFactoryTestCase(TestCase):
-
     @setup
     def setup_action_runs(self):
-        self.run_time = datetime.datetime(2012, 3, 14, 15, 9 ,26)
+        self.run_time = datetime.datetime(2012, 3, 14, 15, 9, 26)
         actions = [Turtle(name='act1'), Turtle(name='act2')]
         self.action_graph = actiongraph.ActionGraph(
-                actions, dict((a.name, a) for a in actions))
+            actions,
+            {a.name: a
+             for a in actions},
+        )
 
         mock_node = mock.create_autospec(node.Node)
-        self.job_run = jobrun.JobRun('jobname', 7, self.run_time, mock_node,
-                action_graph=self.action_graph)
+        self.job_run = jobrun.JobRun(
+            'jobname',
+            7,
+            self.run_time,
+            mock_node,
+            action_graph=self.action_graph,
+            service='foo',
+            deploy_group='test',
+        )
 
         self.action_state_data = {
-            'job_run_id':       'job_run_id',
-            'action_name':      'act1',
-            'state':            'succeeded',
-            'run_time':         'the_run_time',
-            'start_time':       None,
-            'end_time':         None,
-            'command':          'do action1',
-            'node_name':        'anode'
+            'job_run_id': 'job_run_id',
+            'action_name': 'act1',
+            'state': 'succeeded',
+            'run_time': 'the_run_time',
+            'start_time': None,
+            'end_time': None,
+            'command': 'do action1',
+            'node_name': 'anode',
         }
-        self.action_runner = mock.create_autospec(actioncommand.SubprocessActionRunnerFactory)
+        self.action_runner = mock.create_autospec(
+            actioncommand.SubprocessActionRunnerFactory,
+        )
 
     def test_build_action_run_collection(self):
         collection = ActionRunFactory.build_action_run_collection(
-            self.job_run, self.action_runner)
+            self.job_run,
+            self.action_runner,
+        )
         assert_equal(collection.action_graph, self.action_graph)
         assert_in('act1', collection.run_map)
         assert_in('act2', collection.run_map)
@@ -53,17 +84,25 @@ class ActionRunFactoryTestCase(TestCase):
     def test_action_run_collection_from_state(self):
         state_data = [self.action_state_data]
         cleanup_action_state_data = {
-            'job_run_id':       'job_run_id',
-            'action_name':      'cleanup',
-            'state':            'succeeded',
-            'run_time':         self.run_time,
-            'start_time':       None,
-            'end_time':         None,
-            'command':          'do cleanup',
-            'node_name':        'anode'
+            'job_run_id': 'job_run_id',
+            'action_name': 'cleanup',
+            'state': 'succeeded',
+            'run_time': self.run_time,
+            'start_time': None,
+            'end_time': None,
+            'command': 'do cleanup',
+            'node_name': 'anode',
+            'action_runner':
+                {
+                    'status_path': '/tmp/foo',
+                    'exec_path': '/bin/foo'
+                }
         }
         collection = ActionRunFactory.action_run_collection_from_state(
-            self.job_run, state_data, cleanup_action_state_data)
+            self.job_run,
+            state_data,
+            cleanup_action_state_data,
+        )
 
         assert_equal(collection.action_graph, self.action_graph)
         assert_length(collection.run_map, 2)
@@ -72,9 +111,16 @@ class ActionRunFactoryTestCase(TestCase):
 
     def test_build_run_for_action(self):
         action = Turtle(
-            name='theaction', node_pool=None, is_cleanup=False, command="doit")
+            name='theaction',
+            node_pool=None,
+            is_cleanup=False,
+            command="doit",
+        )
         action_run = ActionRunFactory.build_run_for_action(
-            self.job_run, action, self.action_runner)
+            self.job_run,
+            action,
+            self.action_runner,
+        )
 
         assert_equal(action_run.job_run_id, self.job_run.id)
         assert_equal(action_run.node, self.job_run.node)
@@ -85,7 +131,10 @@ class ActionRunFactoryTestCase(TestCase):
     def test_build_run_for_action_with_node(self):
         action = Turtle(name='theaction', is_cleanup=True, command="doit")
         action_run = ActionRunFactory.build_run_for_action(
-            self.job_run, action, self.action_runner)
+            self.job_run,
+            action,
+            self.action_runner,
+        )
 
         assert_equal(action_run.job_run_id, self.job_run.id)
         assert_equal(action_run.node, action.node_pool.next.returns[0])
@@ -93,35 +142,116 @@ class ActionRunFactoryTestCase(TestCase):
         assert_equal(action_run.action_name, action.name)
         assert_equal(action_run.command, action.command)
 
-    def test_action_run_from_state(self):
+    def test_build_run_for_ssh_action(self):
+        action = Turtle(
+            name='theaction',
+            command="doit",
+            executor=ExecutorTypes.ssh,
+        )
+        action_run = ActionRunFactory.build_run_for_action(
+            self.job_run,
+            action,
+            self.action_runner,
+        )
+        assert_equal(action_run.__class__, SSHActionRun)
+
+    def test_build_run_for_paasta_action_default_service(self):
+        action = Turtle(
+            name='theaction',
+            command="doit",
+            executor=ExecutorTypes.paasta,
+            cluster='prod',
+            pool='default',
+            cpus=10,
+            mem=500,
+            service=None,
+            deploy_group=None,
+        )
+        action_run = ActionRunFactory.build_run_for_action(
+            self.job_run,
+            action,
+            self.action_runner,
+        )
+        assert_equal(action_run.__class__, PaaSTAActionRun)
+        assert_equal(action_run.cluster, action.cluster)
+        assert_equal(action_run.pool, action.pool)
+        assert_equal(action_run.cpus, action.cpus)
+        assert_equal(action_run.mem, action.mem)
+        assert_equal(action_run.service, self.job_run.service)
+        assert_equal(action_run.deploy_group, self.job_run.deploy_group)
+
+    def test_build_run_for_paasta_action_overrides_service(self):
+        action = Turtle(
+            name='theaction',
+            command="doit",
+            executor=ExecutorTypes.paasta,
+            service='bar',
+            deploy_group='dev',
+        )
+        action_run = ActionRunFactory.build_run_for_action(
+            self.job_run,
+            action,
+            self.action_runner,
+        )
+        assert_equal(action_run.__class__, PaaSTAActionRun)
+        assert_equal(action_run.service, action.service)
+        assert_equal(action_run.deploy_group, action.deploy_group)
+
+    def test_action_run_from_state_default(self):
         state_data = self.action_state_data
         action_run = ActionRunFactory.action_run_from_state(
-                self.job_run, state_data)
+            self.job_run,
+            state_data,
+        )
 
         assert_equal(action_run.job_run_id, state_data['job_run_id'])
         assert not action_run.is_cleanup
+        assert_equal(action_run.__class__, SSHActionRun)
+
+    def test_action_run_from_state_paasta(self):
+        state_data = self.action_state_data
+        state_data['executor'] = ExecutorTypes.paasta
+        state_data['cluster'] = 'cluster-one'
+        state_data['pool'] = 'private'
+        state_data['cpus'] = 2
+        state_data['mem'] = 200
+        state_data['service'] = 'baz'
+        state_data['deploy_group'] = 'test'
+        action_run = ActionRunFactory.action_run_from_state(
+            self.job_run,
+            state_data,
+        )
+
+        assert_equal(action_run.job_run_id, state_data['job_run_id'])
+        assert_equal(action_run.cluster, state_data['cluster'])
+        assert_equal(action_run.pool, state_data['pool'])
+        assert_equal(action_run.cpus, state_data['cpus'])
+        assert_equal(action_run.mem, state_data['mem'])
+        assert_equal(action_run.service, state_data['service'])
+        assert_equal(action_run.deploy_group, state_data['deploy_group'])
+        assert not action_run.is_cleanup
+        assert_equal(action_run.__class__, PaaSTAActionRun)
 
 
 class ActionRunTestCase(TestCase):
-
     @setup
     def setup_action_run(self):
         self.output_path = filehandler.OutputPath(tempfile.mkdtemp())
-        self.action_runner = mock.create_autospec(
-            actioncommand.NoActionRunnerFactory)
+        self.action_runner = actioncommand.NoActionRunnerFactory()
         self.command = "do command %(actionname)s"
         self.rendered_command = "do command action_name"
         self.action_run = ActionRun(
-                "id",
-                "action_name",
-                mock.create_autospec(node.Node),
-                self.command,
-                output_path=self.output_path,
-                action_runner=self.action_runner)
-
-    @teardown
-    def teardown_action_run(self):
-        shutil.rmtree(self.output_path.base, ignore_errors=True)
+            "id",
+            "action_name",
+            mock.create_autospec(node.Node),
+            self.command,
+            output_path=self.output_path,
+            action_runner=self.action_runner,
+        )
+        # These should be implemented in subclasses, we don't care here
+        self.action_run.submit_command = mock.Mock()
+        self.action_run.stop = mock.Mock()
+        self.action_run.kill = mock.Mock()
 
     def test_init_state(self):
         assert_equal(self.action_run.state, ActionRun.STATE_SCHEDULED)
@@ -129,6 +259,7 @@ class ActionRunTestCase(TestCase):
     def test_start(self):
         self.action_run.machine.transition('ready')
         assert self.action_run.start()
+        assert_equal(self.action_run.submit_command.call_count, 1)
         assert self.action_run.is_starting
         assert self.action_run.start_time
 
@@ -136,83 +267,14 @@ class ActionRunTestCase(TestCase):
         self.action_run.fail()
         assert not self.action_run.start()
 
-    def test_start_invalid_command(self):
+    @mock.patch('tron.core.actionrun.log')
+    def test_start_invalid_command(self, _log):
         self.action_run.bare_command = "%(notfound)s"
         self.action_run.machine.transition('ready')
         assert not self.action_run.start()
         assert self.action_run.is_failed
         assert_equal(self.action_run.exit_status, -1)
 
-    def test_start_node_error(self):
-        def raise_error(c):
-            raise node.Error("The error")
-        self.action_run.node = turtle.Turtle(submit_command=raise_error)
-        self.action_run.machine.transition('ready')
-        assert not self.action_run.start()
-        assert_equal(self.action_run.exit_status, -2)
-        assert self.action_run.is_failed
-
-    @mock.patch('tron.core.actionrun.filehandler', autospec=True)
-    def test_build_action_command(self, mock_filehandler):
-        autospec_method(self.action_run.watch)
-        serializer = mock_filehandler.OutputStreamSerializer.return_value
-        action_command = self.action_run.build_action_command()
-        assert_equal(action_command, self.action_run.action_command)
-        assert_equal(action_command, self.action_runner.create.return_value)
-        self.action_runner.create.assert_called_with(
-            self.action_run.id, self.action_run.command, serializer)
-        mock_filehandler.OutputStreamSerializer.assert_called_with(
-            self.action_run.output_path)
-        self.action_run.watch.assert_called_with(action_command)
-
-    def test_handler_running(self):
-        self.action_run.build_action_command()
-        self.action_run.machine.transition('start')
-        assert self.action_run.handler(
-                self.action_run.action_command, ActionCommand.RUNNING)
-        assert self.action_run.is_running
-
-    def test_handler_failstart(self):
-        self.action_run.build_action_command()
-        assert self.action_run.handler(
-                self.action_run.action_command, ActionCommand.FAILSTART)
-        assert self.action_run.is_failed
-
-    def test_handler_exiting_fail(self):
-        self.action_run.build_action_command()
-        self.action_run.action_command.exit_status = -1
-        self.action_run.machine.transition('start')
-        assert self.action_run.handler(
-            self.action_run.action_command, ActionCommand.EXITING)
-        assert self.action_run.is_failed
-        assert_equal(self.action_run.exit_status, -1)
-
-    def test_handler_exiting_success(self):
-        self.action_run.build_action_command()
-        self.action_run.action_command.exit_status = 0
-        self.action_run.machine.transition('start')
-        self.action_run.machine.transition('started')
-        assert self.action_run.handler(
-            self.action_run.action_command, ActionCommand.EXITING)
-        assert self.action_run.is_succeeded
-        assert_equal(self.action_run.exit_status, 0)
-
-    def test_handler_exiting_failunknown(self):
-        self.action_run.action_command = mock.create_autospec(
-            actioncommand.ActionCommand, exit_status=None)
-        self.action_run.machine.transition('start')
-        self.action_run.machine.transition('started')
-        assert self.action_run.handler(
-            self.action_run.action_command, ActionCommand.EXITING)
-        assert self.action_run.is_unknown
-        assert_equal(self.action_run.exit_status, None)
-
-    def test_handler_unhandled(self):
-        self.action_run.build_action_command()
-        assert self.action_run.handler(
-            self.action_run.action_command, ActionCommand.PENDING) is None
-        assert self.action_run.is_scheduled
-
     def test_success(self):
         assert self.action_run.ready()
         self.action_run.machine.transition('start')
@@ -278,7 +340,8 @@ class ActionRunTestCase(TestCase):
         self.action_run.bare_command = "new command"
         assert_equal(self.action_run.command, self.rendered_command)
 
-    def test_command_failed_render(self):
+    @mock.patch('tron.core.actionrun.log')
+    def test_command_failed_render(self, _log):
         self.action_run.bare_command = "%(this_is_missing)s"
         assert_equal(self.action_run.command, ActionRun.FAILED_RENDER)
 
@@ -307,8 +370,155 @@ class ActionRunTestCase(TestCase):
         assert self.action_run.is_cancelled
 
     def test__getattr__missing_attribute(self):
-        assert_raises(AttributeError,
-            self.action_run.__getattr__, 'is_not_a_real_state')
+        assert_raises(
+            AttributeError,
+            self.action_run.__getattr__,
+            'is_not_a_real_state',
+        )
+
+
+class SSHActionRunTestCase(TestCase):
+    @setup
+    def setup_action_run(self):
+        self.output_path = filehandler.OutputPath(tempfile.mkdtemp())
+        self.action_runner = mock.create_autospec(
+            actioncommand.NoActionRunnerFactory,
+        )
+        self.command = "do command %(actionname)s"
+        self.action_run = SSHActionRun(
+            "id",
+            "action_name",
+            mock.create_autospec(node.Node),
+            self.command,
+            output_path=self.output_path,
+            action_runner=self.action_runner,
+        )
+
+    @teardown
+    def teardown_action_run(self):
+        shutil.rmtree(self.output_path.base, ignore_errors=True)
+
+    def test_start_node_error(self):
+        def raise_error(c):
+            raise node.Error("The error")
+
+        self.action_run.node = turtle.Turtle(submit_command=raise_error)
+        self.action_run.machine.transition('ready')
+        assert not self.action_run.start()
+        assert_equal(self.action_run.exit_status, -2)
+        assert self.action_run.is_failed
+
+    @mock.patch('tron.core.actionrun.filehandler', autospec=True)
+    def test_build_action_command(self, mock_filehandler):
+        autospec_method(self.action_run.watch)
+        serializer = mock_filehandler.OutputStreamSerializer.return_value
+        action_command = self.action_run.build_action_command()
+        assert_equal(action_command, self.action_run.action_command)
+        assert_equal(action_command, self.action_runner.create.return_value)
+        self.action_runner.create.assert_called_with(
+            self.action_run.id,
+            self.action_run.command,
+            serializer,
+        )
+        mock_filehandler.OutputStreamSerializer.assert_called_with(
+            self.action_run.output_path,
+        )
+        self.action_run.watch.assert_called_with(action_command)
+
+    def test_auto_retry(self):
+        self.action_run.retries_remaining = 2
+        self.action_run.exit_statuses = []
+        self.action_run.build_action_command()
+        self.action_run.action_command.exit_status = -1
+        self.action_run.machine.transition('start')
+
+        self.action_run.fail(-1)
+        assert self.action_run.retries_remaining == 1
+        assert not self.action_run.is_failed
+
+        self.action_run.fail(-1)
+        assert self.action_run.retries_remaining == 0
+        assert not self.action_run.is_failed
+
+        self.action_run.fail(-1)
+        assert self.action_run.retries_remaining == 0
+        assert self.action_run.is_failed
+
+        assert_equal(self.action_run.exit_statuses, [-1, -1])
+
+    def test_manual_retry(self):
+        self.action_run.retries_remaining = None
+        self.action_run.exit_statuses = []
+        self.action_run.build_action_command()
+        self.action_run.action_command.exit_status = -1
+        self.action_run.machine.transition('start')
+        self.action_run.fail(-1)
+        self.action_run.retry()
+        self.action_run.is_running
+        assert_equal(self.action_run.exit_statuses, [-1])
+        assert_equal(self.action_run.retries_remaining, 0)
+
+    def test_handler_running(self):
+        self.action_run.build_action_command()
+        self.action_run.machine.transition('start')
+        assert self.action_run.handler(
+            self.action_run.action_command,
+            ActionCommand.RUNNING,
+        )
+        assert self.action_run.is_running
+
+    def test_handler_failstart(self):
+        self.action_run.build_action_command()
+        assert self.action_run.handler(
+            self.action_run.action_command,
+            ActionCommand.FAILSTART,
+        )
+        assert self.action_run.is_failed
+
+    def test_handler_exiting_fail(self):
+        self.action_run.build_action_command()
+        self.action_run.action_command.exit_status = -1
+        self.action_run.machine.transition('start')
+        assert self.action_run.handler(
+            self.action_run.action_command,
+            ActionCommand.EXITING,
+        )
+        assert self.action_run.is_failed
+        assert_equal(self.action_run.exit_status, -1)
+
+    def test_handler_exiting_success(self):
+        self.action_run.build_action_command()
+        self.action_run.action_command.exit_status = 0
+        self.action_run.machine.transition('start')
+        self.action_run.machine.transition('started')
+        assert self.action_run.handler(
+            self.action_run.action_command,
+            ActionCommand.EXITING,
+        )
+        assert self.action_run.is_succeeded
+        assert_equal(self.action_run.exit_status, 0)
+
+    def test_handler_exiting_failunknown(self):
+        self.action_run.action_command = mock.create_autospec(
+            actioncommand.ActionCommand,
+            exit_status=None,
+        )
+        self.action_run.machine.transition('start')
+        self.action_run.machine.transition('started')
+        assert self.action_run.handler(
+            self.action_run.action_command,
+            ActionCommand.EXITING,
+        )
+        assert self.action_run.is_unknown
+        assert_equal(self.action_run.exit_status, None)
+
+    def test_handler_unhandled(self):
+        self.action_run.build_action_command()
+        assert self.action_run.handler(
+            self.action_run.action_command,
+            ActionCommand.PENDING,
+        ) is None
+        assert self.action_run.is_scheduled
 
 
 class ActionRunStateRestoreTestCase(testingutils.MockTimeTestCase):
@@ -320,22 +530,26 @@ class ActionRunStateRestoreTestCase(testingutils.MockTimeTestCase):
         self.parent_context = {}
         self.output_path = ['one', 'two']
         self.state_data = {
-            'job_run_id':       'theid',
-            'action_name':      'theaction',
-            'node_name':        'anode',
-            'command':          'do things',
-            'start_time':       'start_time',
-            'end_time':         'end_time',
-            'state':            'succeeded'
+            'job_run_id': 'theid',
+            'action_name': 'theaction',
+            'node_name': 'anode',
+            'command': 'do things',
+            'start_time': 'start_time',
+            'end_time': 'end_time',
+            'state': 'succeeded',
         }
         self.run_node = Turtle()
 
     def test_from_state(self):
         state_data = self.state_data
-        action_run = ActionRun.from_state(state_data, self.parent_context,
-                list(self.output_path), self.run_node)
+        action_run = ActionRun.from_state(
+            state_data,
+            self.parent_context,
+            list(self.output_path),
+            self.run_node,
+        )
 
-        for key, value in self.state_data.iteritems():
+        for key, value in six.iteritems(self.state_data):
             if key in ['state', 'node_name']:
                 continue
             assert_equal(getattr(action_run, key), value)
@@ -346,76 +560,112 @@ class ActionRunStateRestoreTestCase(testingutils.MockTimeTestCase):
 
     def test_from_state_running(self):
         self.state_data['state'] = 'running'
-        action_run = ActionRun.from_state(self.state_data,
-                self.parent_context, self.output_path, self.run_node)
+        action_run = ActionRun.from_state(
+            self.state_data,
+            self.parent_context,
+            self.output_path,
+            self.run_node,
+        )
         assert action_run.is_unknown
         assert_equal(action_run.exit_status, 0)
         assert_equal(action_run.end_time, self.now)
 
     def test_from_state_queued(self):
         self.state_data['state'] = 'queued'
-        action_run = ActionRun.from_state(self.state_data, self.parent_context,
-                self.output_path, self.run_node)
+        action_run = ActionRun.from_state(
+            self.state_data,
+            self.parent_context,
+            self.output_path,
+            self.run_node,
+        )
         assert action_run.is_queued
 
     def test_from_state_no_node_name(self):
         del self.state_data['node_name']
-        action_run = ActionRun.from_state(self.state_data,
-                self.parent_context, self.output_path, self.run_node)
+        action_run = ActionRun.from_state(
+            self.state_data,
+            self.parent_context,
+            self.output_path,
+            self.run_node,
+        )
         assert_equal(action_run.node, self.run_node)
 
     @mock.patch('tron.core.actionrun.node.NodePoolRepository')
     def test_from_state_with_node_exists(self, mock_store):
-        ActionRun.from_state(self.state_data,
-                self.parent_context, self.output_path, self.run_node)
+        ActionRun.from_state(
+            self.state_data,
+            self.parent_context,
+            self.output_path,
+            self.run_node,
+        )
         mock_store.get_instance().get_node.assert_called_with(
-            self.state_data['node_name'], self.run_node)
+            self.state_data['node_name'],
+            self.run_node,
+        )
 
     def test_from_state_before_rendered_command(self):
         self.state_data['command'] = 'do things %(actionname)s'
         self.state_data['rendered_command'] = None
-        action_run = ActionRun.from_state(self.state_data,
-                self.parent_context, self.output_path, self.run_node)
+        action_run = ActionRun.from_state(
+            self.state_data,
+            self.parent_context,
+            self.output_path,
+            self.run_node,
+        )
         assert_equal(action_run.bare_command, self.state_data['command'])
         assert not action_run.rendered_command
 
     def test_from_state_old_state(self):
         self.state_data['command'] = 'do things %(actionname)s'
-        action_run = ActionRun.from_state(self.state_data,
-                self.parent_context, self.output_path, self.run_node)
+        action_run = ActionRun.from_state(
+            self.state_data,
+            self.parent_context,
+            self.output_path,
+            self.run_node,
+        )
         assert_equal(action_run.bare_command, self.state_data['command'])
         assert not action_run.rendered_command
 
     def test_from_state_after_rendered_command(self):
         self.state_data['command'] = 'do things theaction'
         self.state_data['rendered_command'] = self.state_data['command']
-        action_run = ActionRun.from_state(self.state_data,
-                self.parent_context, self.output_path, self.run_node)
+        action_run = ActionRun.from_state(
+            self.state_data,
+            self.parent_context,
+            self.output_path,
+            self.run_node,
+        )
         assert_equal(action_run.bare_command, self.state_data['command'])
         assert_equal(action_run.rendered_command, self.state_data['command'])
 
 
 class ActionRunCollectionTestCase(TestCase):
-
     def _build_run(self, name):
         mock_node = mock.create_autospec(node.Node)
-        return ActionRun("id", name, mock_node, self.command,
-            output_path=self.output_path)
+        return ActionRun(
+            "id",
+            name,
+            mock_node,
+            self.command,
+            output_path=self.output_path,
+        )
 
     @setup
     def setup_runs(self):
         action_names = ['action_name', 'second_name', 'cleanup']
 
         action_graph = [
-            mock.Mock(name=name, required_actions=[])
-            for name in action_names
+            mock.Mock(name=name, required_actions=[]) for name in action_names
         ]
         self.action_graph = actiongraph.ActionGraph(
-            action_graph, dict((a.name, a) for a in action_graph))
+            action_graph,
+            {a.name: a
+             for a in action_graph},
+        )
         self.output_path = filehandler.OutputPath(tempfile.mkdtemp())
         self.command = "do command"
         self.action_runs = [self._build_run(name) for name in action_names]
-        self.run_map = dict((a.action_name, a) for a in self.action_runs)
+        self.run_map = {a.action_name: a for a in self.action_runs}
         self.run_map['cleanup'].is_cleanup = True
         self.collection = ActionRunCollection(self.action_graph, self.run_map)
 
@@ -492,7 +742,10 @@ class ActionRunCollectionTestCase(TestCase):
         self.run_map['action_name'].machine.state = ActionRun.STATE_FAILED
         self.run_map['second_name'].machine.state = ActionRun.STATE_QUEUED
         autospec_method(self.collection._is_run_blocked)
-        blocked_second_action_run = lambda ar: ar == self.run_map['second_name']
+
+        def blocked_second_action_run(ar, ):
+            return ar == self.run_map['second_name']
+
         self.collection._is_run_blocked.side_effect = blocked_second_action_run
         assert self.collection.is_done
         assert self.collection.is_failed
@@ -528,7 +781,7 @@ class ActionRunCollectionTestCase(TestCase):
             "ActionRunCollection",
             "second_name(scheduled:blocked)",
             "action_name(scheduled:blocked)",
-            "cleanup(scheduled)"
+            "cleanup(scheduled)",
         ]
         for expectation in expected:
             assert_in(expectation, str(self.collection))
@@ -553,30 +806,33 @@ class ActionRunCollectionTestCase(TestCase):
 
 
 class ActionRunCollectionIsRunBlockedTestCase(TestCase):
-
     def _build_run(self, name):
         mock_node = mock.create_autospec(node.Node)
-        return ActionRun("id", name, mock_node, self.command,
-            output_path=self.output_path)
+        return ActionRun(
+            "id",
+            name,
+            mock_node,
+            self.command,
+            output_path=self.output_path,
+        )
 
     @setup
     def setup_collection(self):
         action_names = ['action_name', 'second_name', 'cleanup']
 
         action_graph = [
-            Turtle(name=name, required_actions=[])
-            for name in action_names
+            Turtle(name=name, required_actions=[]) for name in action_names
         ]
         self.second_act = second_act = action_graph.pop(1)
         second_act.required_actions.append(action_graph[0])
-        action_map = dict((a.name, a) for a in action_graph)
+        action_map = {a.name: a for a in action_graph}
         action_map['second_name'] = second_act
         self.action_graph = actiongraph.ActionGraph(action_graph, action_map)
 
         self.output_path = filehandler.OutputPath(tempfile.mkdtemp())
         self.command = "do command"
         self.action_runs = [self._build_run(name) for name in action_names]
-        self.run_map = dict((a.action_name, a) for a in self.action_runs)
+        self.run_map = {a.action_name: a for a in self.action_runs}
         self.run_map['cleanup'].is_cleanup = True
         self.collection = ActionRunCollection(self.action_graph, self.run_map)
 
@@ -599,7 +855,10 @@ class ActionRunCollectionIsRunBlockedTestCase(TestCase):
         assert not self.collection._is_run_blocked(self.run_map['second_name'])
 
     def test_is_run_blocked_required_actions_blocked(self):
-        third_act = Turtle(name='third_act', required_actions=[self.second_act])
+        third_act = Turtle(
+            name='third_act',
+            required_actions=[self.second_act],
+        )
         self.action_graph.action_map['third_act'] = third_act
         self.run_map['third_act'] = self._build_run('third_act')
 
@@ -622,6 +881,10 @@ class ActionRunCollectionIsRunBlockedTestCase(TestCase):
         self.run_map['action_name'].machine.state = ActionRun.STATE_FAILED
         assert self.collection._is_run_blocked(self.run_map['second_name'])
 
+    def test_is_run_blocked_required_actions_missing(self):
+        del self.run_map['action_name']
+        assert not self.collection._is_run_blocked(self.run_map['second_name'])
+
 
 if __name__ == "__main__":
     run()
