@@ -1,20 +1,36 @@
-import datetime
-import mock
+from __future__ import absolute_import
+from __future__ import unicode_literals
 
-from testify import setup, teardown, TestCase, run, assert_equal
+import collections
+import datetime
+
+import mock
+import six
+from testify import assert_equal
+from testify import run
+from testify import setup
 from testify import setup_teardown
+from testify import teardown
+from testify import TestCase
 from testify.assertions import assert_not_equal
+
 from tests import mocks
-from tests.assertions import assert_length, assert_call, assert_mock_calls
-from tests.testingutils import Turtle, autospec_method
 from tests import testingutils
-from tron import node, event, actioncommand
-from tron.core import job, jobrun
+from tests.assertions import assert_call
+from tests.assertions import assert_length
+from tests.assertions import assert_mock_calls
+from tests.testingutils import autospec_method
+from tests.testingutils import Turtle
+from tron import actioncommand
+from tron import event
+from tron import node
+from tron import scheduler
+from tron.core import job
+from tron.core import jobrun
 from tron.core.actionrun import ActionRun
 
 
 class JobTestCase(TestCase):
-
     @setup_teardown
     def setup_job(self):
         action_graph = mock.Mock(names=lambda: ['one', 'two'])
@@ -22,13 +38,19 @@ class JobTestCase(TestCase):
         run_collection = Turtle()
         self.nodes = mock.create_autospec(node.NodePool)
         self.action_runner = mock.create_autospec(
-            actioncommand.SubprocessActionRunnerFactory)
+            actioncommand.SubprocessActionRunnerFactory,
+        )
 
         patcher = mock.patch('tron.core.job.node.NodePoolRepository')
         with patcher as self.mock_node_repo:
-            self.job = job.Job("jobname", scheduler,
-                    run_collection=run_collection, action_graph=action_graph,
-                    node_pool=self.nodes)
+            self.job = job.Job(
+                "jobname",
+                scheduler,
+                run_collection=run_collection,
+                action_graph=action_graph,
+                node_pool=self.nodes,
+                action_runner=actioncommand.NoActionRunnerFactory
+            )
             autospec_method(self.job.notify)
             autospec_method(self.job.watch)
             self.job.event = mock.create_autospec(event.EventRecorder)
@@ -39,32 +61,57 @@ class JobTestCase(TestCase):
 
     @mock.patch('tron.core.job.event', autospec=True)
     def test_from_config(self, _mock_event):
-        action = mock.Mock(name='first', command='doit', node=None, requires=[])
+        action = mock.Mock(
+            name='first',
+            command='doit',
+            node=None,
+            requires=[],
+        )
         job_config = mock.Mock(
             name='ajob',
             node='thenodepool',
+            monitoring={
+                "team": "foo",
+                "page": True,
+            },
             all_nodes=False,
             queueing=True,
             enabled=True,
             run_limit=20,
             actions={action.name: action},
-            cleanup_action=None)
+            cleanup_action=None,
+            service='foo',
+            deploy_group='test',
+        )
         scheduler = 'scheduler_token'
         parent_context = 'parent_context_token'
         output_path = ["base_path"]
         new_job = job.Job.from_config(
-            job_config, scheduler, parent_context, output_path, self.action_runner)
+            job_config,
+            scheduler,
+            parent_context,
+            output_path,
+            self.action_runner,
+        )
 
         assert_equal(new_job.scheduler, scheduler)
         assert_equal(new_job.context.next, parent_context)
         self.mock_node_repo.get_instance().get_by_name.assert_called_with(
-            job_config.node)
+            job_config.node,
+        )
         assert_equal(new_job.enabled, True)
+        assert_equal(new_job.get_monitoring()["team"], "foo")
+        assert_equal(new_job.get_service(), 'foo')
+        assert_equal(new_job.get_deploy_group(), 'test')
         assert new_job.action_graph
 
     def test_update_from_job(self):
         action_runner = mock.Mock()
-        other_job = job.Job('otherjob', 'scheduler', action_runner=action_runner)
+        other_job = job.Job(
+            'otherjob',
+            'scheduler',
+            action_runner=action_runner,
+        )
         self.job.update_from_job(other_job)
         assert_equal(self.job.name, 'otherjob')
         assert_equal(self.job.scheduler, 'scheduler')
@@ -95,18 +142,21 @@ class JobTestCase(TestCase):
         assert_equal(state_data['runs'], self.job.runs.state_data)
         assert state_data['enabled']
 
-    def test_restore_state(self):
-        run_data = ['one', 'two']
-        job_runs = [Turtle(), Turtle()]
-        self.job.runs.restore_state = lambda r, a, o, c, n: job_runs
-        state_data = {'enabled': False, 'runs': run_data}
-
-        self.job.restore_state(state_data)
-
+    def test_get_job_runs_from_state(self):
+        job_runs = [
+            dict(
+                run_num=i,
+                job_name="thename",
+                run_time="sometime",
+                start_time="start_time",
+                end_time="sometime",
+                cleanup_run=None,
+                runs=[],
+            ) for i in range(0, 3)
+        ]
+        state_data = {'enabled': False, 'runs': job_runs}
+        returned_runs = self.job.get_job_runs_from_state(state_data)
         assert not self.job.enabled
-        calls = [mock.call(job_runs[i]) for i in xrange(len(job_runs))]
-        self.job.watch.assert_has_calls(calls)
-        self.job.event.ok.assert_called_with('restored')
 
     def test_build_new_runs(self):
         run_time = datetime.datetime(2012, 3, 14, 15, 9, 26)
@@ -114,8 +164,14 @@ class JobTestCase(TestCase):
 
         self.job.node_pool.next.assert_called_with()
         node = self.job.node_pool.next.return_value
-        assert_call(self.job.runs.build_new_run,
-                0, self.job, run_time, node, manual=False)
+        assert_call(
+            self.job.runs.build_new_run,
+            0,
+            self.job,
+            run_time,
+            node,
+            manual=False,
+        )
         assert_length(runs, 1)
         self.job.watch.assert_called_with(runs[0])
 
@@ -127,10 +183,16 @@ class JobTestCase(TestCase):
         runs = list(self.job.build_new_runs(run_time))
 
         assert_length(runs, node_count)
-        for i in xrange(len(runs)):
+        for i in range(len(runs)):
             node = self.job.node_pool.nodes[i]
-            assert_call(self.job.runs.build_new_run,
-                    i, self.job, run_time, node, manual=False)
+            assert_call(
+                self.job.runs.build_new_run,
+                i,
+                self.job,
+                run_time,
+                node,
+                manual=False,
+            )
 
         self.job.watch.assert_has_calls([mock.call(run) for run in runs])
 
@@ -141,8 +203,14 @@ class JobTestCase(TestCase):
         self.job.node_pool.next.assert_called_with()
         node = self.job.node_pool.next.return_value
         assert_length(runs, 1)
-        assert_call(self.job.runs.build_new_run,
-                0, self.job, run_time, node, manual=True)
+        assert_call(
+            self.job.runs.build_new_run,
+            0,
+            self.job,
+            run_time,
+            node,
+            manual=True,
+        )
         self.job.watch.assert_called_with(runs[0])
 
     def test_handler(self):
@@ -177,59 +245,76 @@ class JobTestCase(TestCase):
 
 
 class JobSchedulerTestCase(TestCase):
-
     @setup
     def setup_job(self):
-        self.scheduler = Turtle()
-        run_collection = Turtle()
-        node_pool = Turtle()
-        self.job = job.Job(
-                "jobname",
-                self.scheduler,
-                run_collection=run_collection,
-                node_pool=node_pool,
-        )
-        self.job_scheduler = job.JobScheduler(self.job)
+        self.scheduler = scheduler.ConstantScheduler()
+        run_collection = jobrun.JobRunCollection(20)
+        mock_graph = mock.Mock(autospec=True)
 
-    def test_restore_job_state(self):
-        run_collection = mocks.MockJobRunCollection(get_scheduled=lambda: ['a'])
-        self.job_scheduler.job = Turtle(runs=run_collection)
-        self.job_scheduler._set_callback = Turtle()
-        state_data = 'state_data_token'
-        self.job_scheduler.restore_state(state_data)
-        assert_call(self.job_scheduler.job.restore_state, 0, state_data)
-        assert_length(self.job_scheduler._set_callback.calls, 1)
-        assert_call(self.job_scheduler._set_callback, 0, 'a')
+        mock_graph.get_action_map.return_value = {}
+        node_pool = node.NodePool.from_node(
+            node=mock.Mock(name="foo", autospec=True)
+        )
+
+        mock_graph.action_map = {}
+        self.job = mock.Mock(autospec=True)
+        self.job.allow_overlap = False
+        self.job.max_runtime = datetime.timedelta(days=1)
+        self.job_scheduler = job.JobScheduler(job=self.job)
+
+    def test_restore_state_sets_job_runs(self):
+        self.job.enabled = False
+        mock_runs = [mock.Mock(), mock.Mock()]
+        mock_action_runner = mock.Mock()
+        job_state_data = {'runs': mock_runs, 'enabled': True}
+
+        self.job_scheduler._set_callback = lambda x: x
+
+        self.job.runs.runs = collections.deque()
+        self.job.runs.get_scheduled.return_value = [mock.Mock()]
+        self.job.get_job_runs_from_state.return_value = mock_runs
+
+        with mock.patch(
+            'tron.core.job.recovery.launch_recovery_actionruns_for_job_runs'
+        ) as mock_launch_recovery:
+            mock_launch_recovery.return_value = mock.Mock(autospec=True)
+            self.job_scheduler.restore_state(
+                job_state_data, mock_action_runner
+            )
+            assert self.job.runs.runs == collections.deque(mock_runs)
+            assert mock_launch_recovery.called_once_with(
+                job_runs=mock_runs, master_action_runner=mock_action_runner
+            )
+            calls = [mock.call(mock_runs[i]) for i in range(0, len(mock_runs))]
+            self.job.watch.assert_has_calls(calls)
 
     def test_disable(self):
         self.job_scheduler.disable()
-        assert not self.job.enabled
-        assert_length(self.job.runs.cancel_pending.calls, 1)
+        assert self.job_scheduler.job.enabled is False
+        self.job_scheduler.job.runs.cancel_pending.assert_called_once()
 
     def test_schedule_reconfigured(self):
-        autospec_method(self.job_scheduler.create_and_schedule_runs)
         self.job_scheduler.schedule_reconfigured()
-        assert_length(self.job.runs.remove_pending.calls, 1)
-        self.job_scheduler.create_and_schedule_runs.assert_called_with(
-            ignore_last_run_time=True)
+        self.job.runs.remove_pending.assert_called_once()
 
     def test_run_job(self):
-        self.job_scheduler.schedule = Turtle()
+        self.job_scheduler.schedule = mock.Mock(autospec=True)
         self.scheduler.schedule_on_complete = False
         self.job.runs.get_active = lambda n: []
-        job_run = Turtle(is_cancelled=False)
+        job_run = mock.Mock(autospec=True)
+        job_run.is_cancelled = False
         self.job_scheduler.run_job(job_run)
-        assert_length(job_run.start.calls, 1)
-        assert_length(self.job_scheduler.schedule.calls, 1)
+        assert job_run.start.called_once()
+        assert self.job_scheduler.schedule.called_once()
 
     def test_run_job_shutdown_requested(self):
         self.job_scheduler.shutdown_requested = True
-        self.job_scheduler.schedule = Turtle()
-        job_run = Turtle()
+        self.job_scheduler.schedule = mock.Mock(autospec=True)
+        job_run = mock.Mock(autospec=True)
         self.job_scheduler.run_job(job_run)
-        assert_length(self.job_scheduler.schedule.calls, 0)
-        assert_length(job_run.start.calls, 0)
-        assert_length(job_run.cancel.calls, 0)
+        assert not self.job_scheduler.schedule.called
+        assert not job_run.start.called
+        assert not job_run.cancel.called
 
     def test_run_job_job_disabled(self):
         self.job_scheduler.schedule = Turtle()
@@ -248,23 +333,25 @@ class JobSchedulerTestCase(TestCase):
         assert_length(self.job_scheduler.schedule.calls, 1)
 
     def test_run_job_already_running_queuing(self):
-        self.job_scheduler.schedule = Turtle()
-        self.job.runs.get_active = lambda s: [Turtle()]
-        job_run = Turtle(is_cancelled=False)
+        self.job_scheduler.schedule = mock.Mock(autospec=True)
+        self.job.runs.get_active = lambda s: [mock.Mock(autospec=True)]
+        job_run = mock.Mock(autospec=True)
+        job_run.is_cancelled = False
         self.job_scheduler.run_job(job_run)
-        assert_length(job_run.start.calls, 0)
-        assert_length(job_run.queue.calls, 1)
-        assert_length(self.job_scheduler.schedule.calls, 0)
+        assert not job_run.start.called
+        job_run.queue.assert_called_once()
+        assert not self.job_scheduler.schedule.called
 
     def test_run_job_already_running_cancel(self):
-        self.job_scheduler.schedule = Turtle()
-        self.job.runs.get_active = lambda s: [Turtle()]
+        self.job_scheduler.schedule = mock.Mock(autospec=True)
+        self.job.runs.get_active = lambda s: [mock.Mock(autospec=True)]
         self.job.queueing = False
-        job_run = Turtle(is_cancelled=False)
+        job_run = mock.Mock(autospec=True)
+        job_run.is_cancelled = False
         self.job_scheduler.run_job(job_run)
-        assert_length(job_run.start.calls, 0)
-        assert_length(job_run.cancel.calls, 1)
-        assert_length(self.job_scheduler.schedule.calls, 1)
+        assert not job_run.start.called
+        job_run.cancel.assert_called_once()
+        self.job_scheduler.schedule.assert_called_once()
 
     def test_run_job_already_running_allow_overlap(self):
         self.job_scheduler.schedule = mock.Mock()
@@ -275,13 +362,14 @@ class JobSchedulerTestCase(TestCase):
         job_run.start.assert_called_with()
 
     def test_run_job_has_starting_queueing(self):
-        self.job_scheduler.schedule = Turtle()
-        self.job.runs.get_active = lambda s: [Turtle()]
-        job_run = Turtle(is_cancelled=False)
+        self.job_scheduler.schedule = mock.Mock(autospec=True)
+        self.job.runs.get_active = lambda s: [mock.Mock(autospec=True)]
+        job_run = mock.Mock(autospec=True)
+        job_run.is_cancelled = False
         self.job_scheduler.run_job(job_run)
-        assert_length(job_run.start.calls, 0)
-        assert_length(job_run.queue.calls, 1)
-        assert_length(self.job_scheduler.schedule.calls, 0)
+        assert not job_run.start.called
+        job_run.queue.assert_called_once()
+        assert not self.job_scheduler.schedule.called
 
     def test_run_job_schedule_on_complete(self):
         self.job_scheduler.schedule = Turtle()
@@ -292,8 +380,8 @@ class JobSchedulerTestCase(TestCase):
         assert_length(job_run.start.calls, 1)
         assert_length(self.job_scheduler.schedule.calls, 0)
 
-class JobSchedulerGetRunsToScheduleTestCase(TestCase):
 
+class JobSchedulerGetRunsToScheduleTestCase(TestCase):
     @setup
     def setup_job(self):
         self.scheduler = mock.Mock()
@@ -320,7 +408,8 @@ class JobSchedulerGetRunsToScheduleTestCase(TestCase):
 
         self.job.runs.get_newest.assert_called_with(include_manual=False)
         self.job.scheduler.next_run_time.assert_called_once_with(
-                self.job.runs.get_newest.return_value.run_time)
+            self.job.runs.get_newest.return_value.run_time,
+        )
         assert_length(job_runs, 1)
         # This should return a JobRun which has the job attached as an observer
         job_runs[0].attach.assert_any_call(True, self.job)
@@ -330,7 +419,8 @@ class JobSchedulerGetRunsToScheduleTestCase(TestCase):
 
         self.job.runs.get_newest.assert_called_with(include_manual=False)
         self.job.scheduler.next_run_time.assert_called_once_with(
-            self.job.runs.get_newest.return_value.run_time)
+            self.job.runs.get_newest.return_value.run_time,
+        )
         assert_length(job_runs, 1)
         # This should return a JobRun which has the job attached as an observer
         job_runs[0].attach.assert_any_call(True, self.job)
@@ -377,6 +467,20 @@ class JobSchedulerManualStartTestCase(testingutils.MockTimeTestCase):
         assert_length(manual_runs, 1)
         self.manual_run.start.assert_called_once_with()
 
+    def test_manual_start_default_with_timezone(self):
+        self.job.time_zone = mock.Mock()
+        with mock.patch(
+            'tron.core.job.timeutils.current_time',
+        ) as mock_current:
+            manual_runs = self.job_scheduler.manual_start()
+            mock_current.assert_called_with(tz=self.job.time_zone)
+            self.job.build_new_runs.assert_called_with(
+                mock_current.return_value,
+                manual=True,
+            )
+        assert_length(manual_runs, 1)
+        self.manual_run.start.assert_called_once_with()
+
     def test_manual_start_with_run_time(self):
         run_time = datetime.datetime(2012, 3, 14, 15, 9, 26)
         manual_runs = self.job_scheduler.manual_start(run_time)
@@ -387,19 +491,30 @@ class JobSchedulerManualStartTestCase(testingutils.MockTimeTestCase):
 
 
 class JobSchedulerScheduleTestCase(TestCase):
-
     @setup
     def setup_job(self):
-        self.scheduler = mock.Mock()
-        run_collection = mock.Mock(has_pending=False)
+        self.scheduler = mock.Mock(autospec=True)
+        self.scheduler.next_run_time.return_value = 0
+        mock_run = mock.Mock()
+        mock_run.seconds_until_run_time.return_value = 0
+        run_collection = mock.Mock(
+            has_pending=False,
+            autospec=True,
+            return_value=[mock_run],
+        )
+        mock_build_new_run = mock.Mock()
+        run_collection.build_new_run.return_value = mock_build_new_run
+        mock_build_new_run.seconds_until_run_time.return_value = 0
         node_pool = mock.Mock()
         self.job = job.Job(
-            "jobname",
-            self.scheduler,
+            name="jobname",
+            scheduler=self.scheduler,
             run_collection=run_collection,
             node_pool=node_pool,
         )
         self.job_scheduler = job.JobScheduler(self.job)
+        self.original_build_new_runs = self.job.build_new_runs
+        self.job.build_new_runs = mock.Mock(return_value=[mock_run])
 
     @setup_teardown
     def mock_eventloop(self):
@@ -418,12 +533,13 @@ class JobSchedulerScheduleTestCase(TestCase):
         assert_length(self.eventloop.call_later.mock_calls, 1)
 
     def test_enable_noop(self):
-        self.job.enalbed = True
+        self.job.enabled = True
         self.job_scheduler.enable()
         assert self.job.enabled
         assert_length(self.eventloop.call_later.mock_calls, 0)
 
     def test_schedule(self):
+        self.job.build_new_runs = self.original_build_new_runs
         self.job_scheduler.schedule()
         assert_length(self.eventloop.call_later.mock_calls, 1)
 
@@ -448,8 +564,12 @@ class JobSchedulerScheduleTestCase(TestCase):
         queued_job_run = mock.Mock()
         self.job.runs.get_first_queued = lambda: queued_job_run
         self.job_scheduler.handle_job_events(self.job, job.Job.NOTIFY_RUN_DONE)
-        self.eventloop.call_later.assert_any_call(0,
-            self.job_scheduler.run_job, queued_job_run, run_queued=True)
+        self.eventloop.call_later.assert_any_call(
+            0,
+            self.job_scheduler.run_job,
+            queued_job_run,
+            run_queued=True,
+        )
 
     def test_handle_job_events_schedule_on_complete(self):
         self.job_scheduler.schedule = mock.Mock()
@@ -464,25 +584,49 @@ class JobSchedulerScheduleTestCase(TestCase):
 
     def test_handler_no_queued(self):
         self.job_scheduler.run_job = mock.Mock()
+
         def get_queued(state):
             if state == ActionRun.STATE_QUEUED:
                 return []
+
         self.job.runs.get_runs_by_state = get_queued
         self.job_scheduler.handler(self.job, job.Job.NOTIFY_RUN_DONE)
         self.job_scheduler.run_job.assert_not_called()
 
+    def test_run_queue_schedule(self):
+        with mock.patch.object(
+            self.job_scheduler,
+            'schedule',
+        ) as mock_schedule:
+            self.job_scheduler.run_job = mock.Mock()
+            self.job.scheduler.schedule_on_complete = False
+            queued_job_run = mock.Mock()
+            self.job.runs.get_first_queued = lambda: queued_job_run
+            self.job_scheduler.run_queue_schedule()
+            self.eventloop.call_later.assert_called_once_with(
+                0,
+                self.job_scheduler.run_job,
+                queued_job_run,
+                run_queued=True,
+            )
+            mock_schedule.assert_called_once_with()
+
 
 class JobSchedulerFactoryTestCase(TestCase):
-
     @setup
     def setup_factory(self):
         self.context = mock.Mock()
         self.output_stream_dir = mock.Mock()
         self.time_zone = mock.Mock()
         self.action_runner = mock.create_autospec(
-            actioncommand.SubprocessActionRunnerFactory)
+            actioncommand.SubprocessActionRunnerFactory,
+        )
         self.factory = job.JobSchedulerFactory(
-            self.context, self.output_stream_dir, self.time_zone, self.action_runner)
+            self.context,
+            self.output_stream_dir,
+            self.time_zone,
+            self.action_runner,
+        )
 
     def test_build(self):
         config = mock.Mock()
@@ -491,14 +635,16 @@ class JobSchedulerFactoryTestCase(TestCase):
             args, _ = mock_job.from_config.call_args
             job_config, scheduler, context, output_path, action_runner = args
             assert_equal(job_config, config)
-            assert_equal(job_scheduler.get_job(), mock_job.from_config.return_value)
+            assert_equal(
+                job_scheduler.get_job(),
+                mock_job.from_config.return_value,
+            )
             assert_equal(context, self.context)
             assert_equal(output_path.base, self.output_stream_dir)
             assert_equal(action_runner, self.action_runner)
 
 
 class JobCollectionTestCase(TestCase):
-
     @setup
     def setup_collection(self):
         self.collection = job.JobCollection()
@@ -511,11 +657,13 @@ class JobCollectionTestCase(TestCase):
         result = self.collection.load_from_config(job_configs, factory, True)
         result = list(result)
         self.collection.jobs.filter_by_name.assert_called_with(job_configs)
-        expected_calls = [mock.call(v) for v in job_configs.itervalues()]
+        expected_calls = [mock.call(v) for v in six.itervalues(job_configs)]
         assert_mock_calls(expected_calls, factory.build.mock_calls)
         assert_length(self.collection.add.mock_calls, len(job_configs) * 2)
         assert_length(result, len(job_configs))
-        job_schedulers = [call[1][0] for call in self.collection.add.mock_calls[::2]]
+        job_schedulers = [
+            call[1][0] for call in self.collection.add.mock_calls[::2]
+        ]
         for job_scheduler in job_schedulers:
             job_scheduler.schedule.assert_called_with()
             job_scheduler.get_job.assert_called_with()
@@ -523,11 +671,17 @@ class JobCollectionTestCase(TestCase):
     def test_update(self):
         mock_scheduler = mock.create_autospec(job.JobScheduler)
         existing_scheduler = mock.create_autospec(job.JobScheduler)
-        autospec_method(self.collection.get_by_name, return_value=existing_scheduler)
+        autospec_method(
+            self.collection.get_by_name,
+            return_value=existing_scheduler,
+        )
         assert self.collection.update(mock_scheduler)
-        self.collection.get_by_name.assert_called_with(mock_scheduler.get_name())
+        self.collection.get_by_name.assert_called_with(
+            mock_scheduler.get_name(),
+        )
         existing_scheduler.get_job().update_from_job.assert_called_with(
-            mock_scheduler.get_job.return_value)
+            mock_scheduler.get_job.return_value,
+        )
         existing_scheduler.schedule_reconfigured.assert_called_with()
 
 
