@@ -1,43 +1,74 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 import datetime
+
 import mock
 import pytz
-from testify import TestCase, setup, assert_equal
+from testify import assert_equal
+from testify import setup
+from testify import TestCase
 from testify.assertions import assert_in
-from tests.assertions import assert_length, assert_raises, assert_call
-from tron import node, event, actioncommand
-from tron.core import jobrun, actionrun, actiongraph, job
-from tests.testingutils import Turtle, autospec_method
+
+from tests.assertions import assert_call
+from tests.assertions import assert_length
+from tests.assertions import assert_raises
+from tests.testingutils import autospec_method
+from tests.testingutils import Turtle
+from tron import actioncommand
+from tron import event
+from tron import node
+from tron.core import actiongraph
+from tron.core import actionrun
+from tron.core import job
+from tron.core import jobrun
 from tron.serialize import filehandler
 
 
 def build_mock_job():
     action_graph = mock.create_autospec(actiongraph.ActionGraph)
     runner = mock.create_autospec(actioncommand.SubprocessActionRunnerFactory)
-    return mock.create_autospec(job.Job,
+    return mock.create_autospec(
+        job.Job,
         action_graph=action_graph,
         output_path=mock.Mock(),
         context=mock.Mock(),
-        action_runner=runner)
+        action_runner=runner,
+        service='foo',
+        deploy_group='test',
+    )
 
 
 class JobRunTestCase(TestCase):
 
-    now = datetime.datetime(2012, 3, 14, 15, 9, 20)
+    now = datetime.datetime(2012, 3, 14, 15, 9, 20, tzinfo=None)
+    now_with_tz = datetime.datetime(2012, 3, 14, 15, 9, 20, tzinfo=pytz.utc)
 
     @setup
     def setup_jobrun(self):
         self.job = build_mock_job()
         self.action_graph = self.job.action_graph
-        self.run_time = datetime.datetime(2012, 3, 14, 15, 9 ,26)
+        self.run_time = datetime.datetime(2012, 3, 14, 15, 9, 26)
         mock_node = mock.create_autospec(node.Node)
-        self.job_run = jobrun.JobRun('jobname', 7, self.run_time, mock_node,
-                action_runs=Turtle(
-                    action_runs_with_cleanup=[],
-                    get_startable_action_runs=lambda: []))
+        self.job_run = jobrun.JobRun(
+            'jobname',
+            7,
+            self.run_time,
+            mock_node,
+            action_runs=Turtle(
+                action_runs_with_cleanup=[],
+                get_startable_action_runs=lambda: [],
+            ),
+            service='baz',
+            deploy_group='other',
+        )
         autospec_method(self.job_run.watch)
         autospec_method(self.job_run.notify)
         self.job_run.event = mock.create_autospec(event.EventRecorder)
-        self.action_run = mock.create_autospec(actionrun.ActionRun)
+        self.action_run = mock.create_autospec(
+            actionrun.ActionRun,
+            is_skipped=False,
+        )
 
     def test__init__(self):
         assert_equal(self.job_run.job_name, 'jobname')
@@ -48,19 +79,31 @@ class JobRunTestCase(TestCase):
         run_num = 6
         mock_node = mock.create_autospec(node.Node)
         run = jobrun.JobRun.for_job(
-                self.job, run_num, self.run_time, mock_node, False)
+            self.job,
+            run_num,
+            self.run_time,
+            mock_node,
+            False,
+        )
 
         assert_equal(run.action_runs.action_graph, self.action_graph)
         assert_equal(run.job_name, self.job.get_name.return_value)
         assert_equal(run.run_num, run_num)
         assert_equal(run.node, mock_node)
         assert not run.manual
+        assert_equal(run.service, self.job.service)
+        assert_equal(run.deploy_group, self.job.deploy_group)
 
     def test_for_job_manual(self):
         run_num = 6
         mock_node = mock.create_autospec(node.Node)
         run = jobrun.JobRun.for_job(
-                self.job, run_num, self.run_time, mock_node, True)
+            self.job,
+            run_num,
+            self.run_time,
+            mock_node,
+            True,
+        )
         assert_equal(run.action_runs.action_graph, self.action_graph)
         assert run.manual
 
@@ -69,14 +112,19 @@ class JobRunTestCase(TestCase):
         assert_equal(state_data['run_num'], 7)
         assert not state_data['manual']
         assert_equal(state_data['run_time'], self.run_time)
+        assert_equal(state_data['service'], 'baz')
+        assert_equal(state_data['deploy_group'], 'other')
 
     def test_set_action_runs(self):
         self.job_run._action_runs = None
         count = 2
         action_runs = [
-            mock.create_autospec(actionrun.ActionRun) for _ in xrange(count)]
-        run_collection = mock.create_autospec(actionrun.ActionRunCollection,
-            action_runs_with_cleanup=action_runs)
+            mock.create_autospec(actionrun.ActionRun) for _ in range(count)
+        ]
+        run_collection = mock.create_autospec(
+            actionrun.ActionRunCollection,
+            action_runs_with_cleanup=action_runs,
+        )
         self.job_run._set_action_runs(run_collection)
         assert_equal(self.job_run.watch.call_count, count)
 
@@ -94,8 +142,11 @@ class JobRunTestCase(TestCase):
 
     def test_set_action_runs_duplicate(self):
         run_collection = mock.create_autospec(actionrun.ActionRunCollection)
-        assert_raises(ValueError,
-            self.job_run._set_action_runs, run_collection)
+        assert_raises(
+            ValueError,
+            self.job_run._set_action_runs,
+            run_collection,
+        )
 
     @mock.patch('tron.core.jobrun.timeutils.current_time', autospec=True)
     def test_seconds_until_run_time(self, mock_current_time):
@@ -105,7 +156,7 @@ class JobRunTestCase(TestCase):
 
     @mock.patch('tron.core.jobrun.timeutils.current_time', autospec=True)
     def test_seconds_until_run_time_with_tz(self, mock_current_time):
-        mock_current_time.return_value = self.now
+        mock_current_time.return_value = self.now_with_tz
         self.job_run.run_time = self.run_time.replace(tzinfo=pytz.utc)
         seconds = self.job_run.seconds_until_run_time()
         assert_equal(seconds, 6)
@@ -132,7 +183,8 @@ class JobRunTestCase(TestCase):
 
     def test_do_start(self):
         startable_runs = [
-            mock.create_autospec(actionrun.ActionRun) for _ in xrange(3)]
+            mock.create_autospec(actionrun.ActionRun) for _ in range(3)
+        ]
         self.job_run.action_runs.get_startable_action_runs = lambda: startable_runs
 
         assert self.job_run._do_start()
@@ -161,7 +213,8 @@ class JobRunTestCase(TestCase):
 
     def test_start_action_runs(self):
         startable_runs = [
-            mock.create_autospec(actionrun.ActionRun) for _ in xrange(3)]
+            mock.create_autospec(actionrun.ActionRun) for _ in range(3)
+        ]
         self.job_run.action_runs.get_startable_action_runs = lambda: startable_runs
 
         started_runs = self.job_run._start_action_runs()
@@ -169,7 +222,8 @@ class JobRunTestCase(TestCase):
 
     def test_start_action_runs_failed(self):
         startable_runs = [
-            mock.create_autospec(actionrun.ActionRun) for _ in xrange(3)]
+            mock.create_autospec(actionrun.ActionRun) for _ in range(3)
+        ]
         startable_runs[0].start.return_value = False
         self.job_run.action_runs.get_startable_action_runs = lambda: startable_runs
 
@@ -178,7 +232,8 @@ class JobRunTestCase(TestCase):
 
     def test_start_action_runs_all_failed(self):
         startable_runs = [
-            mock.create_autospec(actionrun.ActionRun) for _ in xrange(2)]
+            mock.create_autospec(actionrun.ActionRun) for _ in range(2)
+        ]
         for startable_run in startable_runs:
             startable_run.start.return_value = False
         self.job_run.action_runs.get_startable_action_runs = lambda: startable_runs
@@ -196,12 +251,15 @@ class JobRunTestCase(TestCase):
 
     def test_handler_with_startable(self):
         startable_run = mock.create_autospec(actionrun.ActionRun)
-        self.job_run.action_runs.get_startable_action_runs = lambda: [startable_run]
+        self.job_run.action_runs.get_startable_action_runs = lambda: [
+            startable_run, ]
         autospec_method(self.job_run.finalize)
         self.action_run.is_broken = False
 
         self.job_run.handler(self.action_run, mock.Mock())
-        self.job_run.notify.assert_called_with(self.job_run.NOTIFY_STATE_CHANGED)
+        self.job_run.notify.assert_called_with(
+            self.job_run.NOTIFY_STATE_CHANGED,
+        )
         startable_run.start.assert_called_with()
         assert not self.job_run.finalize.mock_calls
 
@@ -243,6 +301,14 @@ class JobRunTestCase(TestCase):
         self.job_run.handler(self.action_run, mock.Mock())
         assert not self.job_run._start_action_runs.mock_calls
 
+    def test_handler_action_run_skipped(self):
+        self.action_run.is_broken = False
+        self.action_run.is_skipped = True
+        self.job_run.action_runs.is_scheduled = True
+        autospec_method(self.job_run._start_action_runs)
+        self.job_run.handler(self.action_run, mock.Mock())
+        assert not self.job_run._start_action_runs.mock_calls
+
     def test_state(self):
         assert_equal(self.job_run.state, actionrun.ActionRun.STATE_SUCCEEDED)
 
@@ -282,40 +348,48 @@ class JobRunTestCase(TestCase):
 
 
 class JobRunFromStateTestCase(TestCase):
-
     @setup
     def setup_jobrun(self):
         self.action_graph = mock.create_autospec(actiongraph.ActionGraph)
-        self.run_time = datetime.datetime(2012, 3, 14, 15, 9 ,26)
+        self.run_time = datetime.datetime(2012, 3, 14, 15, 9, 26)
         self.path = ['base', 'path']
         self.output_path = mock.create_autospec(filehandler.OutputPath)
         self.node_pool = mock.create_autospec(node.NodePool)
-        self.action_run_state_data = [{
-            'job_run_id':       'thejobname.22',
-            'action_name':      'blingaction',
-            'state':            'succeeded',
-            'run_time':         'sometime',
-            'start_time':       'sometime',
-            'end_time':         'sometime',
-            'command':          'doit',
-            'node_name':        'thenode'
-        }]
+        self.action_run_state_data = [
+            {
+                'job_run_id': 'thejobname.22',
+                'action_name': 'blingaction',
+                'state': 'succeeded',
+                'run_time': 'sometime',
+                'start_time': 'sometime',
+                'end_time': 'sometime',
+                'command': 'doit',
+                'node_name': 'thenode',
+            }
+        ]
         self.state_data = {
-            'job_name':         'thejobname',
-            'run_num':          22,
-            'run_time':         self.run_time,
-            'node_name':        'thebox',
-            'end_time':         'the_end',
-            'start_time':       'start_time',
-            'runs':             self.action_run_state_data,
-            'cleanup_run':      None,
-            'manual':           True
+            'job_name': 'thejobname',
+            'run_num': 22,
+            'run_time': self.run_time,
+            'node_name': 'thebox',
+            'end_time': 'the_end',
+            'start_time': 'start_time',
+            'runs': self.action_run_state_data,
+            'cleanup_run': None,
+            'manual': True,
+            'service': 'foo',
+            'deploy_group': 'test',
         }
         self.context = mock.Mock()
 
     def test_from_state(self):
-        run = jobrun.JobRun.from_state(self.state_data, self.action_graph,
-            self.output_path, self.context, self.node_pool)
+        run = jobrun.JobRun.from_state(
+            self.state_data,
+            self.action_graph,
+            self.output_path,
+            self.context,
+            self.node_pool,
+        )
         assert_length(run.action_runs.run_map, 1)
         assert_equal(run.job_name, self.state_data['job_name'])
         assert_equal(run.run_time, self.run_time)
@@ -323,10 +397,17 @@ class JobRunFromStateTestCase(TestCase):
         assert_equal(run.output_path, self.output_path)
         assert run.context.next
         assert run.action_graph
+        assert_equal(run.service, self.state_data['service'])
+        assert_equal(run.deploy_group, self.state_data['deploy_group'])
 
     def test_from_state_node_no_longer_exists(self):
-        run = jobrun.JobRun.from_state(self.state_data, self.action_graph,
-            self.output_path, self.context, self.node_pool)
+        run = jobrun.JobRun.from_state(
+            self.state_data,
+            self.action_graph,
+            self.output_path,
+            self.context,
+            self.node_pool,
+        )
         assert_length(run.action_runs.run_map, 1)
         assert_equal(run.job_name, 'thejobname')
         assert_equal(run.run_time, self.run_time)
@@ -360,7 +441,6 @@ class MockJobRun(Turtle):
 
 
 class JobRunCollectionTestCase(TestCase):
-
     def _mock_run(self, **kwargs):
         return MockJobRun(**kwargs)
 
@@ -369,10 +449,12 @@ class JobRunCollectionTestCase(TestCase):
         self.run_collection = jobrun.JobRunCollection(5)
         self.job_runs = [
             self._mock_run(state=actionrun.ActionRun.STATE_QUEUED, run_num=4),
-            self._mock_run(state=actionrun.ActionRun.STATE_RUNNING, run_num=3)
+            self._mock_run(state=actionrun.ActionRun.STATE_RUNNING, run_num=3),
         ] + [
-            self._mock_run(state=actionrun.ActionRun.STATE_SUCCEEDED, run_num=i)
-            for i in xrange(2,0,-1)
+            self._mock_run(
+                state=actionrun.ActionRun.STATE_SUCCEEDED,
+                run_num=i,
+            ) for i in range(2, 0, -1)
         ]
         self.run_collection.runs.extend(self.job_runs)
         self.mock_node = mock.create_autospec(node.Node)
@@ -385,8 +467,7 @@ class JobRunCollectionTestCase(TestCase):
         runs = jobrun.JobRunCollection.from_config(job_config)
         assert_equal(runs.run_limit, 20)
 
-    def test_restore_state(self):
-        run_collection = jobrun.JobRunCollection(20)
+    def test_job_runs_from_state(self):
         state_data = [
             dict(
                 run_num=i,
@@ -395,30 +476,32 @@ class JobRunCollectionTestCase(TestCase):
                 start_time="start_time",
                 end_time="sometime",
                 cleanup_run=None,
-                runs=[]
-            ) for i in xrange(3,-1,-1)
+                runs=[],
+            ) for i in range(3, -1, -1)
         ]
         action_graph = mock.create_autospec(actiongraph.ActionGraph)
         output_path = mock.create_autospec(filehandler.OutputPath)
         context = mock.Mock()
         node_pool = mock.create_autospec(node.NodePool)
-
-        restored_runs = run_collection.restore_state(
-                state_data, action_graph, output_path, context, node_pool)
-        assert_equal(run_collection.runs[0].run_num, 3)
-        assert_equal(run_collection.runs[3].run_num, 0)
-        assert_length(restored_runs, 4)
-
-    def test_restore_state_with_runs(self):
-        assert_raises(ValueError,
-                self.run_collection.restore_state, None, None, None, None, None)
+        runs = jobrun.job_runs_from_state(
+            state_data,
+            action_graph,
+            output_path,
+            context,
+            node_pool,
+        )
+        assert len(runs) == 4
+        assert all([type(job) == jobrun.JobRun for job in runs])
 
     def test_build_new_run(self):
         autospec_method(self.run_collection.remove_old_runs)
         run_time = datetime.datetime(2012, 3, 14, 15, 9, 26)
         mock_job = build_mock_job()
         job_run = self.run_collection.build_new_run(
-            mock_job, run_time, self.mock_node)
+            mock_job,
+            run_time,
+            self.mock_node,
+        )
         assert_in(job_run, self.run_collection.runs)
         self.run_collection.remove_old_runs.assert_called_with()
         assert_equal(job_run.run_num, 5)
@@ -429,16 +512,22 @@ class JobRunCollectionTestCase(TestCase):
         run_time = datetime.datetime(2012, 3, 14, 15, 9, 26)
         mock_job = build_mock_job()
         job_run = self.run_collection.build_new_run(
-            mock_job, run_time, self.mock_node, True)
+            mock_job,
+            run_time,
+            self.mock_node,
+            True,
+        )
         assert_in(job_run, self.run_collection.runs)
         self.run_collection.remove_old_runs.assert_called_with()
         assert_equal(job_run.run_num, 5)
         assert job_run.manual
 
     def test_cancel_pending(self):
-        pending_runs = [mock.Mock() for _ in xrange(2)]
-        autospec_method(self.run_collection.get_pending,
-            return_value=pending_runs)
+        pending_runs = [mock.Mock() for _ in range(2)]
+        autospec_method(
+            self.run_collection.get_pending,
+            return_value=pending_runs,
+        )
         self.run_collection.cancel_pending()
         for pending_run in pending_runs:
             pending_run.cancel.assert_called_with()
@@ -501,7 +590,10 @@ class JobRunCollectionTestCase(TestCase):
 
     def test_get_newest_exclude_manual(self):
         run = self._mock_run(
-                state=actionrun.ActionRun.STATE_RUNNING, run_num=5, manual=True)
+            state=actionrun.ActionRun.STATE_RUNNING,
+            run_num=5,
+            manual=True,
+        )
         self.job_runs.insert(0, run)
         newest_run = self.run_collection.get_newest(include_manual=False)
         assert_equal(newest_run, self.job_runs[1])
@@ -513,8 +605,9 @@ class JobRunCollectionTestCase(TestCase):
     def test_pending(self):
         run_num = self.run_collection.next_run_num()
         scheduled_run = self._mock_run(
-                run_num=run_num,
-                state=actionrun.ActionRun.STATE_SCHEDULED)
+            run_num=run_num,
+            state=actionrun.ActionRun.STATE_SCHEDULED,
+        )
         self.run_collection.runs.appendleft(scheduled_run)
         pending = list(self.run_collection.get_pending())
         assert_length(pending, 2)
@@ -523,7 +616,8 @@ class JobRunCollectionTestCase(TestCase):
     def test_get_active(self):
         starting_run = self._mock_run(
             run_num=self.run_collection.next_run_num(),
-            state=actionrun.ActionRun.STATE_STARTING)
+            state=actionrun.ActionRun.STATE_STARTING,
+        )
         self.run_collection.runs.appendleft(starting_run)
         active = list(self.run_collection.get_active())
         assert_length(active, 2)
@@ -532,7 +626,8 @@ class JobRunCollectionTestCase(TestCase):
     def test_get_active_with_node(self):
         starting_run = self._mock_run(
             run_num=self.run_collection.next_run_num(),
-            state=actionrun.ActionRun.STATE_STARTING)
+            state=actionrun.ActionRun.STATE_STARTING,
+        )
         starting_run.node = 'differentnode'
         self.run_collection.runs.appendleft(starting_run)
         active = list(self.run_collection.get_active('anode'))
@@ -546,7 +641,9 @@ class JobRunCollectionTestCase(TestCase):
     def test_get_first_queued(self):
         run_num = self.run_collection.next_run_num()
         second_queued = self._mock_run(
-            run_num=run_num, state=actionrun.ActionRun.STATE_QUEUED)
+            run_num=run_num,
+            state=actionrun.ActionRun.STATE_QUEUED,
+        )
         self.run_collection.runs.appendleft(second_queued)
 
         first_queued = self.run_collection.get_first_queued()
@@ -566,7 +663,8 @@ class JobRunCollectionTestCase(TestCase):
         scheduled_run = self._mock_run(
             run_num=self.run_collection.next_run_num(),
             state=actionrun.ActionRun.STATE_SCHEDULED,
-            node="nine")
+            node="nine",
+        )
         self.run_collection.runs.appendleft(scheduled_run)
 
         next_run = self.run_collection.get_next_to_finish(node="seven")
@@ -619,7 +717,9 @@ class JobRunCollectionTestCase(TestCase):
         action_name = 'action_name'
         self.run_collection.runs = job_runs = [mock.Mock(), mock.Mock()]
         runs = self.run_collection.get_action_runs(action_name)
-        expected = [job_run.get_action_run.return_value for job_run in job_runs]
+        expected = [
+            job_run.get_action_run.return_value for job_run in job_runs
+        ]
         assert_equal(runs, expected)
         for job_run in job_runs:
             job_run.get_action_run.assert_called_with(action_name)
