@@ -1,11 +1,21 @@
 """Tests for reconfiguring mcp."""
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 import tempfile
 
-from testify import TestCase, run, setup, assert_equal, teardown, suite
+from testify import assert_equal
+from testify import run
+from testify import setup
+from testify import suite
+from testify import teardown
+from testify import TestCase
 
 from tests.assertions import assert_length
-from tron import mcp, event
-from tron.config import config_parse, schema
+from tron import event
+from tron import mcp
+from tron.config import config_parse
+from tron.config import schema
 from tron.serialize import filehandler
 
 
@@ -22,52 +32,69 @@ class MCPReconfigureTestCase(TestCase):
         ],
         node_pools=[dict(name='nodePool', nodes=['node0', 'node1'])],
         command_context={
-            'thischanges': 'froma'
+            'thischanges': 'froma',
         },
         jobs=[
             dict(
                 name='test_unchanged',
                 node='node0',
                 schedule='daily',
-                actions=[dict(name='action_unchanged',
-                              command='command_unchanged') ]
+                actions=[
+                    dict(
+                        name='action_unchanged',
+                        command='command_unchanged',
+                    ),
+                ],
             ),
             dict(
                 name='test_remove',
                 node='node1',
                 schedule=dict(interval='20s'),
-                actions=[dict(name='action_remove',
-                              command='command_remove')],
-                cleanup_action=dict(name='cleanup', command='doit')
+                actions=[
+                    dict(
+                        name='action_remove',
+                        command='command_remove',
+                    ),
+                ],
+                cleanup_action=dict(name='cleanup', command='doit'),
             ),
             dict(
                 name='test_change',
                 node='nodePool',
                 schedule=dict(interval='20s'),
                 actions=[
-                    dict(name='action_change',
-                         command='command_change'),
-                    dict(name='action_remove2',
-                         command='command_remove2',
-                         requires=['action_change']),
+                    dict(
+                        name='action_change',
+                        command='command_change',
+                    ),
+                    dict(
+                        name='action_remove2',
+                        command='command_remove2',
+                        requires=['action_change'],
+                    ),
                 ],
             ),
             dict(
                 name='test_daily_change',
                 node='node0',
                 schedule='daily',
-                        actions=[dict(name='action_daily_change',
-                                      command='command')],
+                actions=[
+                    dict(
+                        name='action_daily_change',
+                        command='command',
+                    ),
+                ],
             ),
             dict(
                 name='test_action_added',
                 node='node0',
                 schedule=dict(interval='10s'),
                 actions=[
-                    dict(name='action_first', command='command_do_it')
-                ]
-            )
-        ])
+                    dict(name='action_first', command='command_do_it'),
+                ],
+            ),
+        ],
+    )
 
     post_config = dict(
         ssh_options=dict(
@@ -81,38 +108,50 @@ class MCPReconfigureTestCase(TestCase):
         node_pools=[dict(name='nodePool', nodes=['node0', 'node1'])],
         command_context={
             'a_variable': 'is_constant',
-            'thischanges': 'tob'
+            'thischanges': 'tob',
         },
         jobs=[
             dict(
                 name='test_unchanged',
                 node='node0',
                 schedule='daily',
-                actions=[dict(name='action_unchanged',
-                              command='command_unchanged') ]
+                actions=[
+                    dict(
+                        name='action_unchanged',
+                        command='command_unchanged',
+                    ),
+                ],
             ),
             dict(
                 name='test_change',
                 node='nodePool',
                 schedule='daily',
                 actions=[
-                    dict(name='action_change',
-                         command='command_changed'),
+                    dict(
+                        name='action_change',
+                        command='command_changed',
+                    ),
                 ],
             ),
             dict(
                 name='test_daily_change',
                 node='node0',
                 schedule='daily',
-                        actions=[dict(name='action_daily_change',
-                                      command='command_changed')],
+                actions=[
+                    dict(
+                        name='action_daily_change',
+                        command='command_changed',
+                    ),
+                ],
             ),
             dict(
                 name='test_new',
                 node='nodePool',
                 schedule=dict(interval='20s'),
-                actions=[dict(name='action_new',
-                              command='command_new')]
+                actions=[dict(
+                    name='action_new',
+                    command='command_new',
+                )],
             ),
             dict(
                 name='test_action_added',
@@ -121,9 +160,10 @@ class MCPReconfigureTestCase(TestCase):
                 actions=[
                     dict(name='action_first', command='command_do_it'),
                     dict(name='action_second', command='command_ok'),
-                ]
-            )
-        ])
+                ],
+            ),
+        ],
+    )
 
     def _get_config(self, idx, output_dir):
         config = dict(self.post_config if idx else self.pre_config)
@@ -161,9 +201,9 @@ class MCPReconfigureTestCase(TestCase):
         assert 'MASTER.test_unchanged' in self.mcp.jobs
         job_sched = self.mcp.jobs.get_by_name('MASTER.test_unchanged')
         orig_job = job_sched.job
-        run0 = job_sched.get_runs_to_schedule(False).next()
+        run0 = next(job_sched.get_runs_to_schedule(False))
         run0.start()
-        run1 = job_sched.get_runs_to_schedule(False).next()
+        run1 = next(job_sched.get_runs_to_schedule(False))
 
         assert_equal(job_sched.job.name, "MASTER.test_unchanged")
         action_map = job_sched.job.action_graph.action_map
@@ -186,7 +226,7 @@ class MCPReconfigureTestCase(TestCase):
     def test_job_unchanged_disabled(self):
         job_sched = self.mcp.jobs.get_by_name('MASTER.test_unchanged')
         orig_job = job_sched.job
-        job_sched.get_runs_to_schedule(False).next()
+        next(job_sched.get_runs_to_schedule(False))
         job_sched.disable()
 
         self.reconfigure()
@@ -198,9 +238,9 @@ class MCPReconfigureTestCase(TestCase):
     def test_job_removed(self):
         assert 'MASTER.test_remove' in self.mcp.jobs
         job_sched = self.mcp.jobs.get_by_name('MASTER.test_remove')
-        run0 = job_sched.get_runs_to_schedule(False).next()
+        run0 = next(job_sched.get_runs_to_schedule(False))
         run0.start()
-        run1 = job_sched.get_runs_to_schedule(False).next()
+        run1 = next(job_sched.get_runs_to_schedule(False))
 
         assert_equal(job_sched.job.name, "MASTER.test_remove")
         action_map = job_sched.job.action_graph.action_map
@@ -208,7 +248,7 @@ class MCPReconfigureTestCase(TestCase):
         assert_equal(action_map['action_remove'].name, 'action_remove')
 
         self.reconfigure()
-        assert not 'test_remove' in self.mcp.jobs
+        assert 'test_remove' not in self.mcp.jobs
         assert not job_sched.job.enabled
         assert not run1.is_scheduled
 
@@ -216,9 +256,9 @@ class MCPReconfigureTestCase(TestCase):
     def test_job_changed(self):
         assert 'MASTER.test_change' in self.mcp.jobs
         job_sched = self.mcp.jobs.get_by_name('MASTER.test_change')
-        run0 = job_sched.get_runs_to_schedule(False).next()
+        run0 = next(job_sched.get_runs_to_schedule(False))
         run0.start()
-        job_sched.get_runs_to_schedule(False).next()
+        next(job_sched.get_runs_to_schedule(False))
         assert_equal(len(job_sched.job.runs.runs), 2)
 
         assert_equal(job_sched.job.name, "MASTER.test_change")
@@ -252,7 +292,7 @@ class MCPReconfigureTestCase(TestCase):
 
     @suite('integration')
     def test_job_new(self):
-        assert not 'test_new' in self.mcp.jobs
+        assert 'test_new' not in self.mcp.jobs
         self.reconfigure()
 
         assert 'MASTER.test_new' in self.mcp.jobs
@@ -270,7 +310,7 @@ class MCPReconfigureTestCase(TestCase):
     def test_daily_reschedule(self):
         job_sched = self.mcp.jobs.get_by_name('MASTER.test_daily_change')
 
-        job_sched.get_runs_to_schedule(False).next()
+        next(job_sched.get_runs_to_schedule(False))
 
         assert_equal(len(job_sched.job.runs.runs), 1)
         run = job_sched.job.runs.runs[0]
@@ -291,5 +331,6 @@ class MCPReconfigureTestCase(TestCase):
         job_sched = self.mcp.jobs.get_by_name('MASTER.test_action_added')
         assert_length(job_sched.job.action_graph.action_map, 2)
 
+
 if __name__ == '__main__':
     run()
