@@ -1,14 +1,16 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 import atexit
 import datetime
-from exceptions import KeyError
 import itertools
 import shutil
 import tempfile
+
 from tests.testingutils import Turtle
 
 
 class MockAction(Turtle):
-
     def __init__(self, *args, **kwargs):
         kwargs.setdefault('name', 'action_name')
         kwargs.setdefault('required_actions', [])
@@ -17,7 +19,6 @@ class MockAction(Turtle):
 
 
 class MockActionGraph(Turtle):
-
     def __init__(self, *args, **kwargs):
         action = MockAction()
         kwargs.setdefault('graph', [action])
@@ -34,7 +35,6 @@ class MockActionGraph(Turtle):
 
 
 class MockActionRun(Turtle):
-
     def __init__(self, *args, **kwargs):
         kwargs.setdefault('output_path', [tempfile.mkdtemp()])
         kwargs.setdefault('start_time', datetime.datetime.now())
@@ -44,7 +44,6 @@ class MockActionRun(Turtle):
 
 
 class MockActionRunCollection(Turtle):
-
     def __init__(self, *args, **kwargs):
         kwargs.setdefault('action_graph', MockActionGraph())
         kwargs.setdefault('run_map', {})
@@ -57,18 +56,18 @@ class MockActionRunCollection(Turtle):
 
 
 class MockJobRun(Turtle):
-
     def __init__(self, *args, **kwargs):
         kwargs.setdefault('output_path', [tempfile.mkdtemp()])
         kwargs.setdefault('action_graph', MockActionGraph())
-        action_runs = MockActionRunCollection(action_graph=kwargs['action_graph'])
+        action_runs = MockActionRunCollection(
+            action_graph=kwargs['action_graph'],
+        )
         kwargs.setdefault('action_runs', action_runs)
         atexit.register(lambda: shutil.rmtree(kwargs['output_path'][0]))
         super(MockJobRun, self).__init__(*args, **kwargs)
 
 
 class MockNode(Turtle):
-
     def __init__(self, hostname=None):
         super(MockNode, self).__init__()
         self.name = self.hostname = hostname
@@ -102,7 +101,7 @@ class MockNodePool(object):
             self.nodes.append(MockNode())
 
         if self._ndx_cycle:
-            return self.nodes[self._ndx_cycle.next()]
+            return self.nodes[next(self._ndx_cycle)]
         else:
             return self.nodes[0]
 
@@ -110,6 +109,5 @@ class MockNodePool(object):
 
 
 class MockJobRunCollection(Turtle):
-
     def __iter__(self):
-        return iter(self.runs)
\ No newline at end of file
+        return iter(self.runs)
