@@ -1,11 +1,22 @@
-import mock
-from testify import setup, TestCase, assert_equal, run
-from testify import assert_in, assert_raises
-from testify.assertions import assert_not_in, assert_not_equal
-from testify.test_case import teardown, setup_teardown
-from tests.testingutils import autospec_method
+from __future__ import absolute_import
+from __future__ import unicode_literals
 
-from tron import node, ssh, actioncommand
+import mock
+from testify import assert_equal
+from testify import assert_in
+from testify import assert_raises
+from testify import run
+from testify import setup
+from testify import setup_teardown
+from testify import teardown
+from testify import TestCase
+from testify.assertions import assert_not_equal
+from testify.assertions import assert_not_in
+
+from tests.testingutils import autospec_method
+from tron import actioncommand
+from tron import node
+from tron import ssh
 from tron.config import schema
 from tron.core import actionrun
 from tron.serialize import filehandler
@@ -23,7 +34,6 @@ def create_mock_pool():
 
 
 class NodePoolRepositoryTestCase(TestCase):
-
     @setup
     def setup_store(self):
         self.node = create_mock_node()
@@ -57,16 +67,25 @@ class NodePoolRepositoryTestCase(TestCase):
         node_pool_config = {'c': mock.Mock(nodes=['a', 'b'])}
         ssh_options = mock.Mock(identities=[], known_hosts_file=None)
         node.NodePoolRepository.update_from_config(
-            node_config, node_pool_config, ssh_options)
+            node_config,
+            node_pool_config,
+            ssh_options,
+        )
         node_names = [node_config['a'].name, node_config['b'].name]
-        assert_equal(set(self.repo.pools), set(node_names + [node_pool_config['c'].name]))
-        assert_equal(set(self.repo.nodes), set(node_names + mock_nodes.keys()))
+        assert_equal(
+            set(self.repo.pools),
+            set(node_names + [node_pool_config['c'].name], ),
+        )
+        assert_equal(
+            set(self.repo.nodes),
+            set(list(node_names) + list(mock_nodes.keys())),
+        )
 
     def test_nodes_by_name(self):
         mock_nodes = {'a': mock.Mock(), 'b': mock.Mock()}
         self.repo.nodes.update(mock_nodes)
         nodes = self.repo._get_nodes_by_name(['a', 'b'])
-        assert_equal(nodes, mock_nodes.values())
+        assert_equal(nodes, list(mock_nodes.values()))
 
     def test_get_node(self):
         returned_node = self.repo.get_node(self.node.get_name())
@@ -74,12 +93,11 @@ class NodePoolRepositoryTestCase(TestCase):
 
 
 class KnownHostTestCase(TestCase):
-
     @setup
     def setup_known_hosts(self):
         self.known_hosts = node.KnownHosts(None)
         self.entry = mock.Mock()
-        self.known_hosts._entries.append(self.entry)
+        self.known_hosts._added.append(self.entry)
 
     def test_get_public_key(self):
         hostname = 'hostname'
@@ -93,11 +111,13 @@ class KnownHostTestCase(TestCase):
 
 
 class DetermineJitterTestCase(TestCase):
-
     @setup
     def setup_node_settings(self):
         self.settings = mock.Mock(
-                jitter_load_factor=1, jitter_min_load=4, jitter_max_delay=20)
+            jitter_load_factor=1,
+            jitter_min_load=4,
+            jitter_max_delay=20,
+        )
 
     @setup_teardown
     def patch_random(self):
@@ -121,7 +141,11 @@ class DetermineJitterTestCase(TestCase):
 
 
 def build_node(
-        hostname='localhost', username='theuser', name='thename', pub_key=None):
+    hostname='localhost',
+    username='theuser',
+    name='thename',
+    pub_key=None,
+):
     config = mock.Mock(hostname=hostname, username=username, name=name)
     ssh_opts = mock.create_autospec(ssh.SSHAuthOptions)
     node_settings = mock.create_autospec(schema.ConfigSSHOptions)
@@ -129,7 +153,6 @@ def build_node(
 
 
 class NodeTestCase(TestCase):
-
     class TestConnection(object):
         def openChannel(self, chan):
             self.chan = chan
@@ -146,6 +169,7 @@ class NodeTestCase(TestCase):
         test_node.connection = self.TestConnection()
         test_node.run_states = {action_cmd.id: mock.Mock(state=0)}
         test_node.run_states[action_cmd.id].state = node.RUN_STATE_CONNECTING
+        test_node.run_states[action_cmd.id].run = action_cmd
 
         test_node._open_channel(action_cmd)
         assert test_node.connection.chan is not None
@@ -154,12 +178,20 @@ class NodeTestCase(TestCase):
 
     def test_from_config(self):
         ssh_options = self.node.conch_options
-        node_config = mock.Mock(hostname='localhost', username='theuser', name='thename')
+        node_config = mock.Mock(
+            hostname='localhost',
+            username='theuser',
+            name='thename',
+        )
         ssh_options.__getitem__.return_value = 'something'
         public_key = mock.Mock()
         node_settings = mock.Mock()
         new_node = node.Node.from_config(
-                node_config, ssh_options, public_key, node_settings)
+            node_config,
+            ssh_options,
+            public_key,
+            node_settings,
+        )
         assert_equal(new_node.name, node_config.name)
         assert_equal(new_node.hostname, node_config.hostname)
         assert_equal(new_node.username, node_config.username)
@@ -187,24 +219,27 @@ class NodeTestCase(TestCase):
         assert_not_equal(other_node, self.node)
 
     def test_stop_not_tracked(self):
-        action_command = mock.create_autospec(actioncommand.ActionCommand,
-            id=mock.Mock())
+        action_command = mock.create_autospec(
+            actioncommand.ActionCommand,
+            id=mock.Mock(),
+        )
         self.node.stop(action_command)
 
     def test_stop(self):
         autospec_method(self.node._fail_run)
-        action_command = mock.create_autospec(actioncommand.ActionCommand,
-            id=mock.Mock())
+        action_command = mock.create_autospec(
+            actioncommand.ActionCommand,
+            id=mock.Mock(),
+        )
         self.node.run_states[action_command.id] = mock.Mock()
         self.node.stop(action_command)
         assert_equal(self.node._fail_run.call_count, 1)
 
 
 class NodePoolTestCase(TestCase):
-
     @setup
     def setup_nodes(self):
-        self.nodes = [build_node(name='node%s' % i) for i in xrange(5)]
+        self.nodes = [build_node(name='node%s' % i) for i in range(5)]
         self.node_pool = node.NodePool(self.nodes, 'thename')
 
     def test_from_config(self):
@@ -225,13 +260,13 @@ class NodePoolTestCase(TestCase):
 
     def test_next(self):
         # Call next many times
-        for _ in xrange(len(self.nodes) * 2 + 1):
+        for _ in range(len(self.nodes) * 2 + 1):
             assert_in(self.node_pool.next(), self.nodes)
 
     def test_next_round_robin(self):
         node_order = [
             self.node_pool.next_round_robin()
-            for _ in xrange(len(self.nodes) * 2)
+            for _ in range(len(self.nodes) * 2)
         ]
         assert_equal(node_order, self.nodes + self.nodes)
 
