@@ -1,26 +1,33 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
+import contextlib
+import functools
 import logging
 import os
 import shutil
 import signal
 import socket
-from subprocess import Popen, PIPE, CalledProcessError
 import sys
 import tempfile
 import time
-import contextlib
-import functools
-import mock
+from subprocess import CalledProcessError
+from subprocess import PIPE
+from subprocess import Popen
 
-from testify import TestCase, setup, teardown
+import mock
+from testify import setup
+from testify import teardown
+from testify import TestCase
 from testify.assertions import assert_not_equal
 
 from tron.commands import client
-from tron.config import manager, schema
-
+from tron.config import manager
+from tron.config import schema
 
 # Used for getting the locations of the executable
 test_dir, _ = os.path.split(__file__)
-repo_root, _   = os.path.split(test_dir)
+repo_root, _ = os.path.split(test_dir)
 
 log = logging.getLogger(__name__)
 
@@ -39,8 +46,10 @@ def wait_on_sandbox(func, delay=0.1, max_wait=5.0):
 
 def wait_on_state(client_func, url, state, field='state'):
     """Use client_func(url) to wait until the resource changes to state."""
+
     def wait_func():
         return client_func(url)[field] == state
+
     wait_func.__name__ = '%s wait on %s' % (url, state)
     wait_on_sandbox(wait_func)
 
@@ -49,8 +58,9 @@ def wait_on_proc_terminate(pid):
     def wait_on_terminate():
         try:
             os.kill(pid, 0)
-        except:
+        except Exception:
             return True
+
     wait_on_terminate.__name__ = "Wait on %s to terminate" % pid
     wait_on_sandbox(wait_on_terminate)
 
@@ -59,10 +69,11 @@ def build_waiter_func(client_func, url):
     return functools.partial(wait_on_state, client_func, url)
 
 
-def handle_output(cmd, (stdout, stderr), returncode):
+def handle_output(cmd, out_err, returncode):
     """Log process output before it is parsed. Raise exception if exit code
     is nonzero.
     """
+    stdout, stderr = out_err
     cmd = ' '.join(cmd)
     if stdout:
         log.warn("%s STDOUT: %s", cmd, stdout)
@@ -122,8 +133,8 @@ class ClientProxy(object):
     """
 
     def __init__(self, client, log_filename):
-        self.client         = client
-        self.log_filename   = log_filename
+        self.client = client
+        self.log_filename = log_filename
 
     def log_contents(self):
         """Return the contents of the log file."""
@@ -134,7 +145,7 @@ class ClientProxy(object):
         with mock.patch('tron.commands.client.log'):
             try:
                 return func(*args, **kwargs)
-            except (client.RequestError, ValueError), e:
+            except (client.RequestError, ValueError) as e:
                 # ValueError for JSONDecode errors
                 log_contents = self.log_contents()
                 if log_contents:
@@ -152,7 +163,9 @@ class ClientProxy(object):
 def verify_environment():
     for env_var in ['SSH_AUTH_SOCK', 'PYTHONPATH']:
         if not os.environ.get(env_var):
-            raise TronSandboxException("Missing $%s in test environment." % env_var)
+            raise TronSandboxException(
+                "Missing $%s in test environment." % env_var,
+            )
 
 
 class TronSandbox(object):
@@ -160,19 +173,19 @@ class TronSandbox(object):
 
     def __init__(self):
         """Set up a temp directory and store paths to relevant binaries"""
-        self.tmp_dir        = tempfile.mkdtemp(prefix='tron-')
-        cmd_path_func       = functools.partial(os.path.join, repo_root, 'bin')
-        cmds                = 'tronctl', 'trond', 'tronfig', 'tronview'
-        self.commands       = dict((cmd, cmd_path_func(cmd)) for cmd in cmds)
-        self.log_file       = self.abs_path('tron.log')
-        self.log_conf       = self.abs_path('logging.conf')
-        self.pid_file       = self.abs_path('tron.pid')
-        self.config_path    = self.abs_path('configs/')
-        self.port           = find_unused_port()
-        self.host           = 'localhost'
-        self.api_uri        = 'http://%s:%s' % (self.host, self.port)
-        cclient             = client.Client(self.api_uri)
-        self.client         = ClientProxy(cclient, self.log_file)
+        self.tmp_dir = tempfile.mkdtemp(prefix='tron-')
+        cmd_path_func = functools.partial(os.path.join, repo_root, 'bin')
+        cmds = 'tronctl', 'trond', 'tronfig', 'tronview'
+        self.commands = {cmd: cmd_path_func(cmd) for cmd in cmds}
+        self.log_file = self.abs_path('tron.log')
+        self.log_conf = self.abs_path('logging.conf')
+        self.pid_file = self.abs_path('tron.pid')
+        self.config_path = self.abs_path('configs/')
+        self.port = find_unused_port()
+        self.host = 'localhost'
+        self.api_uri = 'http://%s:%s' % (self.host, self.port)
+        cclient = client.Client(self.api_uri)
+        self.client = ClientProxy(cclient, self.log_file)
         self.setup_logging_conf()
 
     def abs_path(self, filename):
@@ -198,11 +211,11 @@ class TronSandbox(object):
 
     def run_command(self, command_name, args=None, stdin_lines=None):
         """Run the command by name and return (stdout, stderr)."""
-        args        = args or []
-        command     = [sys.executable, self.commands[command_name]] + args
-        stdin       = PIPE if stdin_lines else None
-        proc        = Popen(command, stdout=PIPE, stderr=PIPE, stdin=stdin)
-        streams     = proc.communicate(stdin_lines)
+        args = args or []
+        command = [sys.executable, self.commands[command_name]] + args
+        stdin = PIPE if stdin_lines else None
+        proc = Popen(command, stdout=PIPE, stderr=PIPE, stdin=stdin)
+        streams = proc.communicate(stdin_lines)
         try:
             handle_output(command, streams, proc.returncode)
         except CalledProcessError:
@@ -221,20 +234,24 @@ class TronSandbox(object):
 
     def trond(self, *args):
         args = list(args) if args else []
-        args += ['--working-dir=%s'     % self.tmp_dir,
-                   '--pid-file=%s'      % self.pid_file,
-                   '--port=%d'          % self.port,
-                   '--host=%s'          % self.host,
-                   '--config-path=%s'   % self.config_path,
-                   '--log-conf=%s'      % self.log_conf]
+        args += [
+            '--working-dir=%s' % self.tmp_dir,
+            '--pid-file=%s' % self.pid_file,
+            '--port=%d' % self.port,
+            '--host=%s' % self.host,
+            '--config-path=%s' % self.config_path,
+            '--log-conf=%s' % self.log_conf,
+        ]
 
         self.run_command('trond', args)
         wait_on_sandbox(lambda: bool(self.client.home()))
 
-    def tronfig(self,
-                config_content=None,
-                name=schema.MASTER_NAMESPACE,
-                no_header=False):
+    def tronfig(
+        self,
+        config_content=None,
+        name=schema.MASTER_NAMESPACE,
+        no_header=False,
+    ):
         args = ['--server', self.api_uri, name]
         if no_header:
             args += ['--no-header']
