@@ -1,27 +1,36 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 import calendar
 import datetime
+
 import mock
 import pytz
-
-from testify import setup, run, assert_equal, TestCase
-from testify import assert_gte, assert_lte, assert_gt, assert_lt
+from testify import assert_equal
+from testify import assert_gt
+from testify import assert_gte
+from testify import assert_lt
+from testify import assert_lte
+from testify import run
+from testify import setup
 from testify import setup_teardown
-from tests import testingutils
+from testify import TestCase
 
+from tests import testingutils
 from tron import scheduler
-from tron.config import schedule_parse, config_utils
+from tron.config import config_utils
+from tron.config import schedule_parse
 from tron.config.config_utils import NullConfigContext
 from tron.config.schedule_parse import parse_groc_expression
 from tron.utils import timeutils
 
 
 class SchedulerFromConfigTestCase(TestCase):
-
     def test_cron_scheduler(self):
         line = "cron */5 * * 7,8 *"
         config_context = mock.Mock(path='test')
         config = schedule_parse.valid_schedule(line, config_context)
-        sched = scheduler.scheduler_from_config(config, mock.Mock())
+        sched = scheduler.scheduler_from_config(config=config, time_zone=None)
         start_time = datetime.datetime(2012, 3, 14, 15, 9, 26)
         next_time = sched.next_run_time(start_time)
         assert_equal(next_time, datetime.datetime(2012, 7, 1, 0))
@@ -31,7 +40,7 @@ class SchedulerFromConfigTestCase(TestCase):
         config_context = config_utils.NullConfigContext
         line = "daily 17:32 MWF"
         config = schedule_parse.valid_schedule(line, config_context)
-        sched = scheduler.scheduler_from_config(config, mock.Mock())
+        sched = scheduler.scheduler_from_config(config=config, time_zone=None)
         assert_equal(sched.time_spec.hours, [17])
         assert_equal(sched.time_spec.minutes, [32])
         start_time = datetime.datetime(2012, 3, 14, 15, 9, 26)
@@ -117,7 +126,11 @@ class GeneralSchedulerTodayTest(GeneralSchedulerTimeTestBase):
 
         assert_equal(next_run_date, self.now.date())
         earlier_time = datetime.datetime(
-            self.now.year, self.now.month, self.now.day, hour=13)
+            self.now.year,
+            self.now.month,
+            self.now.day,
+            hour=13,
+        )
         assert_lte(earlier_time, run_time)
 
 
@@ -132,8 +145,12 @@ class GeneralSchedulerTomorrowTest(GeneralSchedulerTimeTestBase):
         tomorrow = self.now.date() + datetime.timedelta(days=1)
 
         assert_equal(next_run_date, tomorrow)
-        earlier_time = datetime.datetime(year=tomorrow.year, month=tomorrow.month,
-            day=tomorrow.day, hour=13)
+        earlier_time = datetime.datetime(
+            year=tomorrow.year,
+            month=tomorrow.month,
+            day=tomorrow.day,
+            hour=13,
+        )
         assert_lte(earlier_time, run_time)
 
 
@@ -158,6 +175,7 @@ class GeneralSchedulerLongJobRunTest(GeneralSchedulerTimeTestBase):
 class GeneralSchedulerDSTTest(testingutils.MockTimeTestCase):
 
     now = datetime.datetime(2011, 11, 6, 1, 10, 0)
+    now_utc = timeutils.current_time(tz=pytz.timezone('UTC'))
 
     def hours_until_time(self, run_time, sch):
         tz = sch.time_zone
@@ -232,6 +250,23 @@ class GeneralSchedulerDSTTest(testingutils.MockTimeTestCase):
         self._assert_range(s2b - s2a, 23.99, 24.11)
         self._assert_range(s1a - s2a, -0.61, -0.59)
 
+    def test_handles_tz_specific_jobs_with_tz_specific_start_time(self):
+        sch = scheduler.GeneralScheduler(time_zone=pytz.timezone('UTC'))
+        next_run_time = sch.next_run_time(self.now_utc)
+        assert_equal(next_run_time.hour, 0)
+
+    def test_handles_unsetting_the_time_zone(self):
+        sch = scheduler.GeneralScheduler(time_zone=None)
+        next_run_time = sch.next_run_time(self.now_utc)
+        assert_equal(next_run_time.hour, 0)
+
+    def test_handles_changing_the_time_zone(self):
+        pacific_now = datetime.datetime.now(pytz.timezone('US/Pacific'))
+        pacific_offset = pacific_now.utcoffset().total_seconds() / 60 / 60
+        sch = scheduler.GeneralScheduler(time_zone=pytz.timezone('US/Pacific'))
+        next_run_time = sch.next_run_time(self.now_utc)
+        assert_equal(next_run_time.hour, -pacific_offset)
+
 
 def parse_groc(config):
     config = schedule_parse.ConfigGenericSchedule('groc daily', config, None)
@@ -249,26 +284,30 @@ class ComplexParserTest(testingutils.MockTimeTestCase):
     def test_parse_all(self):
         config_string = '1st,2nd,3rd,4th monday,Tue of march,apr,September at 00:00'
         cfg = parse_groc(config_string)
-        assert_equal(cfg.ordinals, set((1, 2, 3, 4)))
+        assert_equal(cfg.ordinals, {1, 2, 3, 4})
         assert_equal(cfg.monthdays, None)
-        assert_equal(cfg.weekdays, set((1, 2)))
-        assert_equal(cfg.months, set((3, 4, 9)))
+        assert_equal(cfg.weekdays, {1, 2})
+        assert_equal(cfg.months, {3, 4, 9})
         assert_equal(cfg.timestr, '00:00')
-        assert_equal(scheduler_from_config(config_string),
-                     scheduler_from_config(config_string))
+        assert_equal(
+            scheduler_from_config(config_string),
+            scheduler_from_config(config_string),
+        )
 
     def test_parse_no_weekday(self):
-        cfg = parse_groc('1st,2nd,3rd,10th day of march,apr,September at 00:00')
+        cfg = parse_groc(
+            '1st,2nd,3rd,10th day of march,apr,September at 00:00',
+        )
         assert_equal(cfg.ordinals, None)
-        assert_equal(cfg.monthdays, set((1,2,3,10)))
+        assert_equal(cfg.monthdays, {1, 2, 3, 10})
         assert_equal(cfg.weekdays, None)
-        assert_equal(cfg.months, set((3, 4, 9)))
+        assert_equal(cfg.months, {3, 4, 9})
         assert_equal(cfg.timestr, '00:00')
 
     def test_parse_no_month(self):
         cfg = parse_groc('1st,2nd,3rd,10th day at 00:00')
         assert_equal(cfg.ordinals, None)
-        assert_equal(cfg.monthdays, set((1,2,3,10)))
+        assert_equal(cfg.monthdays, {1, 2, 3, 10})
         assert_equal(cfg.weekdays, None)
         assert_equal(cfg.months, None)
         assert_equal(cfg.timestr, '00:00')
@@ -277,7 +316,7 @@ class ComplexParserTest(testingutils.MockTimeTestCase):
         for test_str in ('1st day', '1st day of month'):
             cfg = parse_groc(test_str)
             assert_equal(cfg.ordinals, None)
-            assert_equal(cfg.monthdays, set([1]))
+            assert_equal(cfg.monthdays, {1})
             assert_equal(cfg.weekdays, None)
             assert_equal(cfg.months, None)
             assert_equal(cfg.timestr, '00:00')
@@ -315,22 +354,32 @@ class ComplexParserTest(testingutils.MockTimeTestCase):
         next_run_date = sch.next_run_time(None)
 
         assert_gte(next_run_date, self.now)
-        assert_equal(calendar.weekday(next_run_date.year,
-                                      next_run_date.month,
-                                      next_run_date.day), 0)
+        assert_equal(
+            calendar.weekday(
+                next_run_date.year,
+                next_run_date.month,
+                next_run_date.day,
+            ),
+            0,
+        )
 
     def test_weekly_in_month(self):
         sch = scheduler_from_config('every monday of january at 00:01')
         next_run_date = sch.next_run_time(None)
 
         assert_gte(next_run_date, self.now)
-        assert_equal(next_run_date.year, self.now.year+1)
+        assert_equal(next_run_date.year, self.now.year + 1)
         assert_equal(next_run_date.month, 1)
         assert_equal(next_run_date.hour, 0)
         assert_equal(next_run_date.minute, 1)
-        assert_equal(calendar.weekday(next_run_date.year,
-                                      next_run_date.month,
-                                      next_run_date.day), 0)
+        assert_equal(
+            calendar.weekday(
+                next_run_date.year,
+                next_run_date.month,
+                next_run_date.day,
+            ),
+            0,
+        )
 
     def test_monthly(self):
         sch = scheduler_from_config('1st day')
@@ -346,7 +395,9 @@ class IntervalSchedulerTestCase(TestCase):
 
     @setup_teardown
     def patch_time(self):
-        with mock.patch('tron.scheduler.timeutils.current_time') as self.mock_now:
+        with mock.patch(
+            'tron.scheduler.timeutils.current_time',
+        ) as self.mock_now:
             self.mock_now.return_value = self.now
             yield
 
@@ -354,7 +405,11 @@ class IntervalSchedulerTestCase(TestCase):
     def build_scheduler(self):
         self.seconds = 7
         self.interval = datetime.timedelta(seconds=self.seconds)
-        self.scheduler = scheduler.IntervalScheduler(self.interval, None)
+        self.scheduler = scheduler.IntervalScheduler(
+            interval=self.interval,
+            jitter=None,
+            time_zone=None,
+        )
 
     def test_next_run_time_no_jitter(self):
         prev_run_time = datetime.datetime(2011, 5, 21)
@@ -371,7 +426,11 @@ class IntervalSchedulerTestCase(TestCase):
         mock_random.randint.return_value = random_jitter = -200
         random_delta = datetime.timedelta(seconds=random_jitter)
         prev_run_time = datetime.datetime(2011, 5, 21)
-        interval_sched = scheduler.IntervalScheduler(self.interval, jitter)
+        interval_sched = scheduler.IntervalScheduler(
+            interval=self.interval,
+            jitter=jitter,
+            time_zone=None,
+        )
         run_time = interval_sched.next_run_time(prev_run_time)
         assert_equal(run_time, prev_run_time + random_delta + self.interval)
 
