@@ -1,17 +1,24 @@
-import logging
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 import functools
-import mock
-
-from testify import  TestCase, setup
-from testify import class_setup, class_teardown
-from testify import teardown
+import logging
 import time
-from tron.utils import timeutils
 
+import mock
+from testify import class_setup
+from testify import class_teardown
+from testify import setup
+from testify import teardown
+from testify import TestCase
+
+from tron.utils import timeutils
 
 log = logging.getLogger(__name__)
 
 # TODO: remove when replaced with tron.eventloop
+
+
 class MockReactorTestCase(TestCase):
     """Patch the reactor to a MockReactor."""
 
@@ -42,8 +49,8 @@ class MockTimeTestCase(TestCase):
     @setup
     def setup_current_time(self):
         assert self.now, "%s must set a now field" % self.__class__
-        self.old_current_time  = timeutils.current_time
-        timeutils.current_time = lambda: self.now
+        self.old_current_time = timeutils.current_time
+        timeutils.current_time = lambda tz=None: self.now
 
     @teardown
     def teardown_current_time(self):
@@ -57,22 +64,26 @@ def retry(max_tries=3, delay=0.1, exceptions=(KeyError, IndexError)):
     """A function decorator for re-trying an operation. Useful for MongoDB
     which is only eventually consistent.
     """
+
     def wrapper(f):
         @functools.wraps(f)
         def wrap(*args, **kwargs):
-            for _ in xrange(max_tries):
+            for _ in range(max_tries):
                 try:
                     return f(*args, **kwargs)
                 except exceptions:
                     time.sleep(delay)
             raise
+
         return wrap
+
     return wrapper
 
 
 # TODO: remove when replaced with mock
 class Turtle(object):
     """A more complete Mock implementation."""
+
     def __init__(self, *args, **kwargs):
         self.__dict__.update(kwargs)
         self.calls = []
@@ -92,4 +103,4 @@ class Turtle(object):
 def autospec_method(method, *args, **kwargs):
     """create an autospec for an instance method."""
     mocked_method = mock.create_autospec(method, *args, **kwargs)
-    setattr(method.im_self, method.__name__, mocked_method)
+    setattr(method.__self__, method.__name__, mocked_method)
