@@ -1,14 +1,20 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 import mock
-from testify import TestCase, setup
-from testify.assertions import assert_in, assert_raises, assert_not_in
+from testify import setup
+from testify import TestCase
 from testify.assertions import assert_equal
+from testify.assertions import assert_in
+from testify.assertions import assert_not_in
+from testify.assertions import assert_raises
+
 from tests.assertions import assert_mock_calls
 from tests.testingutils import autospec_method
 from tron.utils import collections
 
 
 class MappingCollectionsTestCase(TestCase):
-
     @setup
     def setup_collection(self):
         self.name = 'some_name'
@@ -31,13 +37,6 @@ class MappingCollectionsTestCase(TestCase):
         assert_not_in(name, self.collection)
         item.disable.assert_called_with()
 
-    def test_restore_state(self):
-        state_data = {'a': mock.Mock(), 'b': mock.Mock()}
-        self.collection.update({'a': mock.Mock(), 'b': mock.Mock()})
-        self.collection.restore_state(state_data)
-        for key in state_data:
-            self.collection[key].restore_state.assert_called_with(state_data[key])
-
     def test_contains_item_false(self):
         mock_item, mock_func = mock.Mock(), mock.Mock()
         assert not self.collection.contains_item(mock_item, mock_func)
@@ -71,11 +70,13 @@ class MappingCollectionsTestCase(TestCase):
         autospec_method(self.collection.add)
         item = mock.Mock()
         self.collection.replace(item)
-        self.collection.add.assert_called_with(item, self.collection.remove_item)
+        self.collection.add.assert_called_with(
+            item,
+            self.collection.remove_item,
+        )
 
 
 class EnumTestCase(TestCase):
-
     @setup
     def setup_enum(self):
         self.values = ['one', 'two', 'three']
@@ -99,4 +100,4 @@ class EnumTestCase(TestCase):
         assert_raises(AttributeError, lambda: self.enum.seven)
 
     def test__iter__(self):
-        assert_equal(set(self.enum), set(self.values))
\ No newline at end of file
+        assert_equal(set(self.enum), set(self.values))
