@@ -1,16 +1,26 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 import os
 import tempfile
-from testify import TestCase, run, assert_equal, setup, class_teardown, teardown
+
+from testify import assert_equal
+from testify import class_teardown
+from testify import run
+from testify import setup
+from testify import teardown
+from testify import TestCase
+
 from tests.assertions import assert_raises
 from tron.utils import tool_utils
 
-class WorkingDirTestCase(TestCase):
 
+class WorkingDirTestCase(TestCase):
     @setup
     def setup_cwd(self):
         self.cwd = os.getcwd()
-        self.temp_dir = tempfile.mkdtemp()
-        self.second_dir = tempfile.mkdtemp()
+        self.temp_dir = os.path.realpath(tempfile.mkdtemp())
+        self.second_dir = os.path.realpath(tempfile.mkdtemp())
 
     @teardown
     def cleanup(self):
@@ -44,4 +54,4 @@ class WorkingDirTestCase(TestCase):
 
 
 if __name__ == "__main__":
-    run()
\ No newline at end of file
+    run()
