@@ -1,9 +1,10 @@
-__version_info__    = (0, 6, 1, 1)
-__version__         = ".".join("%s" % v for v in __version_info__)
+from __future__ import absolute_import
+from __future__ import unicode_literals
 
-__author__          = 'Yelp <yelplabs@yelp.com>'
-
-__credits__         = [
+__version_info__ = (0, 9, 0, 6)
+__version__ = ".".join("%s" % v for v in __version_info__)
+__author__ = 'Yelp <yelplabs@yelp.com>'
+__credits__ = [
     'Rhett Garber <rhettg@gmail.com>',
     'Eugene Baumstein <eugene.baumstein@gmail>',
     'James Brown <jbrown@yelp.com>',
