@@ -1,16 +1,22 @@
 """Utilities used for configuration parsing and validation."""
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
+import datetime
 import functools
 import itertools
 import re
-import datetime
+
+import six
+from six import string_types
+
 from tron.config import ConfigError
 from tron.config.schema import MASTER_NAMESPACE
 from tron.utils import dicts
 from tron.utils.dicts import FrozenDict
 
-
-MAX_IDENTIFIER_LENGTH       = 255
-IDENTIFIER_RE               = re.compile(r'^[A-Za-z_][\w\-]{0,254}$')
+MAX_IDENTIFIER_LENGTH = 255
+IDENTIFIER_RE = re.compile(r'^[A-Za-z_][\w\-]{0,254}$')
 
 
 class UniqueNameDict(dict):
@@ -20,6 +26,7 @@ class UniqueNameDict(dict):
      fmt_string - format string used to create an error message, expects a
                   single format argument of 'key'
     """
+
     def __init__(self, fmt_string):
         super(dict, self).__init__()
         self.fmt_string = fmt_string
@@ -48,10 +55,12 @@ def build_type_validator(validator, error_fmt):
             config_context - a ConfigContext object
             Returns True if the value is valid
     """
+
     def f(value, config_context):
         if not validator(value):
             raise ConfigError(error_fmt % (config_context.path, value))
         return value
+
     return f
 
 
@@ -68,24 +77,34 @@ def valid_number(type_func, value, config_context):
 
     return value
 
-valid_int   = functools.partial(valid_number, int)
+
+valid_int = functools.partial(valid_number, int)
 valid_float = functools.partial(valid_number, float)
 
 valid_identifier = build_type_validator(
-    lambda s: isinstance(s, basestring) and IDENTIFIER_RE.match(s),
-    'Identifier at %s is not a valid identifier: %s')
+    lambda s: isinstance(s, string_types) and IDENTIFIER_RE.match(s),
+    'Identifier at %s is not a valid identifier: %s',
+)
 
 valid_list = build_type_validator(
-    lambda s: isinstance(s, list), 'Value at %s is not a list: %s')
+    lambda s: isinstance(s, list),
+    'Value at %s is not a list: %s',
+)
 
-valid_string  = build_type_validator(
-    lambda s: isinstance(s, basestring), 'Value at %s is not a string: %s')
+valid_string = build_type_validator(
+    lambda s: isinstance(s, string_types),
+    'Value at %s is not a string: %s',
+)
 
 valid_dict = build_type_validator(
-    lambda s: isinstance(s, dict), 'Value at %s is not a dictionary: %s')
+    lambda s: isinstance(s, dict),
+    'Value at %s is not a dictionary: %s',
+)
 
 valid_bool = build_type_validator(
-    lambda s: isinstance(s, bool), 'Value at %s is not a boolean: %s')
+    lambda s: isinstance(s, bool),
+    'Value at %s is not a boolean: %s',
+)
 
 
 def build_enum_validator(enum):
@@ -99,24 +118,26 @@ def valid_time(value, config_context):
     for format in ['%H:%M', '%H:%M:%S']:
         try:
             return datetime.datetime.strptime(value, format)
-        except ValueError, exc:
+        except ValueError:
             pass
-
-    msg = 'Value at %s is not a valid time: %s'
-    raise ConfigError(msg % (config_context.path, exc))
+    msg = 'Value at %s is not a valid time'
+    raise ConfigError(msg % config_context.path)
 
 
 # Translations from possible configuration units to the argument to
 # datetime.timedelta
-TIME_INTERVAL_UNITS = dicts.invert_dict_list({
-    'days':     ['d', 'day', 'days'],
-    'hours':    ['h', 'hr', 'hrs', 'hour', 'hours'],
-    'minutes':  ['m', 'min', 'mins', 'minute', 'minutes'],
-    'seconds':  ['s', 'sec', 'secs', 'second', 'seconds']
-})
+TIME_INTERVAL_UNITS = dicts.invert_dict_list(
+    {
+        'days': ['d', 'day', 'days'],
+        'hours': ['h', 'hr', 'hrs', 'hour', 'hours'],
+        'minutes': ['m', 'min', 'mins', 'minute', 'minutes'],
+        'seconds': ['s', 'sec', 'secs', 'second', 'seconds'],
+    }
+)
 
 TIME_INTERVAL_RE = re.compile(r"^\s*(?P<value>\d+)\s*(?P<units>[a-zA-Z]+)\s*$")
 
+
 def valid_time_delta(value, config_context):
     error_msg = "Value at %s is not a valid time delta: %s"
     matches = TIME_INTERVAL_RE.match(value)
@@ -142,6 +163,7 @@ def build_list_of_type_validator(item_validator, allow_empty=False):
     """Build a validator which validates a list contains items which pass
     item_validator.
     """
+
     def validator(value, config_context):
         if allow_empty and not value:
             return ()
@@ -150,18 +172,21 @@ def build_list_of_type_validator(item_validator, allow_empty=False):
             msg = "Required non-empty list at %s"
             raise ConfigError(msg % config_context.path)
         return tuple(item_validator(item, config_context) for item in seq)
+
     return validator
 
 
 def build_dict_name_validator(item_validator, allow_empty=False):
     """Build a validator which validates a list, and returns a dict."""
     valid = build_list_of_type_validator(item_validator, allow_empty)
+
     def validator(value, config_context):
         msg = "Duplicate name %%s at %s" % config_context.path
         name_dict = UniqueNameDict(msg)
         for item in valid(value, config_context):
             name_dict[item.name] = item
         return FrozenDict(**name_dict)
+
     return validator
 
 
@@ -172,16 +197,17 @@ class ConfigContext(object):
     """
     partial = False
 
-    def __init__(self, path, nodes, command_context, namespace):
+    def __init__(self, path, nodes, clusters, command_context, namespace):
         self.path = path
         self.nodes = set(nodes or [])
+        self.clusters = set(clusters or [])
         self.command_context = command_context or {}
         self.namespace = namespace
 
     def build_child_context(self, path):
         """Construct a new ConfigContext based on this one."""
         path = '%s.%s' % (self.path, path)
-        args = path, self.nodes, self.command_context, self.namespace
+        args = path, self.nodes, self.clusters, self.command_context, self.namespace
         return ConfigContext(*args)
 
 
@@ -205,6 +231,7 @@ class PartialConfigContext(object):
 class NullConfigContext(object):
     path = ''
     nodes = set()
+    clusters = set()
     command_context = {}
     namespace = MASTER_NAMESPACE
     partial = False
@@ -219,10 +246,10 @@ class Validator(object):
     """Base class for validating a collection and creating a mutable
     collection from the source.
     """
-    config_class            = None
-    defaults                = {}
-    validators              = {}
-    optional                = False
+    config_class = None
+    defaults = {}
+    validators = {}
+    optional = False
 
     def validate(self, in_dict, config_context):
         if self.optional and in_dict is None:
@@ -281,7 +308,7 @@ class Validator(object):
 
         missing_key_str = ', '.join(missing_keys)
         if 'name' in self.all_keys and 'name' in in_dict:
-            msg  = "%s %s is missing options: %s"
+            msg = "%s %s is missing options: %s"
             name = in_dict['name']
             raise ConfigError(msg % (self.type_name, name, missing_key_str))
 
@@ -294,7 +321,7 @@ class Validator(object):
         if not extra_keys:
             return
 
-        msg  = "Unknown keys in %s %s: %s"
+        msg = "Unknown keys in %s %s: %s"
         name = in_dict.get('name', '')
         raise ConfigError(msg % (self.type_name, name, ', '.join(extra_keys)))
 
@@ -302,7 +329,7 @@ class Validator(object):
         """Set any default values for any optional values that were not
         specified.
         """
-        for key, value in self.defaults.iteritems():
+        for key, value in six.iteritems(self.defaults):
             output_dict.setdefault(key, value)
 
     def path_name(self, name=None):
@@ -326,10 +353,10 @@ class Validator(object):
     def validate_contents(self, input, config_context):
         """Override this to validate each value in the input."""
         valid_input = {}
-        for key, value in input.iteritems():
+        for key, value in six.iteritems(input):
             if key in self.validators:
                 child_context = config_context.build_child_context(key)
                 valid_input[key] = self.validators[key](value, child_context)
             else:
                 valid_input[key] = value
-        return valid_input
\ No newline at end of file
+        return valid_input
