@@ -1,34 +1,48 @@
 """
 Parse and validate scheduler configuration and return immutable structures.
 """
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 import calendar
-from collections import namedtuple
 import datetime
 import re
+from collections import namedtuple
 
-from tron.config import ConfigError, config_utils, schema
+from six import string_types
+
+from tron.config import config_utils
+from tron.config import ConfigError
+from tron.config import schema
 from tron.utils import crontab
 
-
 ConfigGenericSchedule = schema.config_object_factory(
     'ConfigGenericSchedule',
     ['type', 'value'],
-    ['jitter']
+    ['jitter'],
 )
 
-ConfigGrocScheduler = namedtuple('ConfigGrocScheduler',
-    'original ordinals weekdays monthdays months timestr jitter')
+ConfigGrocScheduler = namedtuple(
+    'ConfigGrocScheduler',
+    'original ordinals weekdays monthdays months timestr jitter',
+)
 
-ConfigCronScheduler = namedtuple('ConfigCronScheduler',
-    'original minutes hours monthdays months weekdays ordinals jitter')
+ConfigCronScheduler = namedtuple(
+    'ConfigCronScheduler',
+    'original minutes hours monthdays months weekdays ordinals jitter',
+)
 
-ConfigDailyScheduler    = namedtuple('ConfigDailyScheduler',
-    'original hour minute second days jitter')
+ConfigDailyScheduler = namedtuple(
+    'ConfigDailyScheduler',
+    'original hour minute second days jitter',
+)
 
 ConfigConstantScheduler = namedtuple('ConfigConstantScheduler', [])
 
-ConfigIntervalScheduler = namedtuple('ConfigIntervalScheduler',
-    'timedelta jitter')
+ConfigIntervalScheduler = namedtuple(
+    'ConfigIntervalScheduler',
+    'timedelta jitter',
+)
 
 
 class ScheduleParseError(ConfigError):
@@ -38,13 +52,17 @@ class ScheduleParseError(ConfigError):
 def pad_sequence(seq, size, padding=None):
     """Force a sequence to size. Pad with padding if too short, and ignore
     extra pieces if too long."""
-    return (list(seq) + [padding for _ in xrange(size)])[:size]
+    return (list(seq) + [padding for _ in range(size)])[:size]
 
 
 def schedule_config_from_string(schedule, config_context):
     """Return a scheduler config object from a string."""
     schedule = schedule.strip()
-    name, schedule_config = pad_sequence(schedule.split(None, 1), 2, padding='')
+    name, schedule_config = pad_sequence(
+        schedule.split(None, 1),
+        2,
+        padding='',
+    )
     if name not in schedulers:
         config = ConfigGenericSchedule('groc daily', schedule, jitter=None)
         return parse_groc_expression(config, config_context)
@@ -76,7 +94,7 @@ def schedule_config_from_legacy_dict(schedule, config_context):
 
 
 def valid_schedule(schedule, config_context):
-    if isinstance(schedule, basestring):
+    if isinstance(schedule, string_types):
         return schedule_config_from_string(schedule, config_context)
 
     if 'type' not in schedule:
@@ -96,27 +114,38 @@ def valid_daily_scheduler(config, config_context):
     schedule_config = config.value
     time_string, days = pad_sequence(schedule_config.split(), 2)
     time_string = time_string or '00:00:00'
-    time_spec   = config_utils.valid_time(time_string, config_context)
-    days        = config_utils.valid_string(days or "", config_context)
+    time_spec = config_utils.valid_time(time_string, config_context)
+    days = config_utils.valid_string(days or "", config_context)
 
     def valid_day(day):
         if day not in CONVERT_DAYS_INT:
-            raise ConfigError("Unknown day %s at %s" % (day, config_context.path))
+            raise ConfigError(
+                "Unknown day %s at %s" % (
+                    day,
+                    config_context.path,
+                )
+            )
         return CONVERT_DAYS_INT[day]
 
     original = "%s %s" % (time_string, days)
-    weekdays = set(valid_day(day) for day in days or ())
-    return ConfigDailyScheduler(original,
-        time_spec.hour, time_spec.minute, time_spec.second, weekdays,
-        jitter=config.jitter)
+    weekdays = {valid_day(day) for day in days or ()}
+    return ConfigDailyScheduler(
+        original,
+        time_spec.hour,
+        time_spec.minute,
+        time_spec.second,
+        weekdays,
+        jitter=config.jitter,
+    )
 
 
 # Shortcut values for intervals
 TIME_INTERVAL_SHORTCUTS = {
-    'hourly': datetime.timedelta(hours=1)
+    'hourly': datetime.timedelta(hours=1),
 }
 
-def valid_interval_scheduler(config,  config_context):
+
+def valid_interval_scheduler(config, config_context):
     def build_config(delta):
         return ConfigIntervalScheduler(timedelta=delta, jitter=config.jitter)
 
@@ -125,12 +154,14 @@ def valid_interval_scheduler(config,  config_context):
         return build_config(TIME_INTERVAL_SHORTCUTS[interval_key])
 
     return build_config(
-        config_utils.valid_time_delta(config.value, config_context))
+        config_utils.valid_time_delta(config.value, config_context),
+    )
 
 
 def normalize_weekdays(seq):
     return seq[6:7] + seq[:6]
 
+
 def day_canonicalization_map():
     """Build a map of weekday synonym to int index 0-6 inclusive."""
     canon_map = dict()
@@ -139,8 +170,25 @@ def day_canonicalization_map():
     weekday_lists = [
         normalize_weekdays(calendar.day_name),
         normalize_weekdays(calendar.day_abbr),
-        ('u', 'm', 't', 'w', 'r', 'f', 's',),
-        ('su', 'mo', 'tu', 'we', 'th', 'fr', 'sa',)]
+        (
+            'u',
+            'm',
+            't',
+            'w',
+            'r',
+            'f',
+            's',
+        ),
+        (
+            'su',
+            'mo',
+            'tu',
+            'we',
+            'th',
+            'fr',
+            'sa',
+        ),
+    ]
     for day_list in weekday_lists:
         for day_name_synonym, day_index in zip(day_list, range(7)):
             canon_map[day_name_synonym] = day_index
@@ -149,8 +197,9 @@ def day_canonicalization_map():
 
     return canon_map
 
+
 # Canonicalize weekday names to integer indices
-CONVERT_DAYS_INT = day_canonicalization_map()   # day name/abbrev => {0123456}
+CONVERT_DAYS_INT = day_canonicalization_map()  # day name/abbrev => {0123456}
 
 
 def month_canonicalization_map():
@@ -193,10 +242,10 @@ def build_groc_schedule_parser_re():
     """
 
     # m|mon|monday|...|day
-    DAY_VALUES = '|'.join(CONVERT_DAYS_INT.keys() + ['day'])
+    DAY_VALUES = '|'.join(list(CONVERT_DAYS_INT.keys()) + ['day'])
 
     # jan|january|...|month
-    MONTH_VALUES = '|'.join(CONVERT_MONTHS.keys() + ['month'])
+    MONTH_VALUES = '|'.join(list(CONVERT_MONTHS.keys()) + ['month'])
 
     DATE_SUFFIXES = 'st|nd|rd|th'
 
@@ -208,14 +257,20 @@ def build_groc_schedule_parser_re():
     # [at] 00:00
     TIME_EXPR = r'((at\s+)?(?P<time>\d\d:\d\d))?'
 
-    DAILY_SCHEDULE_EXPR = ''.join([
-        r'^',
-        MONTH_DAYS_EXPR, r'\s*',
-        DAYS_EXPR, r'\s*',
-        MONTHS_EXPR, r'\s*',
-         TIME_EXPR, r'\s*',
-        r'$'
-    ])
+    DAILY_SCHEDULE_EXPR = ''.join(
+        [
+            r'^',
+            MONTH_DAYS_EXPR,
+            r'\s*',
+            DAYS_EXPR,
+            r'\s*',
+            MONTHS_EXPR,
+            r'\s*',
+            TIME_EXPR,
+            r'\s*',
+            r'$',
+        ]
+    )
     return re.compile(DAILY_SCHEDULE_EXPR)
 
 
@@ -228,6 +283,7 @@ DAILY_SCHEDULE_RE = build_groc_schedule_parser_re()
 def _parse_number(day):
     return int(''.join(c for c in day if c.isdigit()))
 
+
 def parse_groc_expression(config, config_context):
     """Given an expression of the form in the docstring of
     daily_schedule_parser_re(), return the parsed values in a
@@ -246,14 +302,13 @@ def parse_groc_expression(config, config_context):
     if m.group('days') in (None, 'day'):
         weekdays = None
     else:
-        weekdays = set(CONVERT_DAYS_INT[d] for d in m.group('days').split(','))
+        weekdays = {CONVERT_DAYS_INT[d] for d in m.group('days').split(',')}
 
     monthdays = None
     ordinals = None
 
     if m.group('month_days') != 'every':
-        values = set(_parse_number(n)
-                     for n in m.group('month_days').split(','))
+        values = {_parse_number(n) for n in m.group('month_days').split(',')}
         if weekdays is None:
             monthdays = values
         else:
@@ -262,7 +317,7 @@ def parse_groc_expression(config, config_context):
     if m.group('months') in (None, 'month'):
         months = None
     else:
-        months = set(CONVERT_MONTHS[mo] for mo in m.group('months').split(','))
+        months = {CONVERT_MONTHS[mo] for mo in m.group('months').split(',')}
 
     return ConfigGrocScheduler(
         original=expression,
@@ -271,7 +326,8 @@ def parse_groc_expression(config, config_context):
         monthdays=monthdays,
         months=months,
         timestr=timestr,
-        jitter=config.jitter)
+        jitter=config.jitter,
+    )
 
 
 def valid_cron_scheduler(config, config_context):
@@ -279,18 +335,19 @@ def valid_cron_scheduler(config, config_context):
     try:
         crontab_kwargs = crontab.parse_crontab(config.value)
         return ConfigCronScheduler(
-            original=config.value, jitter=config.jitter, **crontab_kwargs)
-    except ValueError, e:
+            original=config.value, jitter=config.jitter, **crontab_kwargs
+        )
+    except ValueError as e:
         msg = "Invalid cron scheduler %s: %s"
         raise ConfigError(msg % (config_context.path, e))
 
 
 schedulers = {
-    'constant':     valid_constant_scheduler,
-    'daily':        valid_daily_scheduler,
-    'interval':     valid_interval_scheduler,
-    'cron':         valid_cron_scheduler,
-    'groc daily':   parse_groc_expression,
+    'constant': valid_constant_scheduler,
+    'daily': valid_daily_scheduler,
+    'interval': valid_interval_scheduler,
+    'cron': valid_cron_scheduler,
+    'groc daily': parse_groc_expression,
 }
 
 
@@ -298,9 +355,9 @@ class ScheduleValidator(config_utils.Validator):
     """Validate the structure of a scheduler config."""
     config_class = ConfigGenericSchedule
     defaults = {
-        'jitter':       datetime.timedelta()
+        'jitter': datetime.timedelta(),
     }
     validators = {
-        'type':         config_utils.build_enum_validator(schedulers.keys()),
-        'jitter':       config_utils.valid_time_delta
+        'type': config_utils.build_enum_validator(schedulers.keys()),
+        'jitter': config_utils.valid_time_delta,
     }
