@@ -1,14 +1,31 @@
-import logging
-import itertools
+from __future__ import absolute_import
+from __future__ import unicode_literals
 
-from tron import command_context, event, node, eventloop
-from tron.core import jobrun
+import datetime
+import logging
+
+import humanize
+import six
+from six.moves import filter
+
+from tron import command_context
+from tron import event
+from tron import eventloop
+from tron import node
 from tron.core import actiongraph
+from tron.core import jobrun
+from tron.core import recovery
 from tron.core.actionrun import ActionRun
 from tron.scheduler import scheduler_from_config
 from tron.serialize import filehandler
-from tron.utils import timeutils, proxy, iteration, collections
-from tron.utils.observer import Observable, Observer
+from tron.utils import collections
+from tron.utils import iteration
+from tron.utils import maybe_decode
+from tron.utils import proxy
+from tron.utils import timeutils
+from tron.utils.observer import Observable
+from tron.utils.observer import Observer
+
 
 class Error(Exception):
     pass
@@ -32,15 +49,15 @@ class Job(Observable, Observer):
     actions and their dependency graph.
     """
 
-    STATUS_DISABLED         = "disabled"
-    STATUS_ENABLED          = "enabled"
-    STATUS_UNKNOWN          = "unknown"
-    STATUS_RUNNING          = "running"
+    STATUS_DISABLED = "disabled"
+    STATUS_ENABLED = "enabled"
+    STATUS_UNKNOWN = "unknown"
+    STATUS_RUNNING = "running"
 
-    NOTIFY_STATE_CHANGE     = 'notify_state_change'
-    NOTIFY_RUN_DONE         = 'notify_run_done'
+    NOTIFY_STATE_CHANGE = 'notify_state_change'
+    NOTIFY_RUN_DONE = 'notify_run_done'
 
-    context_class           = command_context.JobContext
+    context_class = command_context.JobContext
 
     # These attributes determine equality between two Job objects
     equality_attributes = [
@@ -54,54 +71,95 @@ class Job(Observable, Observer):
         'action_runner',
         'max_runtime',
         'allow_overlap',
+        'monitoring',
+        'time_zone',
+        'service',
+        'deploy_group',
+        'expected_runtime',
     ]
 
     # TODO: use config object
-    def __init__(self, name, scheduler, queueing=True, all_nodes=False,
-            node_pool=None, enabled=True, action_graph=None,
-            run_collection=None, parent_context=None, output_path=None,
-            allow_overlap=None, action_runner=None, max_runtime=None):
+    def __init__(
+        self,
+        name,
+        scheduler,
+        queueing=True,
+        all_nodes=False,
+        monitoring=None,
+        node_pool=None,
+        enabled=True,
+        action_graph=None,
+        run_collection=None,
+        parent_context=None,
+        output_path=None,
+        allow_overlap=None,
+        action_runner=None,
+        max_runtime=None,
+        time_zone=None,
+        service=None,
+        deploy_group=None,
+        expected_runtime=None
+    ):
         super(Job, self).__init__()
-        self.name               = name
-        self.action_graph       = action_graph
-        self.scheduler          = scheduler
-        self.runs               = run_collection
-        self.queueing           = queueing
-        self.all_nodes          = all_nodes
-        self.enabled            = enabled
-        self.node_pool          = node_pool
-        self.allow_overlap      = allow_overlap
-        self.action_runner      = action_runner
-        self.max_runtime        = max_runtime
-        self.output_path        = output_path or filehandler.OutputPath()
+        self.name = maybe_decode(name)
+        self.monitoring = monitoring
+        self.action_graph = action_graph
+        self.scheduler = scheduler
+        self.runs = run_collection
+        self.queueing = queueing
+        self.all_nodes = all_nodes
+        self.enabled = enabled
+        self.node_pool = node_pool
+        self.allow_overlap = allow_overlap
+        self.action_runner = action_runner
+        self.max_runtime = max_runtime
+        self.time_zone = time_zone
+        self.service = service
+        self.deploy_group = deploy_group
+        self.expected_runtime = expected_runtime
+        self.output_path = output_path or filehandler.OutputPath()
         self.output_path.append(name)
-        self.event              = event.get_recorder(self.name)
+        self.event = event.get_recorder(self.name)
         self.context = command_context.build_context(self, parent_context)
         self.event.ok('created')
 
     @classmethod
-    def from_config(cls,
-            job_config, scheduler, parent_context, output_path, action_runner):
+    def from_config(
+        cls,
+        job_config,
+        scheduler,
+        parent_context,
+        output_path,
+        action_runner,
+    ):
         """Factory method to create a new Job instance from configuration."""
         action_graph = actiongraph.ActionGraph.from_config(
-                job_config.actions, job_config.cleanup_action)
-        runs         = jobrun.JobRunCollection.from_config(job_config)
-        node_repo    = node.NodePoolRepository.get_instance()
+            job_config.actions,
+            job_config.cleanup_action,
+        )
+        runs = jobrun.JobRunCollection.from_config(job_config)
+        node_repo = node.NodePoolRepository.get_instance()
 
         return cls(
-            name                = job_config.name,
-            queueing            = job_config.queueing,
-            all_nodes           = job_config.all_nodes,
-            node_pool           = node_repo.get_by_name(job_config.node),
-            scheduler           = scheduler,
-            enabled             = job_config.enabled,
-            run_collection      = runs,
-            action_graph        = action_graph,
-            parent_context      = parent_context,
-            output_path         = output_path,
-            allow_overlap       = job_config.allow_overlap,
-            action_runner       = action_runner,
-            max_runtime         = job_config.max_runtime)
+            name=job_config.name,
+            monitoring=job_config.monitoring,
+            time_zone=job_config.time_zone,
+            queueing=job_config.queueing,
+            all_nodes=job_config.all_nodes,
+            node_pool=node_repo.get_by_name(job_config.node),
+            scheduler=scheduler,
+            enabled=job_config.enabled,
+            run_collection=runs,
+            action_graph=action_graph,
+            parent_context=parent_context,
+            output_path=output_path,
+            allow_overlap=job_config.allow_overlap,
+            action_runner=action_runner,
+            max_runtime=job_config.max_runtime,
+            service=job_config.service,
+            deploy_group=job_config.deploy_group,
+            expected_runtime=job_config.expected_runtime,
+        )
 
     def update_from_job(self, job):
         """Update this Jobs configuration from a new config. This method
@@ -120,16 +178,27 @@ class Job(Observable, Observer):
         if self.runs.get_run_by_state(ActionRun.STATE_RUNNING):
             return self.STATUS_RUNNING
 
-        if (self.runs.get_run_by_state(ActionRun.STATE_SCHEDULED) or
-                self.runs.get_run_by_state(ActionRun.STATE_QUEUED)):
+        if self.runs.get_run_by_state(ActionRun.STATE_SCHEDULED):
             return self.STATUS_ENABLED
 
-        log.warn("%s in an unknown state: %s" % (self, self.runs))
+        log.warning("%s in an unknown state: %s" % (self, self.runs))
         return self.STATUS_UNKNOWN
 
     def get_name(self):
         return self.name
 
+    def get_monitoring(self):
+        return self.monitoring
+
+    def get_time_zone(self):
+        return self.time_zone
+
+    def get_service(self):
+        return self.service
+
+    def get_deploy_group(self):
+        return self.deploy_group
+
     def get_runs(self):
         return self.runs
 
@@ -137,23 +206,21 @@ class Job(Observable, Observer):
     def state_data(self):
         """This data is used to serialize the state of this job."""
         return {
-            'runs':             self.runs.state_data,
-            'enabled':          self.enabled
+            'runs': self.runs.state_data,
+            'enabled': self.enabled,
         }
 
-    def restore_state(self, state_data):
+    def get_job_runs_from_state(self, state_data):
         """Apply a previous state to this Job."""
         self.enabled = state_data['enabled']
-        job_runs = self.runs.restore_state(
-                state_data['runs'],
-                self.action_graph,
-                self.output_path.clone(),
-                self.context,
-                self.node_pool)
-        for run in job_runs:
-            self.watch(run)
-
-        self.event.ok('restored')
+        job_runs = jobrun.job_runs_from_state(
+            state_data['runs'],
+            self.action_graph,
+            self.output_path.clone(),
+            self.context,
+            self.node_pool,
+        )
+        return job_runs
 
     def build_new_runs(self, run_time, manual=False):
         """Uses its JobCollection to build new JobRuns. If all_nodes is set,
@@ -162,8 +229,8 @@ class Job(Observable, Observer):
         """
         pool = self.node_pool
         nodes = pool.nodes if self.all_nodes else [pool.next()]
-        for node in nodes:
-            run = self.runs.build_new_run(self, run_time, node, manual=manual)
+        for n in nodes:
+            run = self.runs.build_new_run(self, run_time, n, manual=manual)
             self.watch(run)
             yield run
 
@@ -180,11 +247,14 @@ class Job(Observable, Observer):
         if event == jobrun.JobRun.NOTIFY_DONE:
             self.notify(self.NOTIFY_RUN_DONE)
             return
+
     handler = handle_job_run_state_change
 
     def __eq__(self, other):
-        return all(getattr(other, attr, None) == getattr(self, attr, None)
-                   for attr in self.equality_attributes)
+        return all(
+            getattr(other, attr, None) == getattr(self, attr, None)
+            for attr in self.equality_attributes
+        )
 
     def __ne__(self, other):
         return not self == other
@@ -200,16 +270,27 @@ class JobScheduler(Observer):
     """
 
     def __init__(self, job):
-        self.job                = job
+        self.job = job
         self.shutdown_requested = False
         self.watch(job)
 
-    def restore_state(self, job_state_data):
+    def restore_state(self, job_state_data, config_action_runner):
         """Restore the job state and schedule any JobRuns."""
-        self.job.restore_state(job_state_data)
+        job_runs = self.job.get_job_runs_from_state(job_state_data)
+        for run in job_runs:
+            self.job.watch(run)
+        self.job.runs.runs.extend(job_runs)
+        self.job.event.ok('restored')
+
+        recovery.launch_recovery_actionruns_for_job_runs(
+            job_runs=job_runs, master_action_runner=config_action_runner
+        )
+
         scheduled = self.job.runs.get_scheduled()
+        # for those that were already scheduled, we reschedule them to run.
         for job_run in scheduled:
             self._set_callback(job_run)
+
         # Ensure we have at least 1 scheduled run
         self.schedule()
 
@@ -237,7 +318,7 @@ class JobScheduler(Observer):
 
     def manual_start(self, run_time=None):
         """Trigger a job run manually (instead of from the scheduler)."""
-        run_time = run_time or timeutils.current_time()
+        run_time = run_time or timeutils.current_time(tz=self.job.time_zone)
         manual_runs = list(self.job.build_new_runs(run_time, manual=True))
         for r in manual_runs:
             r.start()
@@ -259,8 +340,14 @@ class JobScheduler(Observer):
 
     def _set_callback(self, job_run):
         """Set a callback for JobRun to fire at the appropriate time."""
-        log.info("Scheduling next Jobrun for %s", self.job.name)
         seconds = job_run.seconds_until_run_time()
+        human_time = humanize.naturaltime(datetime.timedelta(seconds=seconds))
+        log.info(
+            "Scheduling next Jobrun for %s about %s from now (%d seconds)",
+            self.job.name,
+            human_time,
+            seconds,
+        )
         eventloop.call_later(seconds, self.run_job, job_run)
 
     # TODO: new class for this method
@@ -282,8 +369,12 @@ class JobScheduler(Observer):
         # also possible this job was run (or is running) manually by a user.
         # Alternatively, if run_queued is True, this job_run is already queued.
         if not run_queued and not job_run.is_scheduled:
-            log.info("%s in state %s already out of scheduled state." % (
-                    job_run, job_run.state))
+            log.info(
+                "%s in state %s already out of scheduled state." % (
+                    job_run,
+                    job_run.state,
+                )
+            )
             return self.schedule()
 
         node = job_run.node if self.job.all_nodes else None
@@ -317,7 +408,9 @@ class JobScheduler(Observer):
         """
         if event != Job.NOTIFY_RUN_DONE:
             return
+        self.run_queue_schedule()
 
+    def run_queue_schedule(self):
         # TODO: this should only start runs on the same node if this is an
         # all_nodes job, but that is currently not possible
         queued_run = self.job.runs.get_first_queued()
@@ -328,6 +421,7 @@ class JobScheduler(Observer):
         # previous run was cancelled from a scheduled state, or if the job
         # scheduler is `schedule_on_complete`.
         self.schedule()
+
     handler = handle_job_events
 
     def get_runs_to_schedule(self, ignore_last_run_time):
@@ -370,16 +464,23 @@ class JobSchedulerFactory(object):
     """Construct JobScheduler instances from configuration."""
 
     def __init__(self, context, output_stream_dir, time_zone, action_runner):
-        self.context            = context
-        self.output_stream_dir  = output_stream_dir
-        self.time_zone          = time_zone
-        self.action_runner      = action_runner
+        self.context = context
+        self.output_stream_dir = output_stream_dir
+        self.time_zone = time_zone
+        self.action_runner = action_runner
 
     def build(self, job_config):
         log.debug("Building new job %s", job_config.name)
         output_path = filehandler.OutputPath(self.output_stream_dir)
-        scheduler = scheduler_from_config(job_config.schedule, self.time_zone)
-        job = Job.from_config(job_config, scheduler, self.context, output_path, self.action_runner)
+        time_zone = job_config.time_zone or self.time_zone
+        scheduler = scheduler_from_config(job_config.schedule, time_zone)
+        job = Job.from_config(
+            job_config,
+            scheduler,
+            self.context,
+            output_path,
+            self.action_runner,
+        )
         return JobScheduler(job)
 
 
@@ -388,13 +489,17 @@ class JobCollection(object):
 
     def __init__(self):
         self.jobs = collections.MappingCollection('jobs')
-        self.proxy = proxy.CollectionProxy(self.jobs.itervalues, [
-            proxy.func_proxy('request_shutdown',    iteration.list_all),
-            proxy.func_proxy('enable',              iteration.list_all),
-            proxy.func_proxy('disable',             iteration.list_all),
-            proxy.func_proxy('schedule',            iteration.list_all),
-            proxy.attr_proxy('is_shutdown',         all)
-        ])
+        self.proxy = proxy.CollectionProxy(
+            lambda: six.itervalues(self.jobs),
+            [
+                proxy.func_proxy('request_shutdown', iteration.list_all),
+                proxy.func_proxy('enable', iteration.list_all),
+                proxy.func_proxy('disable', iteration.list_all),
+                proxy.func_proxy('schedule', iteration.list_all),
+                proxy.func_proxy('run_queue_schedule', iteration.list_all),
+                proxy.attr_proxy('is_shutdown', all),
+            ],
+        )
 
     def load_from_config(self, job_configs, factory, reconfigure):
         """Apply a configuration to this collection and return a generator of
@@ -408,8 +513,8 @@ class JobCollection(object):
                     job_scheduler.schedule()
                 yield job_scheduler.get_job()
 
-        seq = (factory.build(config) for config in job_configs.itervalues())
-        return map_to_job_and_schedule(itertools.ifilter(self.add, seq))
+        seq = (factory.build(config) for config in six.itervalues(job_configs))
+        return map_to_job_and_schedule(filter(self.add, seq))
 
     def add(self, job_scheduler):
         return self.jobs.add(job_scheduler, self.update)
@@ -421,8 +526,10 @@ class JobCollection(object):
         job_scheduler.schedule_reconfigured()
         return True
 
-    def restore_state(self, job_state_data):
-        self.jobs.restore_state(job_state_data)
+    def restore_state(self, job_state_data, config_action_runner):
+        for name, state in job_state_data.items():
+            self.jobs[name].restore_state(state, config_action_runner)
+        log.info("Loaded state for %d jobs", len(job_state_data))
 
     def get_by_name(self, name):
         return self.jobs.get(name)
@@ -437,7 +544,7 @@ class JobCollection(object):
         return [sched.get_job_runs() for sched in self]
 
     def __iter__(self):
-        return self.jobs.itervalues()
+        return six.itervalues(self.jobs)
 
     def __getattr__(self, name):
         return self.proxy.perform(name)
