@@ -1,18 +1,29 @@
 """
  Classes to manage job runs.
 """
+from __future__ import absolute_import
+from __future__ import unicode_literals
 
-from collections import deque
 import logging
-import itertools
-from tron import node, command_context, event
-from tron.core.actionrun import ActionRun, ActionRunFactory
+from collections import deque
+
+from six.moves import filter
+
+from tron import command_context
+from tron import event
+from tron import node
+from tron.core.actionrun import ActionRun
+from tron.core.actionrun import ActionRunFactory
 from tron.serialize import filehandler
-from tron.utils import timeutils, proxy
-from tron.utils.observer import Observable, Observer
+from tron.utils import maybe_decode
+from tron.utils import proxy
+from tron.utils import timeutils
+from tron.utils.observer import Observable
+from tron.utils.observer import Observer
 
 log = logging.getLogger(__name__)
 
+
 class Error(Exception):
     pass
 
@@ -23,31 +34,44 @@ class JobRun(Observable, Observer):
     dependencies.
     """
 
-    NOTIFY_DONE           = 'notify_done'
-    NOTIFY_STATE_CHANGED  = 'notify_state_changed'
+    NOTIFY_DONE = 'notify_done'
+    NOTIFY_STATE_CHANGED = 'notify_state_changed'
 
-    context_class         = command_context.JobRunContext
+    context_class = command_context.JobRunContext
 
     # TODO: use config object
-    def __init__(self, job_name, run_num, run_time, node, output_path=None,
-                base_context=None, action_runs=None, action_graph=None,
-                manual=None):
+    def __init__(
+        self,
+        job_name,
+        run_num,
+        run_time,
+        node,
+        output_path=None,
+        base_context=None,
+        action_runs=None,
+        action_graph=None,
+        manual=None,
+        service=None,
+        deploy_group=None,
+    ):
         super(JobRun, self).__init__()
-        self.job_name           = job_name
-        self.run_num            = run_num
-        self.run_time           = run_time
-        self.node               = node
-        self.output_path        = output_path or filehandler.OutputPath()
+        self.job_name = maybe_decode(job_name)
+        self.run_num = run_num
+        self.run_time = run_time
+        self.node = node
+        self.output_path = output_path or filehandler.OutputPath()
         self.output_path.append(self.id)
-        self.action_runs_proxy  = None
-        self._action_runs       = None
-        self.action_graph       = action_graph
-        self.manual             = manual
-        self.event              = event.get_recorder(self.id)
+        self.action_runs_proxy = None
+        self._action_runs = None
+        self.action_graph = action_graph
+        self.manual = manual
+        self.service = service
+        self.deploy_group = deploy_group
+        self.event = event.get_recorder(self.full_id)
         self.event.ok('created')
 
         if action_runs:
-            self.action_runs    = action_runs
+            self.action_runs = action_runs
 
         self.context = command_context.build_context(self, base_context)
 
@@ -58,25 +82,41 @@ class JobRun(Observable, Observer):
     @classmethod
     def for_job(cls, job, run_num, run_time, node, manual):
         """Create a JobRun for a job."""
-        run = cls(job.get_name(), run_num, run_time, node,
-                job.output_path.clone(),
-                job.context,
-                action_graph=job.action_graph,
-                manual=manual)
+        run = cls(
+            job.get_name(),
+            run_num,
+            run_time,
+            node,
+            job.output_path.clone(),
+            job.context,
+            action_graph=job.action_graph,
+            manual=manual,
+            service=job.service,
+            deploy_group=job.deploy_group,
+        )
 
-        action_runs     = ActionRunFactory.build_action_run_collection(run, job.action_runner)
+        action_runs = ActionRunFactory.build_action_run_collection(
+            run,
+            job.action_runner,
+        )
         run.action_runs = action_runs
         return run
 
     @classmethod
-    def from_state(cls, state_data, action_graph, output_path, context,
-                run_node):
+    def from_state(
+        cls,
+        state_data,
+        action_graph,
+        output_path,
+        context,
+        run_node,
+    ):
         """Restore a JobRun from a serialized state."""
         pool_repo = node.NodePoolRepository.get_instance()
-        run_node  = pool_repo.get_node(state_data.get('node_name'), run_node)
-        job_name  = state_data['job_name']
+        run_node = pool_repo.get_node(state_data.get('node_name'), run_node)
+        job_name = state_data['job_name']
 
-        job_run =  cls(
+        job_run = cls(
             job_name,
             state_data['run_num'],
             state_data['run_time'],
@@ -84,10 +124,15 @@ class JobRun(Observable, Observer):
             action_graph=action_graph,
             manual=state_data.get('manual', False),
             output_path=output_path,
-            base_context=context
+            base_context=context,
+            service=state_data.get('service'),
+            deploy_group=state_data.get('deploy_group'),
         )
         action_runs = ActionRunFactory.action_run_collection_from_state(
-                job_run, state_data['runs'], state_data['cleanup_run'])
+            job_run,
+            state_data['runs'],
+            state_data['cleanup_run'],
+        )
         job_run.action_runs = action_runs
         return job_run
 
@@ -95,13 +140,15 @@ class JobRun(Observable, Observer):
     def state_data(self):
         """This data is used to serialize the state of this job run."""
         return {
-            'job_name':         self.job_name,
-            'run_num':          self.run_num,
-            'run_time':         self.run_time,
-            'node_name':        self.node.get_name() if self.node else None,
-            'runs':             self.action_runs.state_data,
-            'cleanup_run':      self.action_runs.cleanup_action_state_data,
-            'manual':           self.manual,
+            'job_name': self.job_name,
+            'run_num': self.run_num,
+            'run_time': self.run_time,
+            'node_name': self.node.get_name() if self.node else None,
+            'runs': self.action_runs.state_data,
+            'cleanup_run': self.action_runs.cleanup_action_state_data,
+            'manual': self.manual,
+            'service': self.service,
+            'deploy_group': self.deploy_group,
         }
 
     def _get_action_runs(self):
@@ -134,19 +181,25 @@ class JobRun(Observable, Observer):
                 'is_skipped',
                 'is_starting',
                 'start_time',
-                'end_time'
-            ])
+                'end_time',
+            ],
+        )
 
     def _del_action_runs(self):
         del self._action_runs
 
-    action_runs = property(_get_action_runs, _set_action_runs, _del_action_runs)
+    action_runs = property(
+        _get_action_runs,
+        _set_action_runs,
+        _del_action_runs,
+    )
 
     def seconds_until_run_time(self):
         run_time = self.run_time
-        now = timeutils.current_time()
         if run_time.tzinfo:
-            now = run_time.tzinfo.localize(now)
+            now = timeutils.current_time(tz=run_time.tzinfo)
+        else:
+            now = timeutils.current_time()
         return max(0, timeutils.delta_total_seconds(run_time - now))
 
     def start(self):
@@ -187,6 +240,9 @@ class JobRun(Observable, Observer):
         if not action_run.is_done:
             return
 
+        if action_run.is_skipped and self.action_runs.is_scheduled:
+            return
+
         if not action_run.is_broken and any(self._start_action_runs()):
             log.info("Action runs started for %s." % self)
             return
@@ -206,6 +262,7 @@ class JobRun(Observable, Observer):
         # action to be triggered more then once. Guard against that.
         if cleanup_run.check_state('start'):
             cleanup_run.start()
+
     handler = handle_action_run_state_change
 
     def finalize(self):
@@ -226,7 +283,7 @@ class JobRun(Observable, Observer):
     def cleanup(self):
         """Cleanup any resources used by this JobRun."""
         self.event.notice('removed')
-        event.EventManager.get_instance().remove(str(self))
+        event.EventManager.get_instance().remove(self.full_id)
         self.clear_observers()
         self.action_runs.cleanup()
         self.node = None
@@ -260,16 +317,19 @@ class JobRun(Observable, Observer):
         if self.action_runs.is_queued:
             return ActionRun.STATE_QUEUED
 
-        log.info("%s in an unknown state: %s" % (self, self.action_runs))
         return ActionRun.STATE_UNKNOWN
 
+    @property
+    def full_id(self):
+        return "JobRun:%s" % self.id
+
     def __getattr__(self, name):
         if self.action_runs_proxy:
             return self.action_runs_proxy.perform(name)
         raise AttributeError(name)
 
     def __str__(self):
-        return "JobRun:%s" % self.id
+        return self.full_id
 
 
 class JobRunCollection(object):
@@ -293,28 +353,19 @@ class JobRunCollection(object):
         """Factory method for creating a JobRunCollection from a config."""
         return cls(job_config.run_limit)
 
-    def restore_state(self, state_data, action_graph, output_path, context,
-            node_pool):
-        """Apply state to all jobs from the state dict."""
-        if self.runs:
-            msg = "State can not be restored to a collection with runs."
-            raise ValueError(msg)
-
-        restored_runs = [
-            JobRun.from_state(run_state, action_graph, output_path.clone(),
-                context, node_pool.next())
-            for run_state in state_data
-        ]
-        self.runs.extend(restored_runs)
-        return restored_runs
-
     def build_new_run(self, job, run_time, node, manual=False):
         """Create a new run for the job, add it to the runs list,
         and return it.
         """
         run_num = self.next_run_num()
-        log.info("Building JobRun %s for %s on %s at %s" %
-             (run_num, job, node, run_time))
+        log.info(
+            "Building JobRun %s for %s on %s at %s" % (
+                run_num,
+                job,
+                node,
+                run_time,
+            )
+        )
 
         run = JobRun.for_job(job, run_num, run_time, node, manual)
         self.runs.appendleft(run)
@@ -335,7 +386,7 @@ class JobRunCollection(object):
     def _get_runs_using(self, func, reverse=False):
         """Filter runs using func()."""
         job_runs = self.runs if not reverse else reversed(self.runs)
-        return itertools.ifilter(func, job_runs)
+        return filter(func, job_runs)
 
     def _get_run_using(self, func, reverse=False):
         """Find the first run (from most recent to least recent), where func()
@@ -343,7 +394,7 @@ class JobRunCollection(object):
         argument (a JobRun), and return True or False.
         """
         try:
-            return self._get_runs_using(func, reverse).next()
+            return next(self._get_runs_using(func, reverse))
         except StopIteration:
             return None
 
@@ -372,7 +423,10 @@ class JobRunCollection(object):
 
     def get_newest(self, include_manual=True):
         """Returns the most recently created JobRun."""
-        func = lambda r: True if include_manual else not r.manual
+
+        def func(r):
+            return True if include_manual else not r.manual
+
         return self._get_run_using(func)
 
     def get_pending(self):
@@ -385,15 +439,22 @@ class JobRunCollection(object):
 
     def get_active(self, node=None):
         if node:
-            func = lambda r: (r.is_running or r.is_starting) and r.node == node
+
+            def func(r):
+                return (r.is_running or r.is_starting) and r.node == node
         else:
-            func = lambda r: r.is_running or r.is_starting
+
+            def func(r):
+                return r.is_running or r.is_starting
+
         return self._get_runs_using(func)
 
     def get_first_queued(self, node=None):
         state = ActionRun.STATE_QUEUED
         if node:
-            queued_func = lambda r: r.state == state and r.node == node
+
+            def queued_func(r):
+                return r.state == state and r.node == node
         else:
             queued_func = self._filter_by_state(state)
         return self._get_run_using(queued_func, reverse=True)
@@ -406,11 +467,13 @@ class JobRunCollection(object):
         """Return the most recent run which is either running or scheduled. If
         node is not None, then only looks for runs on that node.
         """
+
         def compare(run):
             if node and run.node != node:
                 return False
             if run.is_running or run.is_scheduled:
                 return run
+
         return self._get_run_using(compare)
 
     def next_run_num(self):
@@ -448,6 +511,24 @@ class JobRunCollection(object):
 
     def __str__(self):
         return "%s[%s]" % (
-                type(self).__name__,
-                ', '.join("%s(%s)" % (r.run_num, r.state) for r in self.runs)
+            type(self).__name__,
+            ', '.join("%s(%s)" % (r.run_num, r.state) for r in self.runs),
         )
+
+
+def job_runs_from_state(
+    runs,
+    action_graph,
+    output_path,
+    context,
+    node_pool,
+):
+    return [
+        JobRun.from_state(
+            run,
+            action_graph,
+            output_path.clone(),
+            context,
+            node_pool.next(),
+        ) for run in runs
+    ]
