@@ -1,13 +1,17 @@
-from collections import deque
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
+import itertools
 import logging
 import operator
-import itertools
+from collections import deque
+
+import six
 
 from tron.utils import timeutils
 
 log = logging.getLogger(__name__)
 
-
 # Special character used to split an entity name into hierarchy levels
 NAME_CHARACTER = '.'
 
@@ -17,31 +21,44 @@ class EventLevel(object):
     __slots__ = ('order', 'label')
 
     def __init__(self, order, label):
-        self.order          = order
-        self.label          = label
+        self.order = order
+        self.label = label
 
     def __eq__(self, other):
         return self.order == other.order
 
     def __cmp__(self, other):
-        return cmp(self.order, other.order)
+        # https://docs.python.org/3.0/whatsnew/3.0.html#ordering-comparisons
+        # TODO: drop this
+        return (self.order > other.order) - (self.order < other.order)
+
+    def __lt__(self, other):
+        return self.order < other.order
+
+    def __le__(self, other):
+        return self.order <= other.order
+
+    def __gt__(self, other):
+        return self.order > other.order
+
+    def __ge__(self, other):
+        return self.order >= other.order
 
     def __hash__(self):
         return hash(self.order)
 
 
-LEVEL_INFO      = EventLevel(0, "INFO")         # Troubleshooting information
-LEVEL_OK        = EventLevel(1, "OK")           # Expected behaviour
-LEVEL_NOTICE    = EventLevel(2, "NOTICE")       # Troubling behaviour
-LEVEL_CRITICAL  = EventLevel(3, "CRITICAL")     # Major Failure
+LEVEL_INFO = EventLevel(0, "INFO")  # Troubleshooting information
+LEVEL_OK = EventLevel(1, "OK")  # Expected behaviour
+LEVEL_NOTICE = EventLevel(2, "NOTICE")  # Troubling behaviour
+LEVEL_CRITICAL = EventLevel(3, "CRITICAL")  # Major Failure
 
 
 class EventStore(object):
     """An index of event level to a circular buffer of events. Supports
     retrieving events which with a minimal level.
     """
-    DEFAULT_LIMIT   = 10
-    NO_LEVEL        = EventLevel(None, None)
+    DEFAULT_LIMIT = 10
 
     def __init__(self, limits=None):
         self.limits = limits or dict()
@@ -57,11 +74,12 @@ class EventStore(object):
             self.events[level] = self._build_deque(level)
         self.events[level].append(event)
 
-    def get_events(self, min_level=None):
-        min_level       = min_level or self.NO_LEVEL
-        event_iterable  = self.events.iteritems()
-        groups          = (e for key, e in event_iterable if key >= min_level)
+    def get_events(self, min_level=LEVEL_INFO):
+        min_level = min_level or LEVEL_INFO
+        event_iterable = six.iteritems(self.events)
+        groups = (e for key, e in event_iterable if key >= min_level)
         return itertools.chain.from_iterable(groups)
+
     __iter__ = get_events
 
 
@@ -70,11 +88,11 @@ class Event(object):
     __slots__ = ('entity', 'time', 'level', 'name', 'data')
 
     def __init__(self, entity, level, name, **data):
-        self.entity     = entity
-        self.time       = timeutils.current_time()
-        self.level      = level
-        self.name       = name
-        self.data       = data
+        self.entity = entity
+        self.time = timeutils.current_time()
+        self.level = level
+        self.name = name
+        self.data = data
 
 
 class EventRecorder(object):
@@ -84,18 +102,18 @@ class EventRecorder(object):
     __slots__ = ('name', 'children', 'events')
 
     def __init__(self, name):
-        self.name           = name
-        self.children       = {}
-        self.events         = EventStore()
+        self.name = name
+        self.children = {}
+        self.events = EventStore()
 
     def get_child(self, child_key):
         if child_key in self.children:
             return self.children[child_key]
 
-        split_char      = NAME_CHARACTER
-        name_parts      = [self.name, child_key] if self.name else [child_key]
-        child_name      = split_char.join(name_parts)
-        child           = EventRecorder(child_name)
+        split_char = NAME_CHARACTER
+        name_parts = [self.name, child_key] if self.name else [child_key]
+        child_name = split_char.join(name_parts)
+        child = EventRecorder(child_name)
         return self.children.setdefault(child_key, child)
 
     def remove_child(self, child_key):
@@ -118,18 +136,18 @@ class EventRecorder(object):
         """
         for event in self.events.get_events(min_level):
             yield event
-        for child in self.children.itervalues():
+        for child in six.itervalues(self.children):
             for event in child._events_with_child_events(min_level):
                 yield event
 
     def info(self, name, **data):
-        return self._record(LEVEL_INFO,     name, **data)
+        return self._record(LEVEL_INFO, name, **data)
 
     def ok(self, name, **data):
-        return self._record(LEVEL_OK,       name, **data)
+        return self._record(LEVEL_OK, name, **data)
 
     def notice(self, name, **data):
-        return self._record(LEVEL_NOTICE,   name, **data)
+        return self._record(LEVEL_NOTICE, name, **data)
 
     def critical(self, name, **data):
         return self._record(LEVEL_CRITICAL, name, **data)
@@ -178,8 +196,8 @@ class EventManager(object):
 
     def remove(self, entity_name):
         """Remove an event recorder."""
-        recorder        = self.root_recorder
-        name_parts      = self._get_name_parts(entity_name)
+        recorder = self.root_recorder
+        name_parts = self._get_name_parts(entity_name)
         for child_key in name_parts[:-1]:
             recorder = recorder.get_child(child_key)
 
