@@ -1,4 +1,6 @@
 """Minimal abstraction oer an event loop."""
+from __future__ import absolute_import
+from __future__ import unicode_literals
 
 from twisted.internet import reactor
 from twisted.internet.base import DelayedCall
@@ -10,8 +12,8 @@ class Callback(DelayedCall):
     """
     pass
 
-class NullCallback(object):
 
+class NullCallback(object):
     @staticmethod
     def cancel():
         pass
@@ -31,19 +33,20 @@ class UniqueCallback(object):
     """
 
     def __init__(self, delay, func, *args, **kwargs):
-        self.delay          = delay
-        self.func           = func
-        self.args           = args
-        self.kwargs         = kwargs
-        self.delayed_call   = NullCallback
+        self.delay = delay
+        self.func = func
+        self.args = args
+        self.kwargs = kwargs
+        self.delayed_call = NullCallback
 
     def start(self):
         if not self.delay or self.delayed_call.active():
             return
 
         self.delayed_call = call_later(
-            self.delay, self.func, *self.args, **self.kwargs)
+            self.delay, self.func, *self.args, **self.kwargs
+        )
 
     def cancel(self):
         if self.delayed_call.active():
-            self.delayed_call.cancel()
\ No newline at end of file
+            self.delayed_call.cancel()
