@@ -1,16 +1,19 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
 from __future__ import with_statement
+
 import logging
 
-from tron import command_context, actioncommand
-from tron import event
+from tron import actioncommand
+from tron import command_context
 from tron import crash_reporter
+from tron import event
 from tron import node
 from tron.config import manager
-from tron.core import service, job
+from tron.core import job
 from tron.serialize.runstate import statemanager
 from tron.utils import emailer
 
-
 log = logging.getLogger(__name__)
 
 
@@ -28,15 +31,14 @@ class MasterControlProgram(object):
 
     def __init__(self, working_dir, config_path):
         super(MasterControlProgram, self).__init__()
-        self.jobs               = job.JobCollection()
-        self.services           = service.ServiceCollection()
-        self.working_dir        = working_dir
-        self.crash_reporter     = None
-        self.config             = manager.ConfigManager(config_path)
-        self.context            = command_context.CommandContext()
-        self.event_recorder     = event.get_recorder()
+        self.jobs = job.JobCollection()
+        self.working_dir = working_dir
+        self.crash_reporter = None
+        self.config = manager.ConfigManager(config_path)
+        self.context = command_context.CommandContext()
+        self.event_recorder = event.get_recorder()
         self.event_recorder.ok('started')
-        self.state_watcher      = statemanager.StateChangeWatcher()
+        self.state_watcher = statemanager.StateChangeWatcher()
 
     def shutdown(self):
         self.state_watcher.shutdown()
@@ -50,9 +52,11 @@ class MasterControlProgram(object):
         self.event_recorder.ok("reconfigured")
         try:
             self._load_config(reconfigure=True)
-        except Exception:
+        except Exception as e:
             self.event_recorder.critical("reconfigure_failure")
-            log.exception("reconfigure failure")
+            log.exception(
+                "reconfigure failure: %s: %s" % (e.__class__.__name__, e)
+            )
             raise
 
     def _load_config(self, reconfigure=False):
@@ -65,31 +69,40 @@ class MasterControlProgram(object):
         In this case jobs shouldn't be scheduled until the state is applied.
         """
         self._load_config()
-        self.restore_state()
+        self.restore_state(
+            actioncommand.create_action_runner_factory_from_config(
+                self.config.load().get_master().action_runner
+            )
+        )
         # Any job with existing state would have been scheduled already. Jobs
         # without any state will be scheduled here.
-        self.jobs.schedule()
+        self.jobs.run_queue_schedule()
 
     def apply_config(self, config_container, reconfigure=False):
         """Apply a configuration."""
         master_config_directives = [
-            (self.update_state_watcher_config,           'state_persistence'),
-            (self.set_context_base,                      'command_context'),
-            (node.NodePoolRepository.update_from_config, 'nodes',
-                                                         'node_pools',
-                                                         'ssh_options'),
-            (self.apply_notification_options,            'notification_options'),
+            (self.update_state_watcher_config, 'state_persistence'),
+            (self.set_context_base, 'command_context'),
+            (
+                node.NodePoolRepository.update_from_config,
+                'nodes',
+                'node_pools',
+                'ssh_options',
+            ),
+            (self.apply_notification_options, 'notification_options'),
         ]
         master_config = config_container.get_master()
         apply_master_configuration(master_config_directives, master_config)
 
         # TODO: unify NOTIFY_STATE_CHANGE and simplify this
         factory = self.build_job_scheduler_factory(master_config)
-        self.apply_collection_config(config_container.get_jobs(),
-            self.jobs, job.Job.NOTIFY_STATE_CHANGE, factory, reconfigure)
-
-        self.apply_collection_config(config_container.get_services(),
-            self.services, service.Service.NOTIFY_STATE_CHANGE, self.context)
+        self.apply_collection_config(
+            config_container.get_jobs(),
+            self.jobs,
+            job.Job.NOTIFY_STATE_CHANGE,
+            factory,
+            reconfigure,
+        )
 
     def apply_collection_config(self, config, collection, notify_type, *args):
         items = collection.load_from_config(config, *args)
@@ -98,12 +111,14 @@ class MasterControlProgram(object):
     def build_job_scheduler_factory(self, master_config):
         output_stream_dir = master_config.output_stream_dir or self.working_dir
         action_runner = actioncommand.create_action_runner_factory_from_config(
-            master_config.action_runner)
+            master_config.action_runner,
+        )
         return job.JobSchedulerFactory(
             self.context,
             output_stream_dir,
             master_config.time_zone,
-            action_runner)
+            action_runner,
+        )
 
     def update_state_watcher_config(self, state_config):
         """Update the StateChangeWatcher, and save all state if the state config
@@ -112,8 +127,6 @@ class MasterControlProgram(object):
         if self.state_watcher.update_from_config(state_config):
             for job_scheduler in self.jobs:
                 self.state_watcher.save_job(job_scheduler.get_job())
-            for service in self.services:
-                self.state_watcher.save_service(service)
 
     def apply_notification_options(self, conf):
         if not conf:
@@ -132,23 +145,18 @@ class MasterControlProgram(object):
     def get_job_collection(self):
         return self.jobs
 
-    def get_service_collection(self):
-        return self.services
-
     def get_config_manager(self):
         return self.config
 
-    def restore_state(self):
+    def restore_state(self, action_runner):
         """Use the state manager to retrieve to persisted state and apply it
-        to the configured Jobs and Services.
+        to the configured Jobs.
         """
         self.event_recorder.notice('restoring')
-        job_states, service_states = self.state_watcher.restore(
-                self.jobs.get_names(), self.services.get_names())
+        job_states = self.state_watcher.restore(self.jobs.get_names())
 
-        self.jobs.restore_state(job_states)
-        self.services.restore_state(service_states)
+        self.jobs.restore_state(job_states, action_runner)
         self.state_watcher.save_metadata()
 
     def __str__(self):
-        return "MCP"
\ No newline at end of file
+        return "MCP"
