@@ -18,16 +18,19 @@ Tron schedulers
  jobs scheduled with the start_time of the previous run (False), or the
  end time of the previous run (False).
 """
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
+import datetime
 import logging
 import random
-import datetime
 
-from pytz import AmbiguousTimeError, NonExistentTimeError
+from pytz import AmbiguousTimeError
+from pytz import NonExistentTimeError
+
 from tron.config import schedule_parse
-
-from tron.utils import trontimespec
 from tron.utils import timeutils
-
+from tron.utils import trontimespec
 
 log = logging.getLogger(__name__)
 
@@ -38,7 +41,11 @@ def scheduler_from_config(config, time_zone):
         return ConstantScheduler()
 
     if isinstance(config, schedule_parse.ConfigIntervalScheduler):
-        return IntervalScheduler(config.timedelta, config.jitter)
+        return IntervalScheduler(
+            interval=config.timedelta,
+            jitter=config.jitter,
+            time_zone=time_zone,
+        )
 
     if isinstance(config, schedule_parse.ConfigGrocScheduler):
         return GeneralScheduler(
@@ -50,10 +57,12 @@ def scheduler_from_config(config, time_zone):
             weekdays=config.weekdays,
             name='groc',
             original=config.original,
-            jitter=config.jitter)
+            jitter=config.jitter,
+        )
 
     if isinstance(config, schedule_parse.ConfigCronScheduler):
         return GeneralScheduler(
+            time_zone=time_zone,
             minutes=config.minutes,
             hours=config.hours,
             monthdays=config.monthdays,
@@ -63,17 +72,20 @@ def scheduler_from_config(config, time_zone):
             seconds=[0],
             name='cron',
             original=config.original,
-            jitter=config.jitter)
+            jitter=config.jitter,
+        )
 
     if isinstance(config, schedule_parse.ConfigDailyScheduler):
         return GeneralScheduler(
             hours=[config.hour],
+            time_zone=time_zone,
             minutes=[config.minute],
             seconds=[config.second],
             weekdays=config.days,
             name='daily',
             original=config.original,
-            jitter=config.jitter)
+            jitter=config.jitter,
+        )
 
 
 class ConstantScheduler(object):
@@ -120,19 +132,21 @@ class GeneralScheduler(object):
     """
     schedule_on_complete = False
 
-    def __init__(self,
-            ordinals=None,
-            weekdays=None,
-            months=None,
-            monthdays=None,
-            timestr=None,
-            minutes=None,
-            hours=None,
-            seconds=None,
-            time_zone=None,
-            name=None,
-            original=None,
-            jitter=None):
+    def __init__(
+        self,
+        ordinals=None,
+        weekdays=None,
+        months=None,
+        monthdays=None,
+        timestr=None,
+        minutes=None,
+        hours=None,
+        seconds=None,
+        time_zone=None,
+        name=None,
+        original=None,
+        jitter=None,
+    ):
         """Parameters:
           timestr     - the time of day to run, as 'HH:MM'
           ordinals    - first, second, third &c, as a set of integers in 1..5 to
@@ -146,11 +160,11 @@ class GeneralScheduler(object):
                         Defaults to UTC - valid entries are things like
                         Australia/Victoria or PST8PDT.
         """
-        self.time_zone      = time_zone
-        self.jitter         = jitter
-        self.name           = name or 'daily'
-        self.original       = original or ''
-        self.time_spec      = trontimespec.TimeSpecification(
+        self.time_zone = time_zone
+        self.jitter = jitter
+        self.name = name or 'daily'
+        self.original = original or ''
+        self.time_spec = trontimespec.TimeSpecification(
             ordinals=ordinals,
             weekdays=weekdays,
             months=months,
@@ -159,32 +173,53 @@ class GeneralScheduler(object):
             hours=hours,
             minutes=minutes,
             seconds=seconds,
-            timezone=time_zone.zone if time_zone else None)
+            timezone=time_zone.zone if time_zone else None,
+        )
 
     def next_run_time(self, start_time):
         """Find the next time to run."""
         if not start_time:
-            start_time = timeutils.current_time()
+            start_time = timeutils.current_time(tz=self.time_zone)
         elif self.time_zone:
-            try:
-                start_time = self.time_zone.localize(start_time, is_dst=None)
-            except AmbiguousTimeError:
-                # We are in the infamous 1 AM block which happens twice on
-                # fall-back. Pretend like it's the first time, every time.
-                start_time = self.time_zone.localize(start_time, is_dst=True)
-            except NonExistentTimeError:
-                # We are in the infamous 2:xx AM block which does not
-                # exist. Pretend like it's the later time, every time.
-                start_time = self.time_zone.localize(start_time, is_dst=True)
+            if start_time.tzinfo is None or start_time.tzinfo.utcoffset(
+                start_time,
+            ) is None:
+                # tz-naive start times need to be localized first to the requested
+                # time zone.
+                try:
+                    start_time = self.time_zone.localize(
+                        start_time,
+                        is_dst=None,
+                    )
+                except AmbiguousTimeError:
+                    # We are in the infamous 1 AM block which happens twice on
+                    # fall-back. Pretend like it's the first time, every time.
+                    start_time = self.time_zone.localize(
+                        start_time,
+                        is_dst=True,
+                    )
+                except NonExistentTimeError:
+                    # We are in the infamous 2:xx AM block which does not
+                    # exist. Pretend like it's the later time, every time.
+                    start_time = self.time_zone.localize(
+                        start_time,
+                        is_dst=True,
+                    )
 
         return self.time_spec.get_match(start_time) + get_jitter(self.jitter)
 
     def __str__(self):
         return '%s %s%s' % (
-            self.name, self.original, get_jitter_str(self.jitter))
+            self.name,
+            self.original,
+            get_jitter_str(self.jitter),
+        )
 
     def __eq__(self, other):
-        return hasattr(other, 'time_spec') and self.time_spec == other.time_spec
+        return hasattr(
+            other,
+            'time_spec',
+        ) and self.time_spec == other.time_spec
 
     def __ne__(self, other):
         return not self == other
@@ -205,21 +240,27 @@ class IntervalScheduler(object):
     """
     schedule_on_complete = False
 
-    def __init__(self, interval, jitter):
+    def __init__(self, interval, jitter, time_zone):
         self.interval = interval
         self.jitter = jitter
+        self.time_zone = time_zone
 
-    def next_run_time(self, last_run_time):
-        last_run_time = last_run_time or timeutils.current_time()
+    def next_run_time(self, last_run_time, time_zone=None):
+        last_run_time = last_run_time or timeutils.current_time(tz=time_zone)
         return last_run_time + self.interval + get_jitter(self.jitter)
 
     def __str__(self):
         return "%s %s%s" % (
-            self.get_name(), self.interval, get_jitter_str(self.jitter))
+            self.get_name(),
+            self.interval,
+            get_jitter_str(self.jitter),
+        )
 
     def __eq__(self, other):
-        return (isinstance(other, IntervalScheduler) and
-                self.interval == other.interval)
+        return (
+            isinstance(other, IntervalScheduler)
+            and self.interval == other.interval
+        )
 
     def __ne__(self, other):
         return not self == other
