@@ -1,29 +1,30 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
+import operator
 from collections import namedtuple
 from contextlib import contextmanager
-import itertools
-import operator
 
-import yaml
-sqlalchemy = None # pyflakes
+from six.moves import filter
 
-from tron.serialize import runstate
+from tron import yaml
 from tron.config.config_utils import MAX_IDENTIFIER_LENGTH
-
+from tron.serialize import runstate
+sqlalchemy = None  # pyflakes
 
 SQLStateKey = namedtuple('SQLStateKey', ['table', 'id'])
 
 
 class SQLAlchemyStateStore(object):
-
     def __init__(self, name, connection_details):
         import sqlalchemy
         global sqlalchemy
-        assert sqlalchemy # pyflakes
+        assert sqlalchemy  # pyflakes
 
-        self.name               = name
-        self._connection        = None
-        self.encoder            = yaml.dump
-        self.decoder            = yaml.load
+        self.name = name
+        self._connection = None
+        self.encoder = yaml.dump
+        self.decoder = yaml.load
         self._create_engine(connection_details)
         self._build_tables()
         self.create_tables()
@@ -36,19 +37,26 @@ class SQLAlchemyStateStore(object):
         """Build table objects."""
         from sqlalchemy import Table, Column, String, Text
         self._metadata = sqlalchemy.MetaData()
-        self.job_table = Table('job_state_data', self._metadata,
-            Column('id', String(MAX_IDENTIFIER_LENGTH), primary_key=True),
-            Column('state_data', Text)
+        self.job_table = Table(
+            'job_state_data',
+            self._metadata,
+            Column(
+                'id',
+                String(MAX_IDENTIFIER_LENGTH, ),
+                primary_key=True,
+            ),
+            Column('state_data', Text),
         )
 
-        self.service_table = Table('service_state_data', self._metadata,
-            Column('id', String(MAX_IDENTIFIER_LENGTH), primary_key=True),
-            Column('state_data', Text)
-        )
-
-        self.metadata_table = Table('metadata_table', self._metadata,
-            Column('id', String(MAX_IDENTIFIER_LENGTH), primary_key=True),
-            Column('state_data', Text)
+        self.metadata_table = Table(
+            'metadata_table',
+            self._metadata,
+            Column(
+                'id',
+                String(MAX_IDENTIFIER_LENGTH, ),
+                primary_key=True,
+            ),
+            Column('state_data', Text),
         )
 
     def create_tables(self):
@@ -67,8 +75,6 @@ class SQLAlchemyStateStore(object):
         table = None
         if type == runstate.JOB_STATE:
             table = self.job_table
-        if type == runstate.SERVICE_STATE:
-            table = self.service_table
         if type == runstate.MCP_STATE:
             table = self.metadata_table
         return SQLStateKey(table, iden)
@@ -84,8 +90,8 @@ class SQLAlchemyStateStore(object):
 
     def _update(self, conn, key, data):
         """Attempt to update the state_data."""
-        update  = key.table.update()
-        where   = key.table.c.id==key.id
+        update = key.table.update()
+        where = key.table.c.id == key.id
         results = conn.execute(update.where(where).values(state_data=data))
         return results.rowcount
 
@@ -97,11 +103,11 @@ class SQLAlchemyStateStore(object):
     def restore(self, keys):
         with self.connect() as conn:
             items = [(key, self._select(conn, key)) for key in keys]
-            return dict(itertools.ifilter(operator.itemgetter(1), items))
+            return dict(filter(operator.itemgetter(1), items))
 
     def _select(self, conn, key):
         cols = [key.table.c.state_data]
-        select = sqlalchemy.sql.select(cols, key.table.c.id==key.id)
+        select = sqlalchemy.sql.select(cols, key.table.c.id == key.id)
         result = conn.execute(select).fetchone()
         return self.decoder(result[0]) if result else None
 
