@@ -1,12 +1,16 @@
-from contextlib import contextmanager
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
+import itertools
 import logging
 import time
-import itertools
-import tron
+from contextlib import contextmanager
+
+import six
+
 from tron.config import schema
-from tron.core import job, service
+from tron.core import job
 from tron.serialize import runstate
-from tron.serialize.runstate.mongostore import MongoStateStore
 from tron.serialize.runstate.shelvestore import ShelveStateStore
 from tron.serialize.runstate.sqlalchemystore import SQLAlchemyStateStore
 from tron.serialize.runstate.yamlstore import YamlStateStore
@@ -18,6 +22,7 @@ log = logging.getLogger(__name__)
 class VersionMismatchError(ValueError):
     """Raised when the state has a newer version then tron.__version.__."""
 
+
 class PersistenceStoreError(ValueError):
     """Raised if the store can not be created or fails a read or write."""
 
@@ -27,11 +32,11 @@ class PersistenceManagerFactory(object):
 
     @classmethod
     def from_config(cls, persistence_config):
-        store_type              = persistence_config.store_type
-        name                    = persistence_config.name
-        connection_details      = persistence_config.connection_details
-        buffer_size             = persistence_config.buffer_size
-        store                   = None
+        store_type = persistence_config.store_type
+        name = persistence_config.name
+        connection_details = persistence_config.connection_details
+        buffer_size = persistence_config.buffer_size
+        store = None
 
         if store_type not in schema.StatePersistenceTypes:
             raise PersistenceStoreError("Unknown store type: %s" % store_type)
@@ -42,9 +47,6 @@ class PersistenceManagerFactory(object):
         if store_type == schema.StatePersistenceTypes.sql:
             store = SQLAlchemyStateStore(name, connection_details)
 
-        if store_type == schema.StatePersistenceTypes.mongo:
-            store = MongoStateStore(name, connection_details)
-
         if store_type == schema.StatePersistenceTypes.yaml:
             store = YamlStateStore(name)
 
@@ -56,30 +58,34 @@ class StateMetadata(object):
     """A data object for saving state metadata. Conforms to the same
     RunState interface as Jobs and Services.
     """
-    name                        = 'StateMetadata'
-    version                     = tron.__version_info__
+    name = 'StateMetadata'
+
+    # State schema version, only first component counts,
+    # for backwards compatibility
+    version = (0, 7, 0, 0)
 
     def __init__(self):
-        self.state_data         = {
-            'version':              self.version,
-            'create_time':          time.time(),
+        self.state_data = {
+            'version': self.version,
+            'create_time': time.time(),
         }
 
     @classmethod
     def validate_metadata(cls, metadata):
         """Raises an exception if the metadata version is newer then
-        tron.__version__.
+        StateMetadata.version
         """
         if not metadata:
             return
 
-        version = metadata['version']
-        # Names (and state keys) changed in 0.5.2, requires migration
-        # see tools/migration/migrate_state_to_namespace
-        if version > cls.version or version < (0, 5, 2):
-            msg = "State for version %s, expected %s"
+        if metadata['version'][0] > cls.version[0]:
+            msg = "State version %s, expected <= %s"
             raise VersionMismatchError(
-                msg % (metadata['version'] , cls.version))
+                msg % (
+                    metadata['version'],
+                    cls.version,
+                ),
+            )
 
 
 class StateSaveBuffer(object):
@@ -88,20 +94,20 @@ class StateSaveBuffer(object):
     """
 
     def __init__(self, buffer_size):
-        self.buffer_size        = buffer_size
-        self.buffer             = {}
-        self.counter            = itertools.cycle(xrange(buffer_size))
+        self.buffer_size = buffer_size
+        self.buffer = {}
+        self.counter = itertools.cycle(range(buffer_size))
 
     def save(self, key, state_data):
         """Save the state_data indexed by key and return True if the buffer
         is full.
         """
         self.buffer[key] = state_data
-        return not self.counter.next()
+        return not next(self.counter)
 
     def __iter__(self):
         """Return all buffered data and clear the buffer."""
-        for key, item in self.buffer.iteritems():
+        for key, item in six.iteritems(self.buffer):
             yield key, item
         self.buffer.clear()
 
@@ -129,20 +135,21 @@ class PersistentStateManager(object):
     """
 
     def __init__(self, persistence_impl, buffer):
-        self.enabled            = True
-        self._buffer            = buffer
-        self._impl              = persistence_impl
-        self.metadata_key       = self._impl.build_key(
-                                    runstate.MCP_STATE, StateMetadata.name)
+        self.enabled = True
+        self._buffer = buffer
+        self._impl = persistence_impl
+        self.metadata_key = self._impl.build_key(
+            runstate.MCP_STATE,
+            StateMetadata.name,
+        )
 
-    def restore(self, job_names, service_names, skip_validation=False):
+    def restore(self, job_names, skip_validation=False):
         """Return the most recent serialized state."""
         log.debug("Restoring state.")
         if not skip_validation:
             self._restore_metadata()
 
-        return (self._restore_dicts(runstate.JOB_STATE, job_names),
-                self._restore_dicts(runstate.SERVICE_STATE, service_names))
+        return self._restore_dicts(runstate.JOB_STATE, job_names)
 
     def _restore_metadata(self):
         metadata = self._impl.restore([self.metadata_key])
@@ -151,19 +158,21 @@ class PersistentStateManager(object):
     def _keys_for_items(self, item_type, names):
         """Returns a dict of item to the key for that item."""
         keys = (self._impl.build_key(item_type, name) for name in names)
-        return dict(itertools.izip(keys, names))
+        return dict(zip(keys, names))
 
     def _restore_dicts(self, item_type, items):
         """Return a dict mapping of the items name to its state data."""
-        key_to_item_map  = self._keys_for_items(item_type, items)
+        key_to_item_map = self._keys_for_items(item_type, items)
         key_to_state_map = self._impl.restore(key_to_item_map.keys())
-        return dict((key_to_item_map[key], state_data)
-                    for key, state_data in key_to_state_map.iteritems())
+        return {
+            key_to_item_map[key]: state_data
+            for key, state_data in six.iteritems(key_to_state_map)
+        }
 
     def save(self, type_enum, name, state_data):
         """Persist an items state."""
         key = self._impl.build_key(type_enum, name)
-        log.info("Buffering state save for: %s", key)
+        log.debug("Buffering state save for: %s", key)
         if self._buffer.save(key, state_data) and self.enabled:
             self._save_from_buffer()
 
@@ -178,9 +187,9 @@ class PersistentStateManager(object):
         with self._timeit():
             try:
                 self._impl.save(key_state_pairs)
-            except Exception, e:
+            except Exception as e:
                 msg = "Failed to save state for %s: %s" % (keys, e)
-                log.warn(msg)
+                log.warning(msg)
                 raise PersistenceStoreError(msg)
 
     def cleanup(self):
@@ -228,14 +237,16 @@ class StateChangeWatcher(observer.Observer):
 
     def __init__(self):
         self.state_manager = NullStateManager
-        self.config        = None
+        self.config = None
 
     def update_from_config(self, state_config):
         if self.config == state_config:
             return False
 
         self.shutdown()
-        self.state_manager = PersistenceManagerFactory.from_config(state_config)
+        self.state_manager = PersistenceManagerFactory.from_config(
+            state_config,
+        )
         self.config = state_config
         return True
 
@@ -243,15 +254,10 @@ class StateChangeWatcher(observer.Observer):
         """Handle a state change in an observable by saving its state."""
         if isinstance(observable, job.Job):
             self.save_job(observable)
-        if isinstance(observable, service.Service):
-            self.save_service(observable)
 
     def save_job(self, job):
         self._save_object(runstate.JOB_STATE, job)
 
-    def save_service(self, service):
-        self._save_object(runstate.SERVICE_STATE, service)
-
     def save_metadata(self):
         self._save_object(runstate.MCP_STATE, StateMetadata())
 
@@ -265,5 +271,5 @@ class StateChangeWatcher(observer.Observer):
     def disabled(self):
         return self.state_manager.disabled()
 
-    def restore(self, jobs, services):
-        return self.state_manager.restore(jobs, services)
+    def restore(self, jobs):
+        return self.state_manager.restore(jobs)
