@@ -1,21 +1,31 @@
 """
  Daemonize trond.
 """
+from __future__ import absolute_import
+from __future__ import print_function
+from __future__ import unicode_literals
+
 import logging
 import logging.config
 import os
-import pkg_resources
-import daemon
-
-from twisted.internet import pollreactor, defer
-from twisted.internet.main import installReactor
-
-import lockfile
-from tron.utils import flockfile
+import platform
 import signal
-from twisted.python import log as twisted_log
-import tron
 
+import daemon
+import lockfile
+import pkg_resources
+import six
+from twisted.internet import defer
+from twisted.internet.main import installReactor
+from twisted.python import log as twisted_log
+
+import tron
+from tron.utils import flockfile
+
+if platform.system() == 'Linux':
+    from twisted.internet.epollreactor import EPollReactor as Reactor
+else:
+    from twisted.internet.selectreactor import SelectReactor as Reactor
 
 log = logging.getLogger(__name__)
 
@@ -46,7 +56,13 @@ class PIDFile(object):
 
         if pid:
             self._try_unlock()
-            raise SystemExit("Daemon was running as %s. Remove PID file." % pid)
+            raise SystemExit(
+                "A tron pidfile is already present at %s using PID %s. The existing pidfile must be removed before starting another tron daemon."
+                % (
+                    self.filename,
+                    pid,
+                ),
+            )
 
     def is_process_running(self, pid):
         """Return True if the process is still running."""
@@ -59,21 +75,21 @@ class PIDFile(object):
             return False
 
     def __enter__(self):
-        print >>self.lock.file, os.getpid()
+        print(os.getpid(), file=self.lock.file)
         self.lock.file.flush()
 
     def _try_unlock(self):
         try:
             self.lock.release()
         except lockfile.NotLocked:
-            log.warn("Lockfile was already unlocked.")
+            log.warning("Lockfile was already unlocked.")
 
     def __exit__(self, *args):
         self._try_unlock()
         try:
             os.unlink(self.filename)
         except OSError:
-            log.warn("Failed to remove pidfile: %s" % self.filename)
+            log.warning("Failed to remove pidfile: %s" % self.filename)
 
 
 def setup_logging(options):
@@ -111,9 +127,9 @@ class NoDaemonContext(object):
     """A mock DaemonContext for running trond without being a daemon."""
 
     def __init__(self, **kwargs):
-        self.signal_map     = kwargs.pop('signal_map', {})
-        self.pidfile        = kwargs.pop('pidfile', None)
-        self.working_dir    = kwargs.pop('working_directory', '.')
+        self.signal_map = kwargs.pop('signal_map', {})
+        self.pidfile = kwargs.pop('pidfile', None)
+        self.working_dir = kwargs.pop('working_directory', '.')
         self.signal_map[signal.SIGUSR1] = self._handle_debug
 
     def _handle_debug(self, *args):
@@ -121,7 +137,7 @@ class NoDaemonContext(object):
         ipdb.set_trace()
 
     def __enter__(self):
-        for signum, handler in self.signal_map.iteritems():
+        for signum, handler in six.iteritems(self.signal_map):
             signal.signal(signum, handler)
 
         os.chdir(self.working_dir)
@@ -135,23 +151,24 @@ class NoDaemonContext(object):
     def terminate(self, *args):
         pass
 
+
 class TronDaemon(object):
     """Daemonize and run the tron daemon."""
 
     WAIT_SECONDS = 5
 
     def __init__(self, options):
-        self.options    = options
-        self.mcp        = None
-        nodaemon        = self.options.nodaemon
-        context_class   = NoDaemonContext if nodaemon else daemon.DaemonContext
-        self.context    = self._build_context(options, context_class)
-        self.reactor    = None
+        self.options = options
+        self.mcp = None
+        nodaemon = self.options.nodaemon
+        context_class = NoDaemonContext if nodaemon else daemon.DaemonContext
+        self.context = self._build_context(options, context_class)
+        self.reactor = Reactor()
 
     def _build_context(self, options, context_class):
         signal_map = {
-            signal.SIGHUP:  self._handle_reconfigure,
-            signal.SIGINT:  self._handle_graceful_shutdown,
+            signal.SIGHUP: self._handle_reconfigure,
+            signal.SIGINT: self._handle_graceful_shutdown,
             signal.SIGTERM: self._handle_shutdown,
         }
         pidfile = PIDFile(options.pid_file)
@@ -160,7 +177,7 @@ class TronDaemon(object):
             umask=0o022,
             pidfile=pidfile,
             signal_map=signal_map,
-            files_preserve=[pidfile.lock.file]
+            files_preserve=[pidfile.lock.file],
         )
 
     def run(self):
@@ -171,9 +188,7 @@ class TronDaemon(object):
             self._run_www_api()
             self._run_reactor()
 
-
     def setup_reactor(self):
-        self.reactor = pollreactor.PollReactor()
         installReactor(self.reactor)
 
     def _run_www_api(self):
@@ -186,16 +201,16 @@ class TronDaemon(object):
     def _run_mcp(self):
         # Local import required because of reactor import in mcp
         from tron import mcp
-        working_dir         = self.options.working_dir
-        config_path         = self.options.config_path
-        self.mcp            = mcp.MasterControlProgram(working_dir, config_path)
+        working_dir = self.options.working_dir
+        config_path = self.options.config_path
+        self.mcp = mcp.MasterControlProgram(working_dir, config_path)
 
         try:
             self.mcp.initial_setup()
-        except Exception, e:
+        except Exception as e:
             msg = "Error in configuration %s: %s"
             log.exception(msg % (config_path, e))
-            raise SystemExit("Failed to configure MCP")
+            raise
 
     def _run_reactor(self):
         """Run the twisted reactor."""
