@@ -1,6 +1,10 @@
 """Utilities for working with collections."""
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 import logging
 
+import six
 
 log = logging.getLogger(__name__)
 
@@ -36,11 +40,6 @@ class MappingCollection(dict):
         log.info("Removing %s %s", self.item_name, name)
         self.pop(name).disable()
 
-    def restore_state(self, state_data):
-        for name, state in state_data.iteritems():
-            self[name].restore_state(state)
-        log.info("Loaded state for %d %s", len(state_data), self.item_name)
-
     def contains_item(self, item, handle_update_func):
         if item == self.get(item.get_name()):
             return True
@@ -81,4 +80,4 @@ class Enum(object):
         raise AttributeError(name)
 
     def __iter__(self):
-        return iter(self.values)
\ No newline at end of file
+        return iter(self.values)
