@@ -1,18 +1,20 @@
 """Parse a crontab entry and return a dictionary."""
+from __future__ import absolute_import
+from __future__ import unicode_literals
 
 import calendar
 import itertools
 import re
 
-
 PREDEFINED_SCHEDULE = {
-    "@yearly":  "0 0 1 1 *",
+    "@yearly": "0 0 1 1 *",
     "@anually": "0 0 1 1 *",
     "@monthly": "0 0 1 * *",
-    "@weekly":  "0 0 * * 0",
-    "@daily":   "0 0 * * *",
-    "@midnight":"0 0 * * *",
-    "@hourly":  "0 * * * *"}
+    "@weekly": "0 0 * * 0",
+    "@daily": "0 0 * * *",
+    "@midnight": "0 0 * * *",
+    "@hourly": "0 * * * *",
+}
 
 
 def convert_predefined(line):
@@ -27,13 +29,16 @@ def convert_predefined(line):
 class FieldParser(object):
     """Parse and validate a field in a crontab entry."""
 
-    name   = None
+    name = None
     bounds = None
-    range_pattern = re.compile(r'''
+    range_pattern = re.compile(
+        r'''
         (?P<min>\d+|\*)         # Initial value
         (?:-(?P<max>\d+))?      # Optional max upper bound
         (?:/(?P<step>\d+))?     # Optional step increment
-        ''', re.VERBOSE)
+        ''',
+        re.VERBOSE,
+    )
 
     def normalize(self, source):
         return source.strip()
@@ -45,8 +50,16 @@ class FieldParser(object):
         if source == '*':
             return None
 
-        groups  = [self.get_values(group) for group in self.get_groups(source)]
-        return sorted(set(itertools.chain.from_iterable(groups)))
+        groups = [self.get_values(group) for group in self.get_groups(source)]
+        groups = set(itertools.chain.from_iterable(groups))
+        has_last = False
+        if 'LAST' in groups:
+            has_last = True
+            groups.remove('LAST')
+        groups = sorted(groups)
+        if has_last:
+            groups.append('LAST')
+        return groups
 
     def get_match_groups(self, source):
         match = self.range_pattern.match(source)
@@ -55,9 +68,9 @@ class FieldParser(object):
         return match.groupdict()
 
     def get_values(self, source):
-        source               = self.normalize(source)
-        match_groups         = self.get_match_groups(source)
-        step                 = 1
+        source = self.normalize(source)
+        match_groups = self.get_match_groups(source)
+        step = 1
         min_value, max_value = self.get_value_range(match_groups)
 
         if match_groups['step']:
@@ -78,11 +91,13 @@ class FieldParser(object):
 
     def get_range(self, min_value, max_value, step):
         if min_value < max_value:
-            return range(min_value, max_value, step)
+            return list(range(min_value, max_value, step))
 
         min_bound, max_bound = self.bounds
         diff = (max_bound - min_value) + (max_value - min_bound)
-        return [(min_value + i) % max_bound for i in xrange(0, diff, step)]
+        return [
+            (min_value + i) % max_bound for i in list(range(0, diff, step))
+        ]
 
     def validate_bounds(self, value):
         min_value, max_value = self.bounds
@@ -93,16 +108,18 @@ class FieldParser(object):
 
 
 class MinuteFieldParser(FieldParser):
-    name    = 'minutes'
-    bounds  = (0, 60)
+    name = 'minutes'
+    bounds = (0, 60)
+
 
 class HourFieldParser(FieldParser):
-    name    = 'hours'
-    bounds  = (0, 24)
+    name = 'hours'
+    bounds = (0, 24)
+
 
 class MonthdayFieldParser(FieldParser):
-    name    = 'monthdays'
-    bounds  = (1, 32)
+    name = 'monthdays'
+    bounds = (1, 32)
 
     def get_values(self, source):
         # Handle special case for last day of month
@@ -112,22 +129,24 @@ class MonthdayFieldParser(FieldParser):
 
         return super(MonthdayFieldParser, self).get_values(source)
 
+
 class MonthFieldParser(FieldParser):
-    name        = 'months'
-    bounds      = (1, 13)
+    name = 'months'
+    bounds = (1, 13)
     month_names = calendar.month_abbr[1:]
 
     def normalize(self, month):
         month = super(MonthFieldParser, self).normalize(month)
         month = month.lower()
-        for month_num, month_name in  enumerate(self.month_names, start=1):
+        for month_num, month_name in enumerate(self.month_names, start=1):
             month = month.replace(month_name.lower(), str(month_num))
         return month
 
+
 class WeekdayFieldParser(FieldParser):
-    name        = 'weekdays'
-    bounds      = (0, 7)
-    day_names   = ['sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat']
+    name = 'weekdays'
+    bounds = (0, 7)
+    day_names = ['sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat']
 
     def normalize(self, day_of_week):
         day_of_week = super(WeekdayFieldParser, self).normalize(day_of_week)
@@ -137,11 +156,11 @@ class WeekdayFieldParser(FieldParser):
         return day_of_week.replace('7', '0').replace('?', '*')
 
 
-minute_parser   = MinuteFieldParser()
-hour_parser     = HourFieldParser()
+minute_parser = MinuteFieldParser()
+hour_parser = HourFieldParser()
 monthday_parser = MonthdayFieldParser()
-month_parser    = MonthdayFieldParser()
-weekday_parser  = WeekdayFieldParser()
+month_parser = MonthdayFieldParser()
+weekday_parser = WeekdayFieldParser()
 
 
 # TODO: support L (for dow), W, #
@@ -150,9 +169,10 @@ def parse_crontab(line):
     minutes, hours, dom, months, dow = line.split(None, 4)
 
     return {
-        'minutes':      minute_parser.parse(minutes),
-        'hours':        hour_parser.parse(hours),
-        'monthdays':    monthday_parser.parse(dom),
-        'months':       month_parser.parse(months),
-        'weekdays':     weekday_parser.parse(dow),
-        'ordinals':     None}
+        'minutes': minute_parser.parse(minutes),
+        'hours': hour_parser.parse(hours),
+        'monthdays': monthday_parser.parse(dom),
+        'months': month_parser.parse(months),
+        'weekdays': weekday_parser.parse(dow),
+        'ordinals': None,
+    }
