@@ -1,4 +1,6 @@
 """Iteration utility functions."""
+from __future__ import absolute_import
+from __future__ import unicode_literals
 
 
 def build_filtered_func(func):
@@ -6,12 +8,14 @@ def build_filtered_func(func):
         """Filter out Nones and return the return of func."""
         if not seq:
             return None
-        seq = filter(None, seq)
-        if not seq:
+        seq = list(filter(None, seq))
+        if len(seq) == 0:
             return None
         return func(seq)
+
     return filtered_func
 
+
 min_filter = build_filtered_func(min)
 max_filter = build_filtered_func(max)
 
@@ -21,4 +25,4 @@ def list_all(seq):
     all(). This differs from the built-in all() which will short circuit
     on the first False.
     """
-    return all(list(seq))
\ No newline at end of file
+    return all(list(seq))
