@@ -1,8 +1,14 @@
 """Implements the Observer/Observable pattern,"""
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 import logging
 
+from six import string_types
+
 log = logging.getLogger(__name__)
 
+
 class Observable(object):
     """An Observable in the Observer/Observable pattern. It stores
     specifications and Observers which can be notified of changes by calling
@@ -20,7 +26,7 @@ class Observable(object):
             <string>                Matches only that event
             <sequence of strings>   Matches any of the events in the sequence
         """
-        if isinstance(watch_spec, (basestring, bool)):
+        if isinstance(watch_spec, (string_types, bool)):
             self._observers.setdefault(watch_spec, []).append(observer)
             return
 
@@ -73,4 +79,4 @@ class Observer(object):
         pass
 
     def stop_watching(self, observable):
-        observable.remove_observer(self)
\ No newline at end of file
+        observable.remove_observer(self)
