@@ -1,4 +1,11 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 import logging
+
+import six
+
+from tron.utils import maybe_decode
 from tron.utils.observer import Observable
 
 
@@ -37,6 +44,9 @@ class NamedEventState(dict):
     def __hash__(self):
         return hash(self.name)
 
+    def __bool__(self):
+        return bool(self.name)
+
     def __nonzero__(self):
         return bool(self.name)
 
@@ -57,9 +67,11 @@ class NamedEventState(dict):
 
 
 def traverse(starting_state, match_func):
-    visited                 = set()
-    state_pairs             = [(None, starting_state)]
-    pair_with_name          = lambda p: (p[0], p[1].name)
+    visited = set()
+    state_pairs = [(None, starting_state)]
+
+    def pair_with_name(p):
+        return (p[0], p[1].name)
 
     while state_pairs:
         transition_state_pair = state_pairs.pop()
@@ -69,22 +81,28 @@ def traverse(starting_state, match_func):
         if match_func(*transition_state_pair):
             yield transition_state_pair
 
-        for next_pair in cur_state.iteritems():
+        for next_pair in six.iteritems(cur_state):
             if pair_with_name(next_pair) not in visited:
                 state_pairs.append(next_pair)
 
 
 def named_event_by_name(starting_state, name):
-    name_match = lambda t, s: s.name == name
+    name = maybe_decode(name)
+
+    def name_match(t, s):
+        return s.name == name
+
     try:
-        _, state = traverse(starting_state, name_match).next()
+        _, state = next(traverse(starting_state, name_match))
         return state
     except StopIteration:
         raise ValueError("State %s not found." % name)
 
 
 def get_transitions(starting_state):
-    transition_match = lambda t, s: bool(t)
+    def transition_match(t, s):
+        return bool(t)
+
     return [trans for trans, _ in traverse(starting_state, transition_match)]
 
 
@@ -105,6 +123,11 @@ class StateMachine(Observable):
         self.state = force_state or self.initial_state
         self._state_by_name = None
         self.delegate = delegate
+        self.transitions = get_transitions(self.initial_state)
+
+    def reset(self):
+        """Force state machine into initial state."""
+        self.state = self.initial_state
 
     def check(self, target):
         """Check if the state can be transitioned to target. Returns the
@@ -114,10 +137,6 @@ class StateMachine(Observable):
         log.debug("Checking for transition from %s to %s", self.state, target)
         return self.state.get(target, None)
 
-    @property
-    def transitions(self):
-        return get_transitions(self.initial_state)
-
     def transition(self, target, stop_item=None):
         """Check our current state for a transition based on the input 'target'
 
