@@ -1,6 +1,10 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 import contextlib
 import os
 
+
 @contextlib.contextmanager
 def working_dir(path):
     """Change the working directory and revert back to previous working
